package biga;
import nme.Lib;
import nme.Vector;
import biga.utils.GeomUtils;
import nme.display.Shape;
import nme.display.Stage;
import nme.display.Sprite;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.geom.Point;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class CrossHair extends Shape 
{
	static private var crossHairs:Vector<CrossHair> = new Vector<CrossHair>();
	static public var background:Sprite = new Sprite();
	static private var instance:CrossHair;
	static private var closest:CrossHair;
	static private var friction:Float = .2;
	private var _fixed:Bool;
	private var _point:Point;
	public function new( color:Int = 0, size:Float = 24 ) 
	{
		super();
		
		point = new Point();
		graphics.lineStyle( 0, color );
		graphics.moveTo( -size / 2, 0 );
		graphics.lineTo( size / 2, 0 );
		graphics.moveTo( 0, -size / 2 );
		graphics.lineTo( 0, size / 2 );
		graphics.drawCircle( 0, 0, size / 2 );
		crossHairs.push( this );
		addEventListener( Event.ADDED_TO_STAGE, addedToStage );
		
		
		if ( instance == null )
		{
			instance = this;
			instance.addEventListener( Event.ENTER_FRAME, update );
			
			background.graphics.beginFill( 0, 0 );
			background.graphics.drawRect( 0, 0, Lib.stage.stageWidth, Lib.stage.stageHeight );
		}
		
	}
	private function update(e:Event):Void 
	{
		if ( closest != null )
		{
			closest.x += ( stage.mouseX - closest.x ) * friction;
			closest.y += ( stage.mouseY - closest.y ) * friction;
		}
	}
	private function addedToStage(e:Event):Void 
	{
		
		removeEventListener(Event.ADDED_TO_STAGE, addedToStage);
		stage.addEventListener( MouseEvent.MOUSE_UP, onMouseHandler );
		stage.addEventListener( MouseEvent.MOUSE_DOWN, onMouseHandler );
		
	}
	private function onMouseHandler(e:MouseEvent):Void 
	{
		switch( e.type) 
		{
			case MouseEvent.MOUSE_DOWN:
				
				#if js
					if ( e.target != background ) return;
				#else
					if ( e.target != stage ) return;
				#end
				
				var mouse:Point = new Point( stage.mouseX, stage.mouseY );
				var dist:Float;
				var minDist:Float = Math.POSITIVE_INFINITY;
				for( c in crossHairs )
				{
					if ( c.fixed ) continue;
					dist = GeomUtils.distance( mouse, c.point );
					if ( dist < minDist ) 
					{
						closest = c;
						minDist = dist;
					}
				}
			return;
			case MouseEvent.MOUSE_UP: 
				closest = null; 
			return;
		}
	}
	public var point( get_point, set_point ):Point;
	public function get_point():Point 
	{
		_point.x = x;
		_point.y = y;
		return _point; 
	}
	public function set_point( p:Point ):Point
	{ 
		if ( _point == null ) _point = new Point(0, 0);
		x = _point.x = p.x;
		y = _point.y = p.y;
		return point;
	}
	public var fixed( get_fixed, set_fixed ):Bool;
	public function get_fixed():Bool { return _fixed; }
	public function set_fixed(value:Bool):Bool 
	{
		_fixed = value;
		return fixed;
	}
}