﻿package biga.distribution;
import nme.geom.Point;
import biga.utils.GeomUtils;
import nme.geom.Rectangle;
import nme.Vector;
using nme.Vector;
/**
 * https://malenkov.dev.java.net/20090226/hexagon.java
 * @author Nicolas Barradeau
 */
class Hexagonal extends Distribution
{
	
	private var _rect:Rectangle;
	private var _radius:Float;
	
	public function new( rect:Rectangle, radius:Float ) 
	{
		super();
		this.rect = rect;
		this.radius = radius;
	}
	
	public function distribute( addCenter:Bool = false, shuffle:Bool = false, jitter:Float = 0 ):Vector<Point>
	{
		
		var apothem:Float = ( Math.sqrt( 3 ) * .5 ) * radius;
		
		var angle:Float = 0;
		var offsetX:Float = 0;
		var offsetY:Float = 0;
		
		var columns:Int = Std.int( rect.width / ( apothem * 1.5 ) );
		var rows:Int = Std.int( rect.height / ( apothem * 1.5 ) );
		
		var points:Vector<Point> = new Vector<Point>();
		
		for ( i in 0...columns )
		{
			
			for ( j in 0...rows )
			{
				
				offsetX = rect.x + i * radius * 1.5;
				
				offsetY = rect.y + ( (j  * 2 + i % 2 ) * apothem );
				
				if ( addCenter )	points.push( new Point( offsetX, offsetY ) );
				
				for ( k in 0...6 )
				{
					angle = k * Math.PI / 3;
					var p:Point = new Point(	offsetX + Math.cos( angle ) * radius,
												offsetY + Math.sin( angle ) * radius	);
					
					if ( rect.containsPoint( p ) )	points.push( p );
					
				}
			}
		}
		if ( shuffle ) 	points.sort( shuffleSort );
		if ( jitter != 0 ) 	jitterPoints( points, jitter );
		return points;
	}
	
	private function get_rect():Rectangle 
	{
		return _rect;
	}
	
	private function set_rect(value:Rectangle):Rectangle 
	{
		return _rect = value;
	}
	
	public var rect(get_rect, set_rect):Rectangle;
	
	private function get_radius():Float 
	{
		return _radius;
	}
	
	private function set_radius(value:Float):Float 
	{
		return _radius = value;
	}
	
	public var radius(get_radius, set_radius):Float;
	
}