package biga.shapes2d;
import nme.Vector;
import biga.utils.GeomUtils;
import nme.display.Graphics;
import nme.geom.Point;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Arc extends Circle 
{
	private var pi:Float;
	private var pi_4:Float;
	private var _startAngle:Float = 0;
	private var _arcLength:Float = 0;
	private var _segment:Bool = false;
	private var _sector:Bool = false;
	public function new( x:Float, y:Float, radius:Float, startAngle:Float = 0, arcLength:Float = 1 ) 
	{
		super( x, y, radius );
		pi = Math.PI;
		pi_4 = pi / 4;
		this._arcLength = arcLength;
		this._startAngle = startAngle;
	}
	//http://www.experts-exchange.com/Software/Photos_Graphics/Web_Graphics/Macromedia_Flash/ActionScript/Q_25121879.html
	override public function draw( graphics:Graphics ):Void
	{
		var start:Float = startAngle;
		var end:Float = arcLength;
		
		var difference:Float = Math.abs( start + ( end - start ) );
		var divisions:Int = Math.floor( difference / pi_4 ) + 1;
		var span:Float = difference / ( 2 * divisions );
		
		var controlRadius:Float = radius / Math.cos(span);
		var controlPoint:Point;
		var anchorPoint:Point;
		
		if ( sector ) 
		{
			graphics.moveTo( x, y );
			graphics.lineTo( x + Math.cos( start ) * radius, y + Math.sin( start ) * radius );
		}
		
		graphics.moveTo( x + Math.cos( start ) * radius, y + Math.sin( start ) * radius );
		for ( i in 0 ... divisions )
		{
			end 	= start + span;
			start 	= end + span;
			controlPoint 	= new Point(x + Math.cos(end) * controlRadius, y + Math.sin(end) * controlRadius);
			anchorPoint 	= new Point(x + Math.cos(start) * radius, y + Math.sin(start) * radius);
			graphics.curveTo(controlPoint.x, controlPoint.y, anchorPoint.x, anchorPoint.y);
		}
		
		if ( sector ) 
		{
			graphics.lineTo( x, y );
		}
		if ( segment )
		{
			graphics.lineTo( x + Math.cos( startAngle ) * radius, y + Math.sin( startAngle ) * radius );
		}
	}
	/**
	 * returns a series of points corresponding to the discretized arc
	 * @param	steps number of subdivisions
	 * @return a list of points
	 */
	public function subdivide( steps:Int ):Vector<Point>
	{
		var points:Vector<Point> = new Vector<Point>();
		for ( i in 0 ... steps )
		{
			points.push( getPointAt( i / ( steps - 1 ) ) );
		}
		return points;
	}
	/**
	 * gets a point on the arc at 0 > t > 1 
	 * @param	t
	 * @return
	 */
	public function getPointAt( t:Float, p:Point = null ):Point
	{
		if( p  == null ) p = new Point();
		var angle:Float = t * arcLength;
		p.x = x + Math.cos( startAngle + angle ) * radius;
		p.y = y + Math.sin( startAngle + angle ) * radius;
		return p;
	}
	
	public var startAngle( get_startAngle, set_startAngle ):Float;
	public function get_startAngle():Float { return _startAngle; }
	public function set_startAngle(value:Float):Float 
	{
		_startAngle = value;
		return startAngle;
	}
	
	public var arcLength( get_arcLength, set_arcLength ):Float;
	public function get_arcLength():Float { return _arcLength; }
	public function set_arcLength(value:Float):Float 
	{
		_arcLength = value;
		return arcLength;
	}
	
	public var segment( get_segment, set_segment ):Bool;
	public function get_segment():Bool { return _segment; }
	public function set_segment(value:Bool):Bool 
	{
		_segment = value;
		return segment;
	}
	
	public var sector( get_sector, set_sector ):Bool;
	public function get_sector():Bool { return _sector; }
	public function set_sector(value:Bool):Bool 
	{
		_sector = value;
		return sector;
	}
}