package biga.shapes2d;
import biga.utils.GeomUtils;
import nme.display.Graphics;
import nme.geom.Point;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Edge
{
	private var _p0:Point;
	private var _p1:Point;
	private var _id:Int;
	private var _angle:Float;
	private var _length:Float;
	public function new( p0:Point = null, p1:Point = null, id:Int = -1 ) 
	{
		this.p0 = p0;
		this.p1 = p1;
		this.id = id;
		if ( p0 != null && p1 != null )
		{
			_angle = GeomUtils.angle( p0, p1 );
			_length = GeomUtils.distance( p0, p1 );
		}
	}
	public var p0( get_p0, set_p0 ):Point;
	public function get_p0():Point { return _p0; }
	public function set_p0(value:Point):Point 
	{
		_p0 = value;
		return p0;
	}
	public var p1( get_p1, set_p1 ):Point;
	public function get_p1():Point { return _p1; }
	public function set_p1(value:Point):Point 
	{
		_p1 = value;
		return p1;
	}
	public var id( get_id, set_id ):Int;
	public function get_id():Int { return _id; }
	public function set_id(value:Int):Int 
	{
		_id = value;
		return id;
	}
	public function intersect( e:Edge, asSegment0:Bool = true, asSegment1:Bool = true ):Point
	{
		return GeomUtils.lineIntersectLine( p0, p1, e.p0, e.p1, asSegment0, asSegment1 );
	}
	public function equals( e:Edge, sameDirection:Bool = true ):Bool
	{
		if ( sameDirection ) return p0.equals( e.p0 ) && p1.equals( e.p1 );
		return ( ( p0 == e.p0 && p1 == e.p1 ) || ( p0 == e.p1 && p1 == e.p0 ) );
	}
	public function getPointAt( t:Float ):Point
	{
		return new Point(	p0.x + Math.cos( angle ) * length * t, 
							p0.y + Math.sin( angle ) * length * t 	);
	}
	public var center( get_center, null ):Point;
	public function get_center():Point
	{
		return new Point( p0.x + width * .5, p0.y + height * .5);
	}
	public var angle( get_angle, null ):Float;
	public function get_angle():Float 
	{ 
		return GeomUtils.angle( p0, p1 ); 
	}
	public var squareLength( get_squareLength, null ):Float;
	public function get_squareLength():Float
	{ 
		return GeomUtils.squareDistance( p0.x, p0.y, p1.x, p1.y ); 
	}
	public var length( get_length, null ):Float;
	public function get_length():Float
	{ 
		return GeomUtils.distance( p0, p1 ); 
	}
	public var width( get_width, null ):Float;
	public function get_width():Float 
	{	
		return p1.x - p0.x;		
	}
	public var height( get_height, null ):Float;
	public function get_height():Float 
	{	
		return p1.y - p0.y;		
	}
	public var leftNormal( get_leftNormal, null ):Edge;
	public function get_leftNormal():Edge
	{ 
		return new Edge( p0, new Point( p0.x + height, p0.y - width ) );
	}
	public var rightNormal( get_rightNormal, null ):Edge;
	public function get_rightNormal():Edge
	{ 
		return new Edge( p0, new Point( p0.x - height, p0.y + width ) );
	}
	public function draw( graphics:Graphics = null ):Void 
	{ 
		graphics.moveTo( p0.x, p0.y );
		graphics.lineTo( p1.x, p1.y );
	}
}