package biga.shapes2d;
import biga.utils.GeomUtils;
import biga.utils.PolygonUtils;
import nme.display.Graphics;
import nme.display.Shape;
import nme.display.Sprite;
import nme.geom.Point;
import nme.geom.Rectangle;
import nme.utils.clearInterval;
import nme.utils.setInterval;
/**
 * @author Nicolas Barradeau
 */
class Quad extends Sprite
{
	private var p0:Point;
	private var p1:Point;
	private var p2:Point;
	private var p3:Point;
	private var _anchor:Point;
	private var target:Point;
	private var lastTarget:Point;
	private var trackInterval:Int;
	private var _centroid:Point;
	private var _points:Vector<Point>;
	private var _edges:Vector<Edge>;
	private var shape:Shape = new Shape();
	public function Quad( p0:Point = null, p1:Point = null, p2:Point = null, p3:Point = null ) 
	{
		this.p0 = p0 || new Point();
		this.p1 = p1 || new Point();
		this.p2 = p2 || new Point();
		this.p3 = p3 || new Point();
		addChild( shape );
		reset();
	}
	public function fromRect( rect:Rectangle ):Quad
	{
		p0 = rect.topLeft.clone();
		p1 = new Point( rect.right, rect.top );
		p2 = rect.bottomRight.clone();
		p3 = new Point( rect.left, rect.bottom );
		reset()
		return this;
	}
	private function reset():Void 
	{
		points = Vector<Point>([	p0, 
									p1, 
									p2, 
									p3	]);
		edges = Vector<Edge>([ new Edge( p0, p1 ), 
								new Edge( p1, p2 ),
								new Edge( p2, p3 ),
								new Edge( p3, p0 )	]);
	}
	public function translate( x:Float = 0, y:Float = 0 ):Void
	{
		for (var p:Point in points ) 
		{
			p.x += x;
			p.y += y;
		}
	}
	public function rotate( angle:Float, anchor:Point = null ):Void
	{
		this.anchor = anchor || this.anchor || centroid;
		for( var p:Point in points )rotatePoint( p, angle, this.anchor );
	}
	private function rotatePoint( p:Point, angle:Float, anchor:Point ):Point
	{
		var a:Float, d:Float;
		a = GeomUtils.angle( anchor, p ) + angle;
		d = GeomUtils.distance( anchor, p );
		p.x = anchor.x + Math.cos( a ) * d;
		p.y = anchor.y + Math.sin( a ) * d;
		return p;
	}
	public function makeUnique():Quad
	{
		p0 = points[ 0 ].clone(); 
		p1 = points[ 1 ].clone(); 
		p2 = points[ 2 ].clone(); 
		p3 = points[ 3 ].clone(); 
		reset();
		return this;
	}
	public function makeConvex():Quad
	{
		points = PolygonUtils.convexHull( points );
		reset();
		return this;
	}
	public var centroid( get_centroid, null ):Point;
	public function get_centroid():Point 
	{
		if( _centroid  == null ) _centroid = new Point();
		_centroid.x = _centroid.y = 0;
		for (var p:Point in points ) 
		{
			_centroid.x += p.x * .25;
			_centroid.y += p.x * .25;
		}
		return _centroid;
	}
	public var points( get_points, set_points ):Vector<Point>;
	public function get_points():Vector<Point> 
	{
		return _points;
	}
	public function set_points(value:Vector<Point>):Void 
	{
		_points = value;
		return points;
	}
	public var edges( get_edges, set_edges ):Vector<Edge>;
	public function get_edges():Vector<Edge> 
	{
		return _edges;
	}
	public function set_edges(value:Vector<Edge>):Void 
	{
		_edges = value;
		return edges;
	}
	public var anchor( get_anchor, set_anchor ):Point;
	public function get_anchor():Point 
	{
		return _anchor;
	}
	public function set_anchor(value:Point):Void 
	{
		_anchor = value;
		shape.x = -anchor.x;
		shape.y = -anchor.y;
		x = anchor.x;
		y = anchor.y;
		return anchor;
	}
	public function track( target:Point ):Void 
	{
		this.target = target;
		if ( target != null )
		{
			lastTarget = this.target.clone();
			trackInterval = setInterval( updateTracker, 20 ); 
		}
		else
		{
			clearInterval( trackInterval ); 
		}
	}
	private function updateTracker():Void 
	{
		anchor = lastTarget;
		//translate( lastTarget.x - target.x, lastTarget.y - target.y );
		lastTarget.x = target.x;
		lastTarget.y = target.y;
	}
	public function draw( color:Int = 0 ):Void
	{
		shape.graphics.clear();
		shape.graphics.lineStyle( 0, color );
		shape.graphics.moveTo( p0.x, p0.y ); 
		shape.graphics.lineTo( p1.x, p1.y ); 
		shape.graphics.lineTo( p2.x, p2.y ); 
		shape.graphics.lineTo( p3.x, p3.y ); 
		shape.graphics.lineTo( p0.x, p0.y ); 
	}
	public var pivot( null, set_pivot ):Int;
	public function set_pivot( id:Int ):Void
	{
		shape.x = -points[ id].x;
		shape.y = -points[ id].y;
		return pivot;
	}
}