package biga.shapes2d;
import nme.Vector;
import biga.utils.GeomUtils;
import nme.display.GradientType;
import nme.display.Graphics;
import nme.geom.Point;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class RegularPolygon extends Point
{
	private var _points:Vector<Point>;
	private var _radius:Float = 50;
	private var _sides:Int = 5;
	private var _sideLength:Float;
	private var _rotation:Float = 0;
	private var _inRadius:Float;
	private var _centralAngle:Float = 0;
	private var _internalAngle:Float = 0;
	private var _externalAngle:Float = 0;
	private var _offset:Point;
	private var _offsetAngle:Float = 0;
	private var _offsetLength:Float = 0;
	public function new( x:Float = 0, y:Float = 0, radius:Float = 50, sides:Int = 5 ) 
	{
		super( x, y );
		_points = new Vector<Point>();
		_radius = radius;
		_sides = sides < 3 ? 3 : sides;
		_offset = new Point();
		resetShape();
	}
	private function resetShape():Void
	{
		var i:Int = 0;
		var angle:Float = _rotation;
		_inRadius = _radius * Math.cos( Math.PI / _sides );
		_sideLength = ( 2 * _radius ) * Math.sin( Math.PI / _sides );
		_centralAngle = ( Math.PI * 2 ) / _sides;
		_internalAngle = ( 1 - 2 / sides ) * Math.PI;
		_externalAngle = Math.PI - _internalAngle;
		for ( i in 0... sides )
		{
			var p:Point = new Point( 	x + Math.cos( angle ) * radius, 
										y + Math.sin( angle ) * radius );
			angle += centralAngle;
			points.push( p );
		}
	}
	public function translate( anchor:Point ):Void
	{
		var p:Point;
		x += anchor.x;
		y += anchor.y;
		for( p in points )
		{
			p.x += anchor.x;
			p.y += anchor.y;
		}
	}
	public function rotate( anchor:Point, angle:Float ):Void
	{
		rotatePoint( this, anchor, angle );
		var p:Point;
		for( p in points )
		{
			rotatePoint( p, anchor, angle );
		}
		_rotation = angle;
	}
	private function rotatePoint( p:Point, anchor:Point, angle:Float ):Point
	{
		var a:Float, d:Float;
		a = GeomUtils.angle( anchor, p ) + angle;
		d = GeomUtils.distance( anchor, p );
		p.x = anchor.x + Math.cos( a ) * d;
		p.y = anchor.y + Math.sin( a ) * d;
		return p;
	}
	public function reflect( anchor0:Point, anchor1:Point ):Void
	{
		var p:Point;
		reflectPoint( this, anchor0, anchor1 );
		for( p in points )
		{
			reflectPoint( p, anchor0, anchor1 );
		}
	}
	private function reflectPoint( p:Point, p0:Point, p1:Point ):Void
	{
		var pp:Point = GeomUtils.project( p, p0, p1 );
		if( Math.isNaN( pp.x ) || Math.isNaN( pp.y ) ) pp = GeomUtils.varrain( p, p0, p1 );
		p.x += ( pp.x - p.x ) * 2;
		p.y += ( pp.y - p.y ) * 2;
	}
	public function glide( anchor0:Point, anchor1:Point = null ):Void
	{
		if ( anchor1 == null ) anchor1 = this.clone();
		translate( anchor0.subtract( anchor1 ) );
		reflect( anchor0, anchor1 );
	}
	public var points( get_points, null ):Vector<Point>;
	public function get_points():Vector<Point> { return _points; }
	public var radius( get_radius, set_radius ):Float;
	public function get_radius():Float { return _radius; }
	public function set_radius(value:Float):Float 
	{
		_radius = value;
		resetShape();
		return radius;
	}
	public var sides( get_sides, set_sides ):Int;
	public function get_sides():Int { return _sides; }
	public function set_sides(value:Int):Int 
	{
		if ( _sides == value || value < 3 ) return 0;
		_sides = value;
		resetShape();
		return sides;
	}
	public var rotation( get_rotation, set_rotation ):Float;
	public function get_rotation():Float { return _rotation; }
	public function set_rotation(value:Float):Float 
	{
		_rotation = value;
		resetShape();
		return rotation;
	}
	public var sideLength( get_sideLength, set_sideLength ):Float;
	public function get_sideLength():Float{ return _sideLength; }
	public function set_sideLength( value:Float ):Float 
	{ 
		_radius = value / ( 2 * Math.sin( Math.PI / _sides ) );
		resetShape();
		return sideLength;
	} 
	public var inRadius( get_inRadius, null ):Float;
	public function get_inRadius():Float { return _inRadius; }
	public var apothem( get_apothem, null ):Float;
	public function get_apothem():Float { return inRadius; }
	public var centralAngle( get_centralAngle, null ):Float;
	public function get_centralAngle():Float { return _centralAngle; }
	public var internalAngle( get_internalAngle, null ):Float;
	public function get_internalAngle():Float { return _internalAngle; }
	public var externalAngle( get_externalAngle, null ):Float;
	public function get_externalAngle():Float { return _externalAngle; }
	public var center( get_center, null ):Point;
	public function get_center():Point
	{
		return new Point( 	( x + Math.cos( rotation + _offsetAngle )* -_offsetLength ),
							( y + Math.sin( rotation + _offsetAngle ) * -_offsetLength )	);
	}
	//public function set_inRadius(value:Float):Float 
	//{
		//_radius = value / Math.cos( Math.PI / sides );
		//resetShape();
	//	return inRadius;
	//}
	//
	//public var sagitta( get_sagitta, null ):Float;
	//public function get_sagitta():Float 
	//{
		//var sin:Float = Math.sin( Math.PI / sides * 2 )
		//return 2 * radius * ( sin * sin );
	//}
	public function draw( graphics:Graphics = null ):Void
	{
		var p:Point = points[ 0 ];
		graphics.moveTo( p.x, p.y );
		//graphics.drawCircle( p.x, p.y, 5 );
		for( p in points )
		{
			graphics.lineTo( p.x, p.y );
		}
		graphics.lineTo( points[ 0 ].x, points[ 0 ].y );
	}
}