package biga.shapes2d.curves;
import nme.Vector;
import nme.geom.Point;
// forked from makc3d's Cardinal Splines Part 4
 /**
 * Exploring formula in Jim Armstrong "Cardinal Splines Part 4"
 * @see http://algorithmist.wordpress.com/2009/10/06/cardinal-splines-part-4/
 */
 class Cardinal implements ICurve2D
 {
	private var _precision:Float = .1;
	private var _loop:Bool = false;
	private var _tension:Float = 1;
	private var _vertices:Vector<Point>;
	
	static private var p0:Point;
	static private var p1:Point;
	static private var p2:Point;
	static private var p3:Point;
	static private var t0:Float;
	static private var t1:Float;
	static private var t2:Float;
	public function new( precision:Float = .1, tension:Float = .1,loop:Bool = false )
	{
		_precision = precision;
		_tension = tension;
		_loop = loop;
	}
	
	public function getPointAt( t:Float, p:Point = null ):Point 
	{
		if ( vertices == null ) return null;
		
		if( p  == null ) p = new Point();
		
		var i:Int = Math.floor( vertices.length * t );
		var delta:Float = 1 / ( vertices.length );
		t = ( t - ( i * delta ) ) / delta;
		
		if ( !_loop )
		{
			p0 = vertices [ cast( Math.max( 0,(i - 1) ), Int ) ];
			p1 = vertices [ i ];
			p2 = vertices [ cast( Math.min( i + 1, vertices.length -1 ), Int ) ];
			p3 = vertices [ cast( Math.min( i + 2, vertices.length -1 ), Int ) ];
		}
		else
		{
			p0 = vertices [ (i - 1) < 0 ? ( vertices.length - 1 ) : (i - 1) ];
			p1 = vertices [ i ];
			p2 = vertices [ ( i + 1 ) % vertices.length ];
			p3 = vertices [ ( i + 2 ) % vertices.length ];
		}
		
		t0 = ( -t * t * t + 2 * t * t - t);
		t1 = ( -t * t * t + t * t);
		t2 = (2 * t * t * t - 3 * t * t + 1);
		p.x = _tension * t0 * p0.x + _tension * t1 * p1.x + t2 * p1.x + _tension * (t * t * t - 2 * t * t + t) * p2.x + ( -2 * t * t * t + 3 * t * t) * p2.x +_tension * (t * t * t - t * t) * p3.x;
		p.y = _tension * t0 * p0.y + _tension * t1 * p1.y + t2 * p1.y + _tension * (t * t * t - 2 * t * t + t) * p2.y + ( -2 * t * t * t + 3 * t * t) * p2.y +_tension * (t * t * t - t * t) * p3.y;
		return p;
		
	}
	
	public function compute( target:Vector<Point> = null ):Vector<Point>
	{
		return range( 0, 1, target );
	}
	
	public function range( start:Float = 0, end:Float = 1, target:Vector<Point> = null ):Vector<Point> 
	{
		
		if(  target == null )  target = new Vector<Point>();
		
		var t:Float = Math.min( start, end );
		var max:Float = Math.max( start, end );
		var increment:Float = precision / _vertices.length;
		
		while ( t < max ) 
		{
			target.push( getPointAt( t ) );
			t += increment;
		}
		
		if ( loop ) target.push( target[ 0 ] );
		return target;
		
	}
	
	private function get_precision():Float 
	{
		return _precision;
	}
	
	private function set_precision(value:Float):Float 
	{
		return _precision = value;
	}
	
	public var precision(get_precision, set_precision):Float;
	
	private function get_loop():Bool 
	{
		return _loop;
	}
	
	private function set_loop(value:Bool):Bool 
	{
		return _loop = value;
	}
	
	public var loop(get_loop, set_loop):Bool;
	
	private function get_vertices():Vector<Point> 
	{
		return _vertices;
	}
	
	private function set_vertices(value:Vector<Point>):Vector<Point> 
	{
		return _vertices = value;
	}
	
	public var vertices(get_vertices, set_vertices):Vector<Point>;
	
	public function get_tension():Float 
	{
		return _tension;
	}
	
	public function set_tension(value:Float):Float 
	{
		_tension = value;
		return tension;
	}
	
	public var tension( get_tension, set_tension ):Float;
	
	
 }