package biga.shapes2d.curves;
import nme.Vector;
import nme.geom.Point;
 /** 
 * Catmull-Rom spline through N vertices.
 * @author makc
 * @license WTFPLv2, http://sam.zoy.org/wtfpl/
 */ 
 class Cubic implements ICurve2D
 {
	private var _precision:Float = .1;
	private var _loop:Bool = false;
	private var _vertices:Vector<Point>;
	
	static private var p0:Point;
	static private var p1:Point;
	static private var p2:Point;
	static private var t0:Float;
	static private var t1:Float;
	static private var t2:Float;
	static private var t3:Float;
	public function new( precision:Float = .1, loop:Bool = false )
	{
		_precision = precision;
		_loop = loop;
	}
	
	public function compute( target:Vector<Point> = null ):Vector<Point>
	{
		return range( 0, 1, target );
	}
	
	public function range( start:Float = 0, end:Float = 1, target:Vector<Point> = null):Vector<Point> 
	{
		
		if (  target  == null )  target = new Vector<Point>();
		
		var t:Float = Math.min( start, end );
		var max:Float = Math.max( start, end );
		var increment:Float = precision / vertices.length;
		
		while ( t < max ) 
		{
			target.push( getPointAt( t ) );
			t += increment;
		}
		
		if ( loop ) target.push( target[ 0 ].clone() );
		return target;
		
	}
	
	public function getPointAt( t:Float, p:Point = null ):Point 
	{
		if ( vertices == null ) return null;
		
		if( p  == null ) p = new Point();
		
		var i:Int = Math.floor( vertices.length * t );
		
		var delta:Float = 1 / ( vertices.length );
		t = ( t - ( i * delta ) ) / delta;
		p1 = vertices [ i ];
		if ( _loop )
		{
			p0 = midPoint( ( i - 1 ) < 0 ? vertices[ vertices.length - 1 ] : vertices[ ( i - 1 ) ], p1 );
			p2 = midPoint( p1, ( i + 1 ) < vertices.length ? vertices[ ( i + 1 ) ] : vertices[ 0 ] );
		}
		else
		{
			p0 = midPoint( ( i - 1 ) < 0 ? vertices[ 0 ] : vertices[ ( i - 1 ) ], p1 );
			p2 = midPoint( p1, ( i + 1 ) < vertices.length ? vertices[ ( i + 1 ) ] : vertices[ vertices.length - 1 ] );
		}
		t0 = 1 - t;
		t1 = t0 * t0;
		t2 = 2 * t * t0;
		t3 = t * t;
		p.x = t1 * p0.x + t2 * p1.x + t3 * p2.x;
		p.y = t1 * p0.y + t2 * p1.y + t3 * p2.y;
		return p;
	}
	
	private function midPoint( p0:Point, p1:Point, dest:Point = null ) :Point
	{
		if( dest  == null ) dest = new Point();
		dest.x = p0.x + ( p1.x - p0.x ) / 2;
		dest.y = p0.y + ( p1.y - p0.y ) / 2;
		return dest;
	}
	
	private function get_precision():Float 
	{
		return _precision;
	}
	
	private function set_precision(value:Float):Float 
	{
		return _precision = value;
	}
	
	public var precision(get_precision, set_precision):Float;
	
	private function get_loop():Bool 
	{
		return _loop;
	}
	
	private function set_loop(value:Bool):Bool 
	{
		return _loop = value;
	}
	
	public var loop(get_loop, set_loop):Bool;
	
	private function get_vertices():Vector<Point> 
	{
		return _vertices;
	}
	
	private function set_vertices(value:Vector<Point>):Vector<Point> 
	{
		return _vertices = value;
	}
	
	public var vertices(get_vertices, set_vertices):Vector<Point>;
	
 }