/*
import nme.Vector;
Copyright (c) 2006 Arpit Mathur
Permission is hereby granted, free of charge, to any person 
obtaining a copy of this software and associated documentation 
files (the "Software"), to deal in the Software without restriction, 
including without limitation the rights to use, copy, modify, merge, 
publish, distribute, sublicense, and/or sell copies of the Software, 
and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:
The above copyright notice and this permission notice shall be 
included in all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package treemap
{
import treemap.layouts.*;
import nme.geom.Rectangle;
import nme.utils.getQualifiedClassName;
 /**
 * The TreeMap class accepts a series of ITreeData items
 * and represents them in a series of Rectangle objects, 
 * the width and height of each of them proportional to 
 * their relative <code>weight</code> property.
 */ 
 class TreeMap
{
	//maintains a list of treemapitems
 private var _itemList:Vector<TreeMapItem>;
 private var zoneRect:Rectangle;
 private var _x:Float = 0;
 private var _y:Float = 0;
 private var _width:Float = 1;
 private var _height:Float = 1;
 private var currentLayoutManager:ILayoutManager;
	private var _layoutDirection:String = TreemapConstants.DIRECTION_HORIZONTAL;
	private var _layoutType:String = TreemapConstants.LAYOUT_SQUARIFIED;
	private var totalWeight:Float;
 /**
 * Constructor
 */ 
 public function new( rect:Rectangle = null ) 
	{
		if ( rect != null ) 
		{
			this.rect = rect;
		}
	}
 /**
 * Set an array of ITreeMapData items
 * as the dataProvider for this instance
 */ 
 public function compute( values:Vector<TreeMapItem>, sortMethod:Function = null ):Vector<TreeMapItem>
	{
		//_itemList = new Vector<TreeMapItem>();
 zoneRect = new Rectangle( x, y, width, height );
		if ( zoneRect == null ) return null;
		// the vector is empty
 if ( values.length == 0 )
		{
 return null;
 }
		//if there's only one entry in the item vector, return one item with the whole area
		if ( values.length == 1 )
		{
			var tmi:ITreeMapItem = values[ 0 ].clone();
			tmi.rect = zoneRect.clone();
			_itemList = Vector<TreeMapItem>( [ tmi ] );
			return items;
 }
		_itemList = values;
		//computes area for item
 processValues( _itemList );
		//sort items in a givent order
		if ( sortMethod != null )
		{
			values.sort( sortMethod );
		}
		//assigns the manager
 getLayoutManager( zoneRect );
		//processes the layout
		_itemList = currentLayoutManager.createMap( _itemList );
		return _itemList;
 }
 /**
 * initializes with the area required by each TreeMapItem and saves the total weight of all weights
 */ 
 private function processValues( _itemList:Vector<TreeMapItem> ):Void
	{
		var item:TreeMapItem;
 totalWeight = 0;
		for( item in _itemList)
		{
 totalWeight += item.weight; 
 }
 var surfaceRatio:Float = ( width * height ) / totalWeight;
 for( item in _itemList)
 {
 item.area = item.weight * surfaceRatio;
			var size:Float = Math.sqrt( item.area );
			item.rect = new Rectangle( zoneRect.x, zoneRect.y, size, size );
		}
 }
 private function getLayoutManager( zoneRect:Rectangle ):Void
	{
		if ( layoutType == TreemapConstants.LAYOUT_SLICE_DICE )
		{
			currentLayoutManager = new SliceDiceLayout( zoneRect, totalWeight );
			( currentLayoutManager as SliceDiceLayout ).direction = layoutDirection;
		}
		if ( layoutType == TreemapConstants.LAYOUT_SQUARIFIED )
		{
			currentLayoutManager = new SquarifiedLayout( zoneRect );
			( currentLayoutManager as SquarifiedLayout ).direction = ( zoneRect.width < zoneRect.height ) ? TreemapConstants.DIRECTION_HORIZONTAL : TreemapConstants.DIRECTION_VERTICAL;
		}
		if ( layoutType == TreemapConstants.LAYOUT_STRIP )
		{
			currentLayoutManager = new StripLayout( zoneRect );
			( currentLayoutManager as StripLayout ).direction = layoutDirection;
		}
 }
	/************************************************
	GETTERS / SETTERS
	************************************************/
	public var x( get_x, set_x ):Float;
	public function get_x():Float { return _x; }
	public function set_x(value:Float):Float 
	{
		_x = value;
		return x;
	}
	public var y( get_y, set_y ):Float;
	public function get_y():Float { return _y; }
	public function set_y(value:Float):Float 
	{
		_y = value;
		return y;
	}
	public var width( get_width, set_width ):Float;
 public function get_width():Float { return _width; }
 public function set_width( value:Float ):Float
	{
 _width = value;
 	return width;
	}
	public var height( get_height, set_height ):Float;
 public function get_height():Float { return _height; }
 public function set_height( value:Float ):Float
	{
 _height = value;
 	return height;
	}
	public var rect( get_rect, set_rect ):Rectangle;
	public function get_rect():Rectangle { return new Rectangle( x, y, width, height ); }
	public function set_rect(value:Rectangle):Rectangle 
	{
		x = value.x;
		y = value.y;
		width = value.width;
		height = value.height;
		return rect;
	}
	public var layoutDirection( get_layoutDirection, set_layoutDirection ):String;
	public function get_layoutDirection():String { return _layoutDirection; }
	public function set_layoutDirection(value:String):String 
	{
		_layoutDirection = value;
		return layoutDirection;
	}
	public var layoutType( get_layoutType, set_layoutType ):String;
	public function get_layoutType():String { return _layoutType; }
	public function set_layoutType(value:String):String 
	{
		_layoutType = value;
		return layoutType;
	}
	public var items( get_items, null ):Vector;
	public function get_items():Vector<TreeMapItem> { return _itemList; }
 }
}
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null
null