package treemap.layouts;
import nme.Vector;
import treemap.TreemapConstants;
import treemap.TreeMapItem;
import nme.geom.Rectangle;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class StripLayout extends LayoutManager 
{
	private var _direction:String;
	private var _rect:Rectangle;
	private var totalSize:Float = 0; 
	public function new( rect:Rectangle ) 
	{
		this.rect = rect;
	}
 override public function createMap( list:Vector<TreeMapItem> ):Vector<TreeMapItem>
	{
		if ( direction == TreemapConstants.DIRECTION_HORIZONTAL )
		{
			layoutHorizontal( list, rect );
		}
		else
		{
			layoutVertical( list, rect );
		}
		return list;
 }
	public var rect( get_rect, set_rect ):Rectangle;
	public function get_rect():Rectangle { return _rect; }
	public function set_rect(value:Rectangle):Rectangle 
	{
		_rect = value;
		return rect;
	}
	public var direction( get_direction, set_direction ):String;
	public function get_direction():String { return _direction; }
	public function set_direction(value:String):String 
	{
		_direction = value;
		return direction;
	}
	public function layoutHorizontal( list:Vector<TreeMapItem>, bounds:Rectangle):Void
	{
		items = list.concat();
		var i:Int;
		for (i in 0... items.length) totalSize += items[i].area;
		var area:Float = bounds.width * bounds.height;
		var scaleFactor:Float = Math.sqrt( area / totalSize );
		var finishedIndex:Int = 0;
		var numItems:Int = 0;
		var height:Float;
		var yoffset:Float = 0;
		var box:Rectangle = bounds.clone();
		box.x /= scaleFactor;
		box.y /= scaleFactor;
		box.width /= scaleFactor;
		box.height /= scaleFactor;
		while ( finishedIndex < items.length)
		{
			numItems = layoutStrip( box, finishedIndex );
			if( (finishedIndex + numItems) < items.length )
			{
				var numItems2:Int;
				var ar2a:Float;
				var ar2b:Float;
				numItems2 = layoutStrip( box, finishedIndex + numItems);
				ar2a = computeAverageAspectRatio(finishedIndex, numItems + numItems2);
				computeHorizontalBoxLayout(box, finishedIndex, numItems + numItems2);
				ar2b = computeAverageAspectRatio(finishedIndex, numItems + numItems2);
				if (ar2b < ar2a) 
				{
					numItems += numItems2;
				}
				else 
				{
					computeHorizontalBoxLayout( box, finishedIndex, numItems );
				}
			}
			for ( i = finishedIndex; i < (finishedIndex + numItems))
			{
				items[i].rect.y = yoffset;
			}
			height = items[ finishedIndex ].rect.height;
			yoffset += height;
			box.y += height;
			box.height -= height;
			finishedIndex += numItems;
		}
		var rect:Rectangle;
		for (i in 0... items.length)
		{
			rect = items[i].rect;
			rect.x *= scaleFactor;
			rect.y *= scaleFactor;
			rect.width *= scaleFactor;
			rect.height *= scaleFactor;
			rect.x += bounds.x;
			rect.y += bounds.y;
		}
		totalSize = 0;
		for ( i = finishedIndex - numItems; i < items.length ) 
		{
			totalSize += items[ i ].weight;
		}
		height = ( bounds.y + bounds.height ) - rect.y;
		var ox:Int = bounds.x;
		for ( i = finishedIndex - numItems; i < items.length ) 
		{
			items[ i ].rect.width = items[ i ].weight * ( bounds.width / totalSize );
			items[ i ].rect.x = ox;
			ox += items[ i ].rect.width;
			items[ i ].rect.height = height;
		}
	}
	public function layoutVertical( list:Vector<TreeMapItem>, bounds:Rectangle):Void
	{
		items = list.concat();
		var i:Int;
		for (i in 0... items.length) totalSize += items[i].area;
		var area:Float = bounds.width * bounds.height;
		var scaleFactor:Float = Math.sqrt( area / totalSize );
		var finishedIndex:Int = 0;
		var numItems:Int = 0;
		var width:Float;
		var xoffset:Float = 0;
		var box:Rectangle = bounds.clone();
		box.x /= scaleFactor;
		box.y /= scaleFactor;
		box.width /= scaleFactor;
		box.height /= scaleFactor;
		while ( finishedIndex < items.length)
		{
			numItems = layoutStrip( box, finishedIndex );
			if( (finishedIndex + numItems) < items.length )
			{
				var numItems2:Int;
				var ar2a:Float;
				var ar2b:Float;
				numItems2 = layoutStrip( box, finishedIndex + numItems);
				ar2a = computeAverageAspectRatio(finishedIndex, numItems + numItems2);
				computeVerticalBoxLayout(box, finishedIndex, numItems + numItems2);
				ar2b = computeAverageAspectRatio(finishedIndex, numItems + numItems2);
				if (ar2b < ar2a) 
				{
					numItems += numItems2;
				}
				else 
				{
					computeVerticalBoxLayout( box, finishedIndex, numItems );
				}
			}
			for ( i = finishedIndex; i < (finishedIndex + numItems))
			{
				items[i].rect.x = xoffset;
			}
			width = items[ finishedIndex ].rect.width;
			xoffset += width;
			box.x += width;
			box.width -= width;
			finishedIndex += numItems;
		}
		var rect:Rectangle;
		for (i in 0... items.length)
		{
			rect = items[i].rect;
			rect.x *= scaleFactor;
			rect.y *= scaleFactor;
			rect.width *= scaleFactor;
			rect.height *= scaleFactor;
			rect.x += bounds.x;
			rect.y += bounds.y;
		}
		totalSize = 0;
		for ( i = finishedIndex - numItems; i < items.length ) 
		{
			totalSize += items[ i ].weight;
		}
		width = ( bounds.x + bounds.width ) - rect.x;
		var oy:Int = bounds.y;
		for ( i = finishedIndex - numItems; i < items.length ) 
		{
			items[ i ].rect.height = items[ i ].weight * ( bounds.height / totalSize );
			items[ i ].rect.y = oy;
			oy += items[ i ].rect.height;
			items[ i ].rect.width = width;
		}
	}
	protected function layoutStrip( box:Rectangle, index:Int ):Int
	{
		var numItems:Int = 0;
		var prevAR:Float = Number.MAX_VALUE;
		var ar:Float = Number.MAX_VALUE;
		do{
			prevAR = ar;
			numItems++;
			ar = computeAverageAspectRatio( index, numItems );
		}while ((ar < prevAR) && ((index + numItems) < items.length))
		if (ar >= prevAR)
		{
			numItems--;
			ar = computeAverageAspectRatio(index, numItems);
		}
		return numItems;
	}
	protected function computeHorizontalBoxLayout( box:Rectangle, index:Int, numItems:Int ):Float
	{
		var i:Int;
		var totalSize:Float = computeSize( index, numItems );
		var height:Float = totalSize / box.width;
		var x:Float = 0;
		var width:Float;
		for (i in 0... numItems)
		{
			width = items[i + index].area / height;
			items[i + index].rect = new Rectangle(x, 0, width, height);
			x += width;
		}
		return height;
	}
	protected function computeVerticalBoxLayout( box:Rectangle, index:Int, numItems:Int ):Float
	{
		var i:Int;
		var totalSize:Float = computeSize( index, numItems );
		var width:Float = totalSize / box.height;
		var y:Float = 0;
		var height:Float;
		for (i in 0... numItems)
		{
			height = items[i + index].area / width;
			items[i + index].rect = new Rectangle( 0, y, width, height);
			y += height;
		}
		return width;
	}
	private function computeSize( index:Int, num:Int ):Float 
	{
		var size:Float = 0;
		for ( i in 0 ... num)
		{
			size += items[ i + index ].area;
		}
		return size;
	}
	private function computeAverageAspectRatio( index:Int, numItems:Int ):Float
	{
		var tar:Float = 0;
		var i:Int;
		for (i in 0... numItems) 
		{
			tar += computeAspectRatio( i );
		}
		return tar / numItems;
	}
	private function computeAspectRatio( index:Int ):Float
	{
		var w:Float = items[index].rect.width;
		var h:Float = items[index].rect.height;
		return Math.min( (w / h), (h / w) );
	}
}