package biga.utils;
import biga.shapes2d.Arc;
import biga.shapes2d.Circle;
import nme.geom.Point;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class ArcUtils 
{
	static public var EPSILON:Float = .0001;
	static private var pi:Float = Math.PI;
	static private var pi2:Float = pi * 2;
	static private var pi_2:Float = pi / 2;
	static private var pi_4:Float = pi / 4;
	/**
	 * initialises an arc after 2 points
	 * @param	arc
	 * @param	p0
	 * @param	p1
	 * @param	circle if not null, will clone the circles properties
	 */
	static public function init( arc:Arc, p0:Point, p1:Point, circle:Circle = null ):Void
	{
		if ( circle != null )
		{
			arc.x = circle.x;
			arc.y = circle.y;
			arc.radius = circle.radius;
		}
		var a0:Float = GeomUtils.angle( arc, p0 );
		var a1:Float = GeomUtils.angle( arc, p1 );
		var max:Float = Math.max( a0, a1 );
		arc.startAngle = Math.min( a0, a1 );
		arc.arcLength = max - arc.startAngle;
		if ( arc.arcLength > pi + EPSILON )
		{
			invert( arc );
		}
	}
	/**
	 * returns a boolean to specify wether the point is on the arc segment
	 * @param	p
	 * @param	arc
	 * @return
	 */
	static public function isPointOnArc( p:Point, arc:Arc ):Bool
	{
		var a:Float = GeomUtils.angle( arc, p );
		var b:Float = ( arc.startAngle + arc.arcLength );
		if ( a < 0 )a += pi2;
		if ( a >= arc.startAngle && a <= b ) return true;
		if( b >= pi2 && ( a >= 0 && a <= ( b-pi2 ) ) ) return true;
		return false;
	}
	/**
	 * inverts the arc 
	 * @param	arc
	 */
	static public function invert( arc:Arc ):Void
	{
		var a:Float = arc.startAngle;
		arc.startAngle = ( arc.startAngle + arc.arcLength ) % pi2;
		arc.arcLength = pi2 - arc.arcLength;
	}
	static public function length( radius:Float, angle:Float ):Float { return radius * angle; }
	static public function angle( radius:Float, length:Float ):Float { return length / radius; }
	static public function radius( angle:Float, length:Float ):Float { return length / angle; }
}