package biga.utils;
import nme.display.BitmapData;
import nme.geom.Point;

/**
 * @author Nicolas Barradeau 
 */

class Bresenham 
{

	static public function line( p0:Point, p1:Point, bd:BitmapData, color:Int ):Void
	{
		
		var x1:Int = Std.int( p0.x );
		var y1:Int = Std.int( p0.y );
		
		var x2:Int = Std.int( p1.x );
		var y2:Int = Std.int( p1.y );
		
		var dx:Int = Std.int( x2 - x1 );
		var dy:Int = Std.int( y2 - y1 );
		
		var sx:Int = (x1 < x2) ? 1 : -1;
		var sy:Int = (y1 < y2) ? 1 : -1;
		
		var err:Int = dx - dy;
		
		while (true) 
		{
			bd.setPixel32( x1, y1, color );
			
			if (x1 == x2 && y1 == y2) {	break;	}
			
			var e2:Int = 2 * err;
			
			if( e2 > -dy ) 
			{
				err = err - dy;
				x1 = x1 + sx;
			}
			
			if( e2 < dx )
			{
				err = err + dx;
				y1 = y1 + sy;
			}
		}
	}
	
	static public function circle( x:Float, y:Float, radius:Float, bd:BitmapData, color:Int ):Void
	{
		var xp:Int = Std.int( x );
		var yp:Int = Std.int( y );
		
		var d:Int = Std.int( 3 - ( 2 * radius ) );
		var _x:Int = 0;
		var _y:Int = Std.int( radius );
		
		bd.lock();
		while( _x <= _y ) 
		{
			bd.setPixel32(	xp + _x,  yp + _y, color );
			bd.setPixel32(	xp + _x,  yp - _y, color );
			bd.setPixel32(	xp - _x,  yp + _y, color );
			bd.setPixel32(	xp - _x,  yp - _y, color );
			bd.setPixel32(	xp + _y,  yp + _x, color );
			bd.setPixel32(	xp + _y,  yp - _x, color );
			bd.setPixel32(	xp - _y,  yp + _x, color );
			bd.setPixel32(	xp - _y,  yp - _x, color );
			
			if ( d < 0 ) 
			{
				d = d + ( 4 * _x ) + 6;
			}
			else 
			{
				d = d + 4 * ( _x - _y ) + 10;
				_y--;
			}
			_x++;
		}
		bd.unlock();
		
	}
	
	static public function disc( x:Float, y:Float, radius:Float, bd:BitmapData, color:Int ):Void
	{
		var xp:Int = Std.int( x );
		var yp:Int = Std.int( y );
		var _x:Int = 0;
		var _y:Int = Std.int( radius );
		var balance:Int = -_y;
		
		bd.lock();
		while (_x <= _y) 
		{
			var p0:Int = xp - _x;
			var p1:Int = xp - _y;
			var w0:Int = _x + _x;
			var w1:Int = _y + _y;
			
			hLine( p0, yp + _y, w0, bd, color );
			hLine( p0, yp - _y, w0, bd, color );
			hLine( p1, yp + _x, w1, bd, color );
			hLine( p1, yp - _x, w1, bd, color );
			
			if ((balance += _x++ + _x) >= 0) 
			{
				balance -= --_y + _y;
			}
		}
		bd.unlock();
		
	}

	static private inline function hLine( xp:Int, yp:Int, w:Int, bd:BitmapData, color:Int ):Void 
	{
		for ( i in 0...w+1 )
		{
			bd.setPixel32( xp + i, yp, color );
		}
	}
	
}