package biga.utils;

import nme.display.Graphics;
import nme.geom.Point;
import nme.Vector;

	
/**
 * @author Nicolas Barradeau
 * @website http://en.nicoptere.net
 */
class Delaunay	
{
	static public var EPSILON:Float = 0.000001;
	static public var SUPER_TRIANGLE_RADIUS:Float = 1000000000;
	
	private var _indices:Vector<Int>;
	private var _circles:Vector<Float>;
	
	public function new()
	{
	}
	public function compute( points:Vector<Point> ):Vector<Int>
	{
		var nv:Int = points.length;
		if (nv < 3) return null;
		
		var d:Float = SUPER_TRIANGLE_RADIUS;
		points.push( 	new Point( 0, -d ) 	);
		points.push( 	new Point( d, d ) 	);
		points.push( 	new Point( -d, d )	);
		
		indices = new Vector<Int>( ); 
		indices.push( points.length - 3 );
		indices.push( points.length - 2 );
		indices.push( points.length - 1 );
		
		circles = new Vector<Float>();
		circles.push( 0 );
		circles.push( 0 );
		circles.push( d );
		
		var edgeIds:Vector<Int> = new Vector<Int>();
		var i:Int, j:Int, k:Int, id0:Int, id1:Int, id2:Int;
		for ( i in 0 ... nv )
		{
			
			//if point tested is inside existing triangle
			j = 0;
			while( j < Std.int( indices.length ) )
			{
				if ( 	circles[ j + 2 ] > EPSILON 		&& 		circleContains( j, points[ i ] )	)
				{
					id0 = indices[ j ];
					id1 = indices[ j + 1 ];
					id2 = indices[ j + 2 ];
					
					edgeIds.push( id0 );
					edgeIds.push( id1 );
					edgeIds.push( id1 );
					edgeIds.push( id2 );
					edgeIds.push( id2 );
					edgeIds.push( id0 );
					
					indices.splice( j, 3 );
					circles.splice( j, 3 );
					j -= 3;
				}
				j += 3;
			}
			
			//removes duplicate edges
			j = 0;
			while ( j < Std.int( edgeIds.length ) )
			{
				k = ( j + 2 );
				while ( k < Std.int( edgeIds.length  ) )
				{
					if(	(	edgeIds[ j ] == edgeIds[ k ] && edgeIds[ j + 1 ] == edgeIds[ k + 1 ]	)
					||	(	edgeIds[ j + 1 ] == edgeIds[ k ] && edgeIds[ j ] == edgeIds[ k + 1 ]	)	)
					{
						edgeIds.splice( k, 2 );
						edgeIds.splice( j, 2 );
						j -= 2;
						k -= 2;
						if ( j < 0 || j > edgeIds.length - 1 ) break;
						if ( k < 0 || k > edgeIds.length - 1 ) break;
					}
					k += 2;
				}
				j += 2;
			}
			j = 0;
			while( j < Std.int( edgeIds.length ) )
			{
				indices.push( edgeIds[ j ] );
				indices.push( edgeIds[ j + 1 ] );
				indices.push( i );
				computeCircle( points, edgeIds[ j ], edgeIds[ j + 1 ], i );
				j += 2;
			}
			edgeIds = new Vector<Int>();
			
		}
		id0 = points.length - 3;
		id1 = points.length - 2;
		id2 = points.length - 1;
		
		i = 0;
		while( i < Std.int( indices.length ) )
		{
			if ( indices[ i ] == id0 || indices[ i ] == id1 || indices[ i ] == id2 
			||	 indices[ i + 1 ] == id0 || indices[ i + 1 ] == id1 || indices[ i + 1 ] == id2 
			||	 indices[ i + 2 ] == id0 || indices[ i + 2 ] == id1 || indices[ i + 2 ] == id2 )
			{
				indices.splice( i, 3 );
				circles.splice( i, 3 );
				if( i > 0 ) i-=3;
				continue;
			}
			i += 3;
		}
		
		points.pop();
		points.pop();
		points.pop();
		return indices;
	}
	
	private function circleContains( circleId:Int, p:Point ):Bool 
	{
		var dx:Float = circles[ circleId ] - p.x;
		var dy:Float = circles[ circleId + 1 ] - p.y;
		return circles[ circleId + 2 ] > dx * dx + dy * dy;
	}
	
	private function computeCircle( points:Vector<Point>, id0:Int, id1:Int, id2:Int ):Void
	{
		var p0:Point = points[ id0 ];
		var p1:Point = points[ id1 ];
		var p2:Point = points[ id2 ];
		var A:Float = p1.x - p0.x;
		var B:Float = p1.y - p0.y;
		var C:Float = p2.x - p0.x;
		var D:Float = p2.y - p0.y;
		var E:Float = A * (p0.x + p1.x) + B * (p0.y + p1.y);
		var F:Float = C * (p0.x + p2.x) + D * (p0.y + p2.y);
		var G:Float = 2.0 * (A * (p2.y - p1.y) - B * (p2.x - p1.x));
		
		var x:Float = (D * E - B * F) / G;
		circles.push( x );
		
		var y:Float = (A * F - C * E) / G;
		circles.push( y );
		
		x -= p0.x;
		y -= p0.y;
		circles.push( x * x + y * y );
	}
	
	public function render( graphics:Graphics, points:Vector<Point>, indices:Vector<Int> ):Void
	{
		var i:Int = 0, id0:Int, id1:Int, id2:Int;
		while ( i < Std.int( indices.length ) )
		{
			
			id0 = indices[ i ];
			id1 = indices[ i + 1 ];
			id2 = indices[ i + 2 ];
			
			graphics.beginFill( 0, .1 + Math.random() * .2 );
			graphics.moveTo( points[ id0 ].x, points[ id0 ].y );
			graphics.lineTo( points[ id1 ].x, points[ id1 ].y );
			graphics.lineTo( points[ id2 ].x, points[ id2 ].y );
			graphics.lineTo( points[ id0 ].x, points[ id0 ].y );
			graphics.endFill();
			
			i += 3;
			
		}
	}
	
	private function get_circles():Vector<Float> 
	{
		return _circles;
	}
	
	private function set_circles(value:Vector<Float>):Vector<Float> 
	{
		return _circles = value;
	}
	
	public var circles(get_circles, set_circles):Vector<Float>;
	
	private function get_indices():Vector<Int> 
	{
		return _indices;
	}
	
	private function set_indices(value:Vector<Int>):Vector<Int> 
	{
		return _indices = value;
	}
	
	public var indices(get_indices, set_indices):Vector<Int>;
	
	
}
