package biga.utils;
import nme.Vector;
import adobe.utils.CustomActions;
import nme.utils.ByteArray;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class VectorUtils 
{
	/**
	 * gets the average value of 
	 * @param	values
	 * @param	start
	 * @param	valuesPerFrame
	 * @param	windowSize
	 * @return
	 */
	static public function backwardsAverageWindow( values:Vector<Float>, start:Int, valuesPerFrame:Int = 1024, windowSize:Int = 43 ):Vector<Float>
	{
		if ( windowSize == 1 ) return values;
		var tmp:Vector<Float> = new Vector<Float>( valuesPerFrame );
		var startPos:Int = Math.max( 0, ( start * valuesPerFrame ) - ( valuesPerFrame * windowSize ) );
		var endPos:Int = Math.min( values.length-valuesPerFrame, start * valuesPerFrame );
		var sum:Float;
		var min:Int, max:Int;
		for (var i:Int = startPos; i < endPos; i+= valuesPerFrame ) 
		{
			for (j in 0 ... valuesPerFrame) 
			{
				tmp[ j ] += values[ i + j ] / windowSize;
			}
		}
		return tmp;
	}
	static public function byteArrayToVector( ba:ByteArray, minMax:Array = null ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		ba.position = 0;
		while ( ba.bytesAvailable > 0 )
		{
			if ( minMax == null ) tmp.push( ba.readFloat() );
			else tmp.push( map( ba.readFloat(), -1, 1, minMax[ 0 ], minMax[ 1 ] ) );
		}
		ba.position = 0;
		return tmp;
	}
	static public function averageWindow( values:Vector<Float>, windowSize:Int ):Vector<Float>
	{
		if ( windowSize == 1 ) return values;
		var tmp:Vector<Float> = values.concat();
		if ( windowSize % 2 == 0 ) windowSize++;
		var halfWindow:Int = ( windowSize - 1 ) / 2;
		var sum:Float;
		var min:Int, max:Int;
		for (i in 0 ... values.length) 
		{
			tmp[ i ] = 0;
			min = Math.max( 0, ( i - halfWindow ) );
			max = Math.min( values.length - 1, ( i + halfWindow ) );
			for (var j:Int = min; j <= max) 
			{
				tmp[ i ] += values[ j ] / windowSize;
			}
		}
		return tmp;
	}
	static public function createBins( values:Vector<Float>, binSize:Int ):Vector<Vector<Float>>
	{
		var bin:Vector<Float>;
		var tmp:Vector<Vector<Float>> = new Vector<Vector<Float>>();
		for (i in 0 ... ( values.length / binSize )) 
		{
			bin = new Vector<Float>();
			for (var j:Int = i * binSize; j < ( i + 1 ) * binSize) 
			{
				if ( j < values.length )
				{
					bin.push( values[ j ] );
				}
				else
				{
					if( bin.length > 0 ) tmp.push( bin );
					return tmp;
				}
			}
			tmp.push( bin );
		}
		return tmp;
	}
	static public function toFixed( values:Vector<Float>, fixedValue:Int = 2 ):Vector<Float>
	{
		for (i in 0 ... values.length) 
		{
			values[ i ] = Number( values[ i ].toFixed( fixedValue ) )
		}
		return values;
	}
	static public function sum( values:Vector<Float> ):Float
	{
		var sum:Float = 0;
		for (var n:Float in values) 
		{
			sum += n;
		}
		return sum;
	}
	static public function rootMeanSquare( values:Vector<Float> ):Float
	{
		var sum:Float = 0;
		for (var n:Float in values) 
		{
			sum += n * n;
		}
		return Math.sqrt(sum / values.length);
	}
	static public function median( values:Vector<Float> ):Float
	{
		if ( values.length == 0 ) return NaN;
		values.sort( medianSort );
		var mid:Int = values.length * .5;
		if ( values.length % 2 == 1 )
		{
			return values[ mid ];
		}
		else
		{
			return ( values[ mid - 1 ] + values[ mid ] ) * .5;
		}
	}
	static private function medianSort( a:Float, b:Float ):Float
	{
		return ( a - b );
	}
	static public function averageSum( values:Vector<Float>, length:Float = 1 ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var binSum:Float;
		if ( length == 1 )
		{
			binSum = sum( values );
			tmp.push( binSum / values.length );
		}
		else
		{
			var bins:Vector<Vector<Float>> = createBins( values, length );
			for( var bin:Vector<Float> in bins )
			{
				binSum = sum( bin );
				tmp.push( binSum / bin.length );
			}
		}
		return tmp;
	}
	static public function averageRms( values:Vector<Float>, length:Float = 1 ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var binSum:Float;
		if ( length == 1 )
		{
			binSum = rootMeanSquare( values );
			tmp.push( binSum / values.length );
		}
		else
		{
			var bins:Vector<Vector<Float>> = createBins( values, length );
			for( var bin:Vector<Float> in bins )
			{
				binSum = rootMeanSquare( bin );
				tmp.push( binSum / bin.length );
			}
		}
		return tmp;
	}
	static public function normallizeMinMax( values:Vector<Float> ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var min:Float = Number.POSITIVE_INFINITY;
		var max:Float = Number.NEGATIVE_INFINITY;
		for (i in 0 ... values.length) 
		{
			if( values[ i ] < min ) min = values[ i ];
			if( values[ i ] > max ) max = values[ i ];
		}
		for ( i in 0... values.length) 
		{
			tmp.push( map( values[ i ], min, max, 0, 1 ) );
		}
		return tmp;
	}
	static public function normallizeAbsolute( values:Vector<Float> ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var sum:Float = 0;
		for (i in 0 ... values.length) 
		{
			sum += values[ i ];
		}
		var div:Float = 1 / sum;
		for ( i in 0... values.length) 
		{
			tmp.push( values[ i ] * div );
		}
		return tmp;
	}
	static public function localMaximum( values:Vector<Float> ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var max:Float = max( values );
		var div:Float = 1 / max;
		for ( i in 0 ... values.length) 
		{
			tmp.push( values[ i ] * div );
		}
		return tmp;
	}
	static public function max( values:Vector<Float> ):Float
	{
		var max:Float = Number.NEGATIVE_INFINITY;
		for (i in 0 ... values.length) 
		{
			if( values[ i ] > max ) max = values[ i ];
		}
		return max;
	}
	static public function min( values:Vector<Float> ):Float
	{
		var min:Float = Number.POSITIVE_INFINITY;
		for (i in 0 ... values.length) 
		{
			if( values[ i ] < min ) min = values[ i ];
		}
		return min;
	}
	/**
	 * shrinks a Vector<Float> down to a given length
	 * @param	values original vector of values
	 * @param	length the desired length
	 * @return a vector of values matching the desired length
	 */
	static public function compress( values:Vector<Float>, length:Int ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var i:Float;
		if ( values.length == 1 )
		{
			i = length;
			while( i-- > 0 ) tmp.push( values[0] );
			return tmp;
		}
		var inputLength:Int = values.length - 2;
		var outputLength:Int = length - 2;
		tmp.push( values[ 0 ] );
		i = 0;
		var j:Int = 0;
		while (j < inputLength) 
		{
			var diff:Int = ( i + 1 ) * inputLength - ( j + 1 ) * outputLength;
			if (diff < inputLength / 2 ) 
			{
				i++;
				tmp.push( values[ j++ ] );
			}
			else j++;
		}
		tmp.push( values[ inputLength + 1 ] );
		return tmp;
	}
	/**
	 * compresses 2 vectors of numbers
	 * @param	u first set of vectors to compress
	 * @param	v second set of vectors to compress
	 * @param	length0 the desired length for vector u
	 * @param	length1 the desired length for vector v
	 * @return a Vector<Float> of doublets (x,y) of the given length interpolating the original values
	 */
	static public function compress2D( u:Vector<Float>, v:Vector<Float>, length0:Int, length1:Int ):Vector<Float>
	{
		var _u:Vector<Float> = compress( u, length0 );
		var _v:Vector<Float> = compress( v, length1 );
		var tmp:Vector<Float> = new Vector<Float>();
		for (i in 0 ... length0) 
		{
			for (j in 0 ... length1) 
			{
				tmp.push( _u[ i ], _v[ j ] );
			}
		}
		return tmp;
	}
	/**
	 * compresses 3 vectors of numbers
	 * @param	u first set of vectors to compress
	 * @param	v second set of vectors to compress
	 * @param	w second set of vectors to compress
	 * @param	length0 the desired length for vector u
	 * @param	length1 the desired length for vector v
	 * @param	length2 the desired length for vector w
	 * @return a Vector<Float> of triplets (x,y,z) of the given length interpolating the original values
	 */
	static public function compress3D( u:Vector<Float>, v:Vector<Float>, w:Vector<Float>, length0:Int, length1:Int, length2:Int ):Vector<Float>
	{
		var _u:Vector<Float> = compress( u, length0 );
		var _v:Vector<Float> = compress( v, length1 );
		var _w:Vector<Float> = compress( w, length2 );
		var tmp:Vector<Float> = new Vector<Float>();
		for (i in 0 ... length0) 
		{
			for (j in 0 ... length1) 
			{
				for (k in 0 ... length2) 
				{
					tmp.push( _u[ i ], _v[ j ], _w[ k ] );
				}
			}
		}
		return tmp;
	}
	/**
	 * expands the content of a vector.<Float> to match a given length
	 * @param	values original vector of values
	 * @param	length the desired length of the new Vector
	 * @return a Vector<Float> of the given length interpolating the original values
	 */
	static public function expandColors( values:Vector<Int>, length:Int, loop:Bool = false ):Vector<Int>
	{
		var tmp:Vector<Int> = new Vector<Int>();
		var i:Float;
		if ( values.length == 1 )
		{
			i = length;
			while( i-- > 0 ) tmp.push( values[0] );
			return tmp;
		}
		if ( loop ) values.push( values[0] );
		var step:Float = 1 / ( ( values.length-1 ) / length );
		for ( i in 0... length - 1; i+= step ) 
		{
			var val0:Float = values[ Math.floor( i / step ) ];
			//var max:Int = ;
			//if ( max > values.length - 1 ) max = values.length - 1;
			var val1:Float = values[ ( Math.floor( i / step ) + 1 ) ];
			var delta:Float = ( i + step > length ) ? ( length - i ) : step;
			for (var j:Float = 0; j < delta) 
			{ 
				tmp.push( 	0xFF<<24 |
							lrp( j / delta, val0 >> 16	& 0xFF, val1 >> 16 & 0xFF	) << 16 |
							lrp( j / delta, val0 >> 8 	& 0xFF, val1 >> 8 & 0xFF	) << 8 |
							lrp( j / delta, val0 		& 0xFF, val1 	 & 0xFF	)
							);
				//_trace( j / delta, Math.floor( i / step ), Math.floor( i / step ) + 1);
				//_trace( '->', val0>>8, val1>>8, ':', interpolateColorsCompact( j / delta, Math.floor( val0 >> 24 ), Math.floor( val1>>24 ) ) );
				//tmp.push( 0xFF<<24 | interpolateColorsCompact( j / delta, Math.floor( val0 ), Math.floor( val1 ) ) );
			}
		}
		while ( tmp.length < length ) tmp.push( values[ values.length - 1 ] );
		if ( loop ) values.pop();
		return tmp;
	}
	/**
	 * expands the content of a vector.<Float> to match a given length
	 * @param	values original vector of values
	 * @param	length the desired length of the new Vector
	 * @return a Vector<Float> of the given length interpolating the original values
	 */
	static public function expand( values:Vector<Float>, length:Int, lerp:Bool = true ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var i:Float;
		if ( values.length == 1 )
		{
			i = length;
			while( i-- > 0 ) tmp.push( values[0] );
			return tmp;
		}
		var step:Float = 1 / length;
		tmp.push( values[ 0 ] );
		var id0:Int, id1:Int;
		var _t:Float, delta:Float;
		for ( var t:Float = step; t < ( 1 - step ); t += step ) 
		{
			if ( lerp )
			{
				id0 = Math.floor( ( values.length - 1 ) * t );
				id1 = id0 + 1;
				delta = 1 / ( values.length - 1 );
				_t = ( t - ( id0 * delta ) ) / delta;
				tmp.push( lrp( _t, values[ id0 ], values[ id1 ]) );
			}
			else
			{
				id0 = Math.floor( values.length * t );
				tmp.push( values[ id0 ] );
			}
		}
		tmp.push( values[ values.length - 1 ] );
		return tmp;
	}
	/**
	 * expands 2 vectors of numbers and combines them together
	 * @param	u first set of vectors to expand
	 * @param	v second set of vectors to expand
	 * @param	length0 the desired length for vector u
	 * @param	length1 the desired length for vector v
	 * @return a Vector<Float> of doublets (x,y) of the given length interpolating the original values
	 */
	static public function expand2D( u:Vector<Float>, v:Vector<Float>, length0:Int, length1:Int ):Vector<Float>
	{
		var _u:Vector<Float> = expand( u, length0 );
		var _v:Vector<Float> = expand( v, length1 );
		var tmp:Vector<Float> = new Vector<Float>();
		for (i in 0 ... length0) 
		{
			for (j in 0 ... length1) 
			{
				tmp.push( _u[ i ], _v[ j ] );
			}
		}
		return tmp;
	}
	/**
	 * expands 3 vectors of numbers
	 * @param	u first set of vectors to expand
	 * @param	v second set of vectors to expand
	public var of( null, set_of ):*;
	 * @param	w second set_of vectors to expand
	 * @param	length0 the desired length for vector u
	 * @param	length1 the desired length for vector v
	 * @param	length2 the desired length for vector w
	 * @return a Vector<Float> of triplets (x,y,z) of the given length interpolating the original values
	 */
	static public function expand3D( u:Vector<Float>, v:Vector<Float>, w:Vector<Float>, length0:Int, length1:Int, length2:Int ):Vector<Float>
	{
		var _u:Vector<Float> = expand( u, length0 );
		var _v:Vector<Float> = expand( v, length1 );
		var _w:Vector<Float> = expand( w, length2 );
		var tmp:Vector<Float> = new Vector<Float>();
		for (i in 0 ... length0) 
		{
			for (j in 0 ... length1) 
			{
				for (k in 0 ... length2) 
				{
					tmp.push( _u[ i ], _v[ j ], _w[ k ] );
					return of;
	}
			}
		}
		return tmp;
	}
	static public function nrm( t:Float, min:Float, max:Float):Float
	{
		return ( t - min ) / ( max - min );
	}
	static public function lrp( t:Float, min:Float, max:Float):Float
	{
		return min + ( max - min ) * t;
	}
	static public function map( t:Float, mi0:Float, ma0:Float, mi1:Float, ma1:Float ):Float
	{
		return lrp( nrm( t, mi0, ma0 ), mi1, ma1 );
	}
	/*
	 * http://wonderfl.net/c/eYx0
	 * This alternative method uses less local variables and less operators to achieve the same result:
	*/
	static private function interpolateColorsCompact( a:Int, b:Int, lerp:Float ):Int
	{ 
		var MASK1:Int = 0xff00ff; 
		var MASK2:Int = 0x00ff00; 
		var f2:Int = 256 * lerp;
		var f1:Int = 256 - f2;
		return ((((( a & MASK1 ) * f1 ) + ( ( b & MASK1 ) * f2 )) >> 8 ) & MASK1 ) | ((((( a & MASK2 ) * f1 ) + ( ( b & MASK2 ) * f2 )) >> 8 ) & MASK2 );
	}
}