package biga.utils;
import nme.display.Graphics;
import nme.geom.Point;
import nme.geom.Rectangle;
import nme.Vector;
/**
 * @author Nicolas Barradeau
 * @website http://en.nicoptere.net
 */
class Voronoi 
{
	private var _delaunay:Delaunay;
	private var indices:Vector<Int>;
	private var circles:Vector<Float>;
	public function new()
	{
	}
	
	public function compute( points:Vector<Point> ):Vector<Vector<Point>>
	{
		//@haxenme thanks for soothing my Flasher's broken heart and making me believe Javascript is an actual language.
		if ( delaunay == null )
		{
			
			delaunay = new Delaunay();
			delaunay.compute( points );
			
		}
		indices = delaunay.indices;
		circles = delaunay.circles;
		
		var i:Int = 0, j:Int, k:Int;
		
		var triangles:Vector<Triangle> = new Vector<Triangle>();
		var polygons:Vector<Vector<Point>> = new Vector<Vector<Point>>();
		var polygon:Vector<Point> = new Vector<Point>();
		
		while ( i < indices.length ) 
		{
			
			triangles.push( new Triangle( indices[ i ], indices[ i + 1 ], indices[ i + 2 ], i ) );
			i += 3;
			
		}
		
		for ( i in 0 ... points.length )
		{
			
			polygon = new Vector<Point>();
			
			for( t in triangles ) 
			{
				if ( t.contains( i ) )
				{
					var p = new Point( circles[ t.circleId ], circles[ t.circleId + 1 ] );
					polygon.push( p );
				}
			}
			
			if ( polygon.length < 2 ) continue;
			//computing the angles of the adjacent triagnles' circumcenters
			var angles:Vector<Float> = new Vector<Float>();
			for ( j in 0 ... polygon.length ) 
			{
				
				angles.push( Math.atan2( polygon[j].y - points[ i ].y, polygon[j].x - points[ i ].x ) + Math.PI * 2 );
				
			}
			//sorting the points of the voronoi cell clockwise
			for ( j in 0...polygon.length - 1)
			{
				for ( k in 0 ... ( polygon.length - 1 - j ) )
				{
					if (angles[k] > angles[k + 1] )
					{
						var polygon1:Point = polygon[k]; 
						polygon[k] = polygon[k + 1]; 
						polygon[k + 1] = polygon1;
						
						var tmp_angle:Float = angles[k]; 
						angles[k] = angles[k + 1]; 
						angles[k+1] = tmp_angle;
					}
				}
			}
			polygons.push( polygon );
			//renderPoly( graphics, polygon, 0x0066DD  );
		}
		return polygons;
	}
	
	public function render(graphics:Graphics, polygons:Vector<Vector<Point>>, rect:Rectangle = null ):Void 
	{
		
		var i:Int, j:Int;
		for ( i in 0 ... polygons.length ) 
		{
			var polygon:Vector<Point> = polygons[ i ];
			graphics.moveTo( polygon[0].x, polygon[0].y );
			for ( p in polygon )
			{
				
				if ( p == null ) continue;
				if ( rect != null )
				{
					if ( p.x < rect.x ) p.x = rect.x;
					if ( p.y < rect.y ) p.y = rect.y;
					if ( p.x > rect.x + rect.width ) p.x = rect.x + rect.width;
					if ( p.y > rect.y + rect.height ) p.y = rect.y + rect.height;
				}
				graphics.lineTo( p.x, p.y );
				
			}
			graphics.lineTo( polygon[0].x, polygon[0].y );
		}
	}
	
	private function get_delaunay():Delaunay 
	{
		return _delaunay;
	}
	
	private function set_delaunay(value:Delaunay):Delaunay 
	{
		return _delaunay = value;
	}
	
	public var delaunay(get_delaunay, set_delaunay):Delaunay;
	
}

class Triangle
{
	public var id0:Int;
	public var id1:Int;
	public var id2:Int;
	public var circleId:Int;
	public function new( id0:Int, id1:Int, id2:Int, circleId:Int )
	{
		this.id0 = id0;
		this.id1 = id1;
		this.id2 = id2;
		this.circleId = circleId;
	}
	public function contains(i:Int):Bool
	{
		return id0 == i || id1 == i || id2 == i;
	}
	
}