package h3d.camera;

import h3d.utils.Constants3D;
import nme.display.Stage;
import nme.errors.Error;
import nme.events.MouseEvent;
import nme.geom.Matrix3D;
import nme.geom.Rectangle;
import nme.geom.Vector3D;
/**
 * @author Nicolas CHESNE
 */
class ArcBallCamera extends Object3D
{
	
	private var _stage:Stage;
	private var _radius:Float;
	private var _lastMouseX:Float;
	private var _lastMouseY:Float;
	private var _zoomSpeed:Int = 10;
	private var _rect:Rectangle;
	public function new( stage:Stage, radius:Float = 5 )
	{
		
		super();
		
		_radius = radius;
		_stage = stage;
		
		_position = new Vector3D(0, 0, _radius);
		
		_target = new Object3D();
		_target.position = Constants3D.ZERO.clone();
		
		_transform.appendTranslation(_position.x, _position.y, _position.z);
		_stage.addEventListener(MouseEvent.MOUSE_DOWN, onDown);
		_stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
		
	}

	private function onMouseWheel(event:MouseEvent):Void 
	{
		_radius += -1 * event.delta * _zoomSpeed;
		_position = new Vector3D(0, 0, _radius);
		
		_transform.appendTranslation(0, 0, -1 * event.delta);
		
		trace( _transform.position );
	}

	private function onDown(event:MouseEvent):Void
	{
		_stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
		_stage.addEventListener(MouseEvent.MOUSE_UP, onUp);
		_lastMouseX = _stage.mouseX;
		_lastMouseY = _stage.mouseY;
	}

	private function onMouseMove(event:MouseEvent):Void
	{
		var dx:Float = _stage.mouseX - _lastMouseX;
		_transform.prependRotation((dx / _stage.stageWidth) * 360, Vector3D.Y_AXIS);
		
		var dy:Float = _stage.mouseY - _lastMouseY;
		_transform.prependRotation((dy / _stage.stageHeight) * 360, Vector3D.X_AXIS);
		
		_transform.appendTranslation(0, 0, _radius );
		
		Constants3D.lookAt( _transform, _transform.position, _target.position );
		
		_lastMouseX = _stage.mouseX;
		_lastMouseY = _stage.mouseY;
		
	}

	override private function get_transform():Matrix3D 
	{
		return _transform;
	}
	override private function set_transform( value:Matrix3D ):Matrix3D 
	{
		return _transform;
	}
	
	private function onUp(event:MouseEvent):Void 
	{
		_stage.removeEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
	}

	private function get_zoomSpeed():Int 
	{
		return _zoomSpeed;
	}
	
	private function set_zoomSpeed(value:Int):Int 
	{
		return _zoomSpeed = value;
	}
	
	public var zoomSpeed(get_zoomSpeed, set_zoomSpeed):Int;
	
	private function get_rect():Rectangle 
	{
		return _rect;
	}
	
	private function set_rect(value:Rectangle):Rectangle 
	{
		return _rect = value;
	}
	
	public var rect(get_rect, set_rect):Rectangle;

}