package h3d.camera;
import h3d.camera.lenses.Lens;
import h3d.core.Object3D;
import h3d.core.Viewport;
import h3d.utils.Constants3D;
import nme.events.Event;
import nme.geom.Matrix3D;
import nme.geom.Vector3D;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Camera extends Object3D
{
	private var _viewport:Viewport;
	private var _lens:Lens;
	
	public function new( viewport:Viewport, lens:Lens = null ) 
	{
		super();
		
		this.viewport = viewport;
		viewport.addEventListener( Event.RESIZE, onResize );
		
		if ( lens == null ) lens = new Lens();
		this.lens = lens;
		
	}
	
	private function onResize(e:Event):Void 
	{
		reset();
	}
	
	override private function get_transform():Matrix3D 
	{
		
		_transform.identity();
		
		var _tmpPosition:Vector3D = _position.clone();
		
		_tmpPosition.negate();
		
		_transform.appendTranslation( 	_tmpPosition.x * Constants3D.X_AXIS.x, 
										_tmpPosition.y * Constants3D.Y_AXIS.y, 
										_tmpPosition.z * Constants3D.Z_AXIS.z ); 
		
		
		if ( target != null )
		{
			
			var tpos:Vector3D = target.transform.position;
			
			tpos.negate();
			
			_transform.append( Constants3D.lookAt( position, tpos ) );
			
		}
		else
		{
			
			var _tmpRotation:Vector3D = _rotation.clone();
			_tmpRotation.negate();
			
			_transform.appendRotation( _tmpRotation.x, Constants3D.X_AXIS);
			_transform.appendRotation( _tmpRotation.y, Constants3D.Y_AXIS);
			_transform.appendRotation( _tmpRotation.z, Constants3D.Z_AXIS);
			
		}
		
		return _transform;
		
	}
	
	/**
	 * LENS PROPERTIES
	 */
	
	private function get_fov():Float { return lens.fov; }
	private function set_fov( value:Float ):Float { lens.fov = value; geomDirty = true; return value; }
	public var fov( get_fov, set_fov ):Float;
	
	private function get_near():Float { return lens.near; }
	private function set_near( value:Float ):Float{ lens.near = value; geomDirty = true; return value; }
	public var near( get_near, set_near ):Float;
	
	private function get_far():Float { return lens.far; }
	private function set_far( value:Float ):Float{ lens.far = value; geomDirty = true; return value; }
	public var far( get_far, set_far ):Float;
	
	public function get_projection():Matrix3D{ return lens; }
	public var projection( get_projection, null ):Matrix3D;
	
	private function get_lens():Lens 
	{
		return _lens;
	}
	
	private function set_lens(value:Lens):Lens 
	{
		_lens = value;
		reset();
		return value;
	}
	
	public var lens(get_lens, set_lens):Lens;
	
	
	private function get_viewport():Viewport 
	{
		return _viewport;
	}
	
	private function set_viewport(value:Viewport):Viewport 
	{
		if( _viewport != null ) _viewport.removeEventListener( Event.RESIZE, onResize );
		_viewport = value;
		_viewport.addEventListener( Event.RESIZE, onResize );
		reset();
		return value;
	}
	
	public var viewport(get_viewport, set_viewport):Viewport;
	
	//reset
	private function reset() 
	{
		if ( lens != null )
		{
			lens.width = viewport.viewWidth;
			lens.height = viewport.viewHeight;
			lens.reset();
		}
	}
	
}