package h3d.core;
import h3d.utils.Constants3D;
import jeash.geom.Matrix;
import nme.geom.Matrix3D;
import nme.geom.Vector3D;
class Object3D
{
	private var _transform:Matrix3D;
	private var _position:Vector3D;
	private var _rotation:Vector3D;
	private var _scale:Vector3D;
	
	private var _parent:Object3D;
	private var _target:Object3D;
	
	private var geomDirty:Bool = true;
	
	public function new()
	{
		identity();
	}
	
	/**************************************
	
		TRANSFORMATIONS
	
	**************************************/
	
	private function identity():Void
	{
		
		_transform 	= new Matrix3D();
		_position 	= new Vector3D( 0, 0, 0 );
		_rotation 	= new Vector3D( 0, 0, 0 );
		_scale 		= new Vector3D( 1, 1, 1 );
		geomDirty = true;
		
	}
	
	private function get_transform():Matrix3D 
	{
		if ( geomDirty || target != null || parent != null  )
		{
			
			_transform = Constants3D.recompose( _scale, _rotation, _position, _transform );
			
			if ( parent != null )
			{
				_transform.append( parent.transform );
			}
			
			geomDirty = false;
			
		}
		
		return _transform;
	}
	
	private function set_transform(value:Matrix3D):Matrix3D 
	{
		_transform = value;
		geomDirty = false;
		return _transform;
	}
	public var transform( get_transform, set_transform ):Matrix3D;
	
	//POSITION
	
	private function get_position():Vector3D {	return _position;	}
	private function set_position(value:Vector3D):Vector3D
	{
		geomDirty = true;
		return _position = value;
	}
	public var position( get_position, set_position ):Vector3D;
	
	private function set_x(value:Float):Float {		_position.x = value;		geomDirty = true; return value;		}
	private function set_y(value:Float):Float {		_position.y = value;		geomDirty = true; return value;		}
	private function set_z(value:Float):Float {		_position.z = value;		geomDirty = true; return value;		}
	
	private function get_x():Float {		return _position.x;		}
	private function get_y():Float {		return _position.y;		}
	private function get_z():Float {		return _position.z;		}

	public var x(get_x, set_x):Float;
	public var y(get_y, set_y):Float;
	public var z(get_z, set_z):Float;
	
	//ROTATION
	
	private function get_rotation():Vector3D {	return _rotation;	}
	private function set_rotation(value:Vector3D):Vector3D 
	{
		geomDirty = true;
		return _rotation = value;
	}
	public var rotation( get_rotation, set_rotation ):Vector3D;
	
	private function set_rotationX(value:Float):Float {		_rotation.x = value;		geomDirty = true;	return value;	}
	private function set_rotationY(value:Float):Float {		_rotation.y = value;		geomDirty = true;	return value;	}
	private function set_rotationZ(value:Float):Float {		_rotation.z = value;		geomDirty = true;	return value;	}
	
	private function get_rotationX():Float {		return _rotation.x;		}
	private function get_rotationY():Float {		return _rotation.y;		}
	private function get_rotationZ():Float {		return _rotation.z;		}
	
	public var rotationX( get_rotationX, set_rotationX ):Float;
	public var rotationY( get_rotationY, set_rotationY ):Float;
	public var rotationZ( get_rotationZ, set_rotationZ ):Float;
	
	//SCALE
	
	private function get_scale():Vector3D {	return _scale;	}
	private function set_scale(value:Vector3D):Vector3D 
	{
		geomDirty = true;
		return _scale = value;
	}
	public var scale( get_rotation, set_rotation ):Vector3D;
	
	private function set_uniform_scale( value:Float ):Float {		_scale.x = _scale.y = _scale.z = value;		geomDirty = true;	return value;	}
	public var uniform_scale( null, set_uniform_scale ):Float;
	
	private function set_scaleX(value:Float):Float {		_scale.x = value;		geomDirty = true;	return value;	}
	private function set_scaleY(value:Float):Float {		_scale.y = value;		geomDirty = true;	return value;	}
	private function set_scaleZ(value:Float):Float {		_scale.z = value;		geomDirty = true;	return value;	}
	
	private function get_scaleX():Float {		return _scale.x;		}
	private function get_scaleY():Float {		return _scale.y;		}
	private function get_scaleZ():Float {		return _scale.z;		}
	
	public var scaleX( get_scaleX, set_scaleX ):Float;
	public var scaleY( get_scaleY, set_scaleY ):Float;
	public var scaleZ( get_scaleZ, set_scaleZ ):Float;
	
	
	private function get_target():Object3D{					return _target;					}
	private function set_target(value:Object3D):Object3D {	geomDirty = true; return _target = value;			}
	public var target(get_target, set_target):Object3D;
	
	private function get_parent():Object3D {				return _parent;					}	
	private function set_parent(value:Object3D):Object3D {	geomDirty = true; return _parent = value;			}	
	public var parent(get_parent, set_parent):Object3D;
	
}