package  h3d.core;

import h3d.camera.Camera;
import h3d.models.Model;
import h3d.renderer.IRenderer;
import h3d.renderer.WireFrameRenderer;
import h3d.utils.Constants3D;
import h3d.utils.PerspectiveMatrix3D;
import nme.display.BitmapData;
import nme.display.Sprite;
import nme.events.Event;
import nme.geom.Matrix3D;
import nme.geom.Vector3D;
import nme.geom.Rectangle;
import nme.Lib;
import nme.text.TextField;
import nme.text.TextFieldAutoSize;
import nme.text.TextFormat;

/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Scene extends Object3DContainer
{
	
	public var backgroundColor:Int = 0x606060;
	
	private var transform3D:Matrix3D;
	
	private var view:Matrix3D;
	
	private var _viewport:Viewport;
	
	private var _camera:Camera;
	
	private var _renderer:IRenderer;
	
	public function new( width:Float, height:Float ) 
	{
		super();
		
		transform3D = new Matrix3D();
		
		_viewport = new Viewport( width, height );
		
		_camera = new Camera( _viewport );
		
		_renderer = new WireFrameRenderer();
		
		_renderer.culling = Constants3D.CULLING_NEGATIVE;
		
	}
	
	//RENDER LOOP
	
	public function render():Void
	{
		viewport.clear();
		
		view = transform.clone();
		view.append( camera.transform );
		view.append( camera.lens );
		
		for( obj in children )
		{
			
			renderObject( obj, view );
			
		}
		
	}
	
	private function renderObject(obj:Object3D, view:Matrix3D ):Void
	{
		
		var model:Model	= cast( obj, Model );
		
		if ( !model.visible ) return;
		if ( model.indices == null ) return;
		if ( model.vertices == null ) return;
		
		transform3D.identity();
		transform3D.append( model.transform );
		transform3D.append( view );
		
		model.project( transform3D, viewport );
		
		renderer.render( viewport, model );
		
		if ( Std.is( obj, Object3DContainer ) )
		{
			for( subobj in cast( obj, Object3DContainer ).children )
			{
				renderObject( subobj, view );
			}
		}
		
	}
	
	//CAMERA
	
	private function get_camera():Camera 
	{
		return _camera;
	}
	
	private function set_camera(value:Camera):Camera 
	{
		return _camera = value;
	}
	
	public var camera( get_camera, set_camera ):Camera;
	
	//VIEWPORT 
	
	private function get_viewport():Viewport 
	{
		return _viewport;
	}
	public var viewport( get_viewport, null ):Viewport;
	
	//RENDERER
	
	private function get_renderer():IRenderer 
	{
		return _renderer;
	}
	
	private function set_renderer(value:IRenderer):IRenderer 
	{
		return _renderer = value;
	}
	
	public var renderer(get_renderer, set_renderer):IRenderer;
	
	
}