package h3d.models;
import h3d.core.Object3DContainer;
import h3d.core.Viewport;
import h3d.material.IMaterial;
import h3d.utils.Constants3D;
import nme.display.BitmapData;
import nme.display.TriangleCulling;
import nme.geom.Matrix3D;
import nme.geom.Point;
import nme.geom.Vector3D;
import nme.Vector;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Model extends Object3DContainer
{
	
	private var _vertices:Vector<Float>;
	private var _projection:Vector<Float>;
	
	private var _indices:Vector<Int>;
	private var _uvs:Vector<Float>;
	
	private var _normals:Vector<Float>;
	private var _normalProjection:Vector<Float>;
	
	private var _material:IMaterial;
	
	private var tmp:Vector<Float>;
	private var _visible:Bool = true;
	
	public function new( vertices:Vector<Float> = null, indices:Vector<Int> = null ) 
	{
		
		super();
		
		this.vertices = vertices;
		this.indices = indices;
		
		_projection = new Vector<Float>();
		tmp = new Vector<Float>();
		visible = true;
		
	}
	
	public function project( worldTransform:Matrix3D, viewport:Viewport ):Void
	{
		
		_projection = new Vector<Float>();
		worldTransform.transformVectors( vertices, tmp );
		
		if ( normals != null )
		{
			_normalProjection = new Vector<Float>();
			worldTransform.transformVectors( normals, normalProjection );
		}
		
		var size:Point = viewport.size;
		var w:Float = size.x;
		var h:Float = size.y;
		
		var w2:Float = size.x * .5;
		var h2:Float = size.y * .5;
		
		var i:Int = 0;
		while ( i < Std.int( tmp.length ) )
		{
			/*
			if ( 	tmp[ i ] < -1 || tmp[ i ] > 1
			||		tmp[ i + 1 ] < -1 || tmp[ i + 1 ] > 1
			)
			{
				_projection.push( Math.NaN );
				_projection.push( Math.NaN );
			}
			else
			{//*/
				_projection.push( w2 - ( tmp[ i ]      / tmp[ i + 2 ]  ) * w );
				_projection.push( h2 - ( tmp[ i + 1 ]  / tmp[ i + 2 ]  ) * h );
			//}
			
			i += 3;
			
		}
		
	}
	
	public function initFromArrays( verts:Array<Float>, inds:Array<Int> ):Void
	{
		
		vertices = new Vector<Float>();
		for ( f in verts )	vertices.push( f );
		
		indices = new Vector<Int>();
		for ( i in inds ) indices.push( i );
		
	}
	
	public function bakeTransform():Void
	{
		
		var v:Vector3D = new Vector3D();
		var i:Int = 0;
		while ( i < Std.int( vertices.length ) )
		{
			
			v.x = vertices[ i ];
			v.y = vertices[ i + 1 ];
			v.z = vertices[ i + 2 ];
			
			v = transform.transformVector( v );
			
			vertices[ i ] = v.x;
			vertices[ i + 1 ] = v.y;
			vertices[ i + 2 ] = v.z;
			
			i += 3;
		}
		identity();
		
		for ( c in children ) 
		{
			if ( Std.is( c, Model ) ) cast( c, Model ).bakeTransform();
		}
		
	}
	
	public function derivateNormals():Void
	{
		normals = new Vector<Float>();
		
		var i:Int = 0, id0:Int, id1:Int, id2:Int;
		
		var len:Float;
		var ax:Float, ay:Float, az:Float;
		var bx:Float, by:Float, bz:Float;
		var nx:Float, ny:Float, nz:Float;
		
		while( i < Std.int( indices.length ) )
		{
			
			id0 = indices[ i ];
			id1 = indices[ i + 1 ];
			id2 = indices[ i + 2 ];
			
			ax = vertices[ id0 ]	 - vertices[ id1 ];
			ay = vertices[ id0 + 1 ] - vertices[ id1 + 1 ];
			az = vertices[ id0 + 2 ] - vertices[ id1 + 2 ];
			
			bx = vertices[ id0 ]	 - vertices[ id2 ];
			by = vertices[ id0 + 1 ] - vertices[ id2 + 1 ];
			bz = vertices[ id0 + 2 ] - vertices[ id2 + 2 ];
			
			nx = ay * bz - az * by;
			ny = az * bx - ax * bz;
			nz = ax * by - ay * bx;
			
			len = 1 / Math.sqrt( nx * nx + ny * ny + nz * nz );
			nx *= len;
			ny *= len;
			nz *= len;
			
			normals.push( nx );
			normals.push( ny );
			normals.push( nz );
			
			i += 3;
			
		}
		trace( 'vl ' + vertices.length / 3 + '  nl ' + normals.length / 3 + ' il ' + indices.length / 3 );
	}
	
	/****************************************
		GEOMETRY
	****************************************/
		
	public function get_vertices():Vector<Float> {	return _vertices;	}
	public function set_vertices(value:Vector<Float>):Vector<Float> {	return _vertices = value;	}
	public var vertices( get_vertices, set_vertices ):Vector<Float>;
	
	public function get_indices():Vector<Int> {	return _indices;	}
	public function set_indices(value:Vector<Int>):Vector<Int> {		return _indices = value;	}
	public var indices( get_indices, set_indices ):Vector<Int>;
	
	public function get_uvs():Vector<Float> {		return _uvs;	}
	public function set_uvs(value:Vector<Float>):Vector<Float> {	return _uvs = value ;	}
	public var uvs( get_uvs, set_uvs ):Vector<Float>;
	
	private function get_normals():Vector<Float> {	return _normals;	}
	private function set_normals(value:Vector<Float>):Vector<Float> {	return _normals = value;	}
	public var normals(get_normals, set_normals):Vector<Float>;
	
	public function get_visible():Bool {	return _visible;	}
	public function set_visible(value:Bool):Bool { return _visible = value;		}
	public var visible( get_visible, set_visible ):Bool;
	
	private function set_projection(value:Vector<Float>):Vector<Float> {	return _projection = value;	}
	private function get_projection():Vector<Float> {	return _projection;	}
	public var projection(get_projection, set_projection):Vector<Float>;
	
	private function get_normalProjection():Vector<Float> {	return _normalProjection;	}
	private function set_normalProjection(value:Vector<Float>):Vector<Float> {	return _normalProjection = value;	}
	public var normalProjection(get_normalProjection, set_normalProjection):Vector<Float>;
	
	private function get_material():IMaterial {	return _material;		}
	private function set_material(value:IMaterial):IMaterial {	return _material = value;	}
	public var material(get_material, set_material):IMaterial;
	
}