package bigal.models.primitives 
{
	import bigal.Model;
	import bigal.Scene;
	import bigal.utils.AgalUtils;
	import flash.display3D.Context3D;
	import flash.geom.Point;
	import flash.geom.Vector3D;
	
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class Cylinder extends Model 
	{
		private var OPTIONS:Vector.<Number> = new Vector.<Number>( 4, true );
		private var _radius:Number;
		private var _sides:uint;
		private var _height:Number;
		private var _upAxis:Vector3D;
		private var _cap:Boolean;
		
		public function Cylinder( radius:Number = 100, sides:uint = 12, height:Number = 100, cap:Boolean = true, upAxis:Vector3D = null ) 
		{
			
			_radius = radius;
			_sides = sides;
			_height = height;
			_upAxis = upAxis || Vector3D.Y_AXIS;
			_cap = cap;
			
			resetGeom();
		}
		
		private function resetGeom():void 
		{
			
			var vs:Vector.<Number> = new Vector.<Number>();
			var ind:Vector.<uint> = new Vector.<uint>();
			
			var pi2:Number = Math.PI * 2;
			var angle:Number = 0, angleStep:Number = pi2 / sides;
			var id:int = 0, i0:uint, i1:uint, i2:uint, i3:uint;
			
			for (var i:int = 0; i < 360; i+=( 360 / sides ), angle += angleStep, id+=2 ) 
			{
				
				var p:Point = new Point( Math.cos( angle ), Math.sin( angle ) );
				
				switch( upAxis )
				{
					
					case Vector3D.X_AXIS:	vs.push( 	.5, p.y, p.x,	-.5, p.y, p.x );	break;
					case Vector3D.Y_AXIS:	vs.push( 	p.x, .5, p.y,	p.x, -.5, p.y );	break;
					case Vector3D.Z_AXIS:	vs.push( 	p.x, p.y, .5, 	p.x, p.y, -.5 );	break;
					
				}
				
				i0 = id;
				i1 = id + 1;
				i2 = ( id + 2 );
				i3 = ( id + 3 );
				
				if ( i3 > sides * 2 - 1 )
				{
					i2 %= sides;
					i3 %= sides;
				}
				
				ind.push( 	i0, i2, i3, i0, i3, i1	);
				
			}
			
			if ( cap )
			{
				i0 = vs.length / 3;
				i1 = vs.length / 3 + 1;
				switch( upAxis )
				{
					
					case Vector3D.X_AXIS:	vs.push( 	.5, 0, 0,	-.5, 0, 0 );	break;
					case Vector3D.Y_AXIS:	vs.push( 	0, .5, 0,	0, -.5, 0 );	break;
					case Vector3D.Z_AXIS:	vs.push( 	0, 0, .5, 	0, 0, -.5 );	break;
					
				}
				
				for ( i = 0; i < sides * 2 - 1; i+=2 )
				{
					ind.push( i0, i, i + 2 );
					ind.push( i1, i + 1, i + 3 );
				}
				
			}
			
			
			vertices = vs;
			indices = ind;
			
			vertexShader = 	
							
							"mul vt0, va0, vc4\n" 			+ // projection : applies the Model's transforms
							
							"nrm vt1.xyz, vt0.xyz\n" 			+//normalize
							
							"mul vt1.xy, vt0.xy, vc5.xy\n" 			+//rescale
							
							"add vt0.xyz, vt0.xyz, vt1.xyz\n" 		+
							
							"m44 op, vt0, vc0\n" 			+//finds the color of the final coordinate
							
							"nrm vt0.xyz, vt0.xyz \n"		+//normalize it to smooth
							"mov v0, vt0";
							
			fragmentShader = "mov oc, v0";
			
			resetShaderValues();
		}
		
		private function resetShaderValues():void 
		{
			
			OPTIONS[0] = OPTIONS[1] = OPTIONS[2] = radius;
			
			switch( upAxis )
			{
				
				case Vector3D.X_AXIS : 	OPTIONS[0] = height;		break;
				case Vector3D.Y_AXIS : 	OPTIONS[1] = height;		break;
				case Vector3D.Z_AXIS : 	OPTIONS[2] = height;		break;
				
			}
			OPTIONS[3] = 1;
			
		}
		
		override public function prepareContext(context:Context3D):void 
		{
			super.prepareContext(context);
			Scene.setVertexVectorConstant( 4, OPTIONS );
			
			var t:Vector.<Number> = AgalUtils.TIME;//[ time, 1 / time, cos( time ), sin( time ) ]
			//Scene.setVertexVectorConstant( 5, Vector.<Number>([ t[ 2 ], t[ 3 ], t[ 2 ] * t[ 3 ], 1 ] ) );
		}
		override public function clearContext(context:Context3D):void 
		{
			super.clearContext(context);
		}
		
		/*******************************
		
		affects variables
		
		*******************************/
		
		public function get radius():Number 
		{
			return _radius;
		}
		
		public function set radius(value:Number):void 
		{
			_radius = value;
			resetShaderValues();
		}
		
		public function get height():Number 
		{
			return _height;
		}
		
		public function set height(value:Number):void 
		{
			_height = value;
			resetShaderValues();
		}
		
		/*******************************
		
		affects geometry
		
		*******************************/
		
		public function get sides():uint 
		{
			return _sides;
		}
		
		public function set sides(value:uint):void 
		{
			_sides = value;
			resetGeom();
		}
		
		public function get upAxis():Vector3D 
		{
			return _upAxis;
		}
		
		public function set upAxis(value:Vector3D):void 
		{
			_upAxis = value;
			resetGeom();
		}
		
		public function get cap():Boolean 
		{
			return _cap;
		}
		
		public function set cap(value:Boolean):void 
		{
			_cap = value;
			resetGeom();
		}
	}

}