package bigal.models.primitives 
{
	import bigal.Model;
	/**
	 * 
	 * original by Keim At Si http://www.yomogi.sakura.ne.jp/~si/
	 * found on http://wonderfl.net/c/uCLR
	 * 
	 */
	public class SolidFactory 
	{
		static private var size:Number = .5;
		
		static private function createFace( indices:Vector.<uint>, i0:uint, i1:uint, i2:uint, i3:int = -1 ):Vector.<uint>
		{
			
			indices.push( i0, i1, i2 );
			if( i3 != -1 ) indices.push( i3, i2, i1);
			return indices;
			
		}
		
		static private function updateModel(model:Model, vertices:Vector.<Number>, indices:Vector.<uint>, scale:Number = 1 ):Model 
		{
			
			if ( model == null ) 
			{
				model = new Model( vertices, indices );
			}
			else 
			{
				model.vertices = vertices;
				model.indices = indices;
			}
			
			model.scale = scale;
			
			return model;
		}
		
		static public function plane( model:Model = null, scale:Number = 1 ) : Model 
		{
			
			var vertices:Vector.<Number> = Vector.<Number>([ 
															-size, size, 0, 
															size, size, 0, 
															size, -size, 0, 
															-size, -size, 0 
															]);
			var indices:Vector.<uint> = Vector.<uint>([  0,1,2, 0,2,3 ]);
			
			return updateModel( model, vertices, indices, scale );
			
		}
		
		static public function tetrahedron( model:Model = null, scale:Number = 1 ) : Model 
		{
			
			var vertices:Vector.<Number> = Vector.<Number>([size, size, size, size, -size, -size, -size, size, -size, -size, -size, size]);
			var indices:Vector.<uint> = Vector.<uint>([]);
			
			createFace( indices, 0, 2, 1, 3);
			createFace( indices, 1, 3, 0, 2);
			
			return updateModel( model, vertices, indices, scale );
			
		}
		
		
		static public function hexahedron(model:Model=null, scale:Number = 1 ) : Model 
		{
			
			var vertices:Vector.<Number> = Vector.<Number>([]);
			var indices:Vector.<uint> = Vector.<uint>([]);
			
			for (var i:int=0; i<8; i++) vertices.push((i&1)?size:-size, ((i>>1)&1)?size:-size, (i>>2)?size:-size);
			createFace(indices,0,1,2,3);createFace(indices,1,0,5,4);createFace(indices,0,2,4,6);
			createFace(indices,2,3,6,7);createFace(indices,3,1,7,5);createFace(indices,5,4,7,6);
			
			return updateModel( model, vertices, indices, scale );
			
		}
		
		static public function octahedron(model:Model=null, scale:Number = 1 ) : Model 
		{
			
			var vertices:Vector.<Number> = Vector.<Number>([]);
			var indices:Vector.<uint> = Vector.<uint>([]);
			
			vertices.push(0,0,-size, -size,0,0, 0,-size,0, size,0,0, 0,size,0, 0,0,size);
			createFace(indices,0,1,2,5);createFace(indices,0,2,3,5);createFace(indices,0,3,4,5);createFace(indices,0,4,1,5);
			
			return updateModel( model, vertices, indices, scale );
			
		}
		
		static public function dodecahedron(model:Model, scale:Number = 1 ) : Model 
		{
			
			var vertices:Vector.<Number> = Vector.<Number>([]);
			var indices:Vector.<uint> = Vector.<uint>([]);
						
			var a:Number=size*0.149071198, b:Number=size*0.241202266, c:Number=size*0.283550269, 
				d:Number=size*0.390273464, e:Number=size*0.458793973, f:Number=size*0.631475730, 
				g:Number=size*0.742344243;
			vertices.push(c,f,d, e,f,-a, 0,f,-b-b, -e,f,-a, -c,f,d);
			vertices.push(e,a,f, g,a,-b, 0,a,-d-d, -g,a,-b, -e,a,f);
			vertices.push(0,-a,d+d, g,-a,b, e,-a,-f, -e,-a,-f, -g,-a,b);
			vertices.push(0,-f,b+b, e,-f,a, c,-f,-d, -c,-f,-d, -e,-f,a);
			createFace(indices,0,3,1,2);createFace(indices,0,4,3);createFace(indices,4,5,9,10);createFace(indices,4,0,5);
			createFace(indices,0,6,5,11);createFace(indices,0,1,6);createFace(indices,1,7,6,12);createFace(indices,1,2,7);
			createFace(indices,2,8,7,13);createFace(indices,2,3,8);createFace(indices,3,9,8,14);createFace(indices,3,4,9);
			createFace(indices,17,11,12,6);createFace(indices,17,16,11);createFace(indices,16,10,11,5);createFace(indices,16,15,10);
			createFace(indices,15,14,10,9);createFace(indices,15,19,14);createFace(indices,19,13,14,8);createFace(indices,19,18,13);
			createFace(indices,18,12,13,7);createFace(indices,18,17,12);createFace(indices,16,18,15,19);createFace(indices,16,17,18);
			
			return updateModel( model, vertices, indices, scale );
			
		}
		
		static public function icosahedron(model:Model, scale:Number = 1 ) : Model
		{
			var vertices:Vector.<Number> = Vector.<Number>([]);
			var indices:Vector.<uint> = Vector.<uint>([]);
			
			var a:Number=size*0.276393202, b:Number=size*0.447213595, c:Number=size*0.525731112, 
				d:Number=size*0.723606798, e:Number=size*0.850650808;
			vertices.push(0,size,0, 0,b,b+b, e,b,a, c,b,-d, -c,b,-d, -e,b,a);
			vertices.push(e,-b,-a, c,-b,d, -c,-b,d, -e,-b,-a, 0,-b,-b-b, 0,-size,0);
			createFace(indices,0,2,1,7);createFace(indices,0,3,2,6);createFace(indices,0,4,3,10);createFace(indices,0,5,4,9);createFace(indices,0,1,5,8);
			createFace(indices,1,7,8,11);createFace(indices,2,6,7,11);createFace(indices,3,10,6,11);createFace(indices,4,9,10,11);createFace(indices,5,8,9,11);
			
			return updateModel( model, vertices, indices, scale );
			
		}
		
		static public function sphere(model:Model, scale:Number = 1 ) : Model
		{
			return icosahedron(model, scale);
		}
	}

}