package h3d.renderer;
import h3d.core.Object3DContainer;
import h3d.core.Viewport;
import h3d.models.Model;
import h3d.models.sparse.SparseModel;
import nme.display.GraphicsPath;
import nme.display.TriangleCulling;
import nme.geom.Matrix3D;
import nme.geom.Point;
import nme.Vector;
using nme.Vector;

/**
 * @author Nicolas Barradeau 
 */

class LineRenderer implements IRenderer
{
	private var projection:Vector<Float>;
	private var indices:Vector<Int>;
	
	private var _culling:TriangleCulling;
	
	public function new() 
	{
	}
	
	public function render( viewport:Viewport, model:Model ):Void
	{
		
		projection = model.projection;
		indices = model.indices;
		
		model.material.prepareContext( viewport );
		
		var colors:Vector<Int> = model.indices;
		if ( Std.is( model, SparseModel ) )
		{
			colors = cast( model, SparseModel ).colors;
		}
		
		var w2:Float = viewport.width * .5;
		var h2:Float = viewport.height * .5;
		
		var i:Int = 0, id1:Int, id2:Int;
		while( i < Std.int( indices.length ) )
		{
			
			id1 = Std.int( indices[ i ] 	* 2	);
			id2 = Std.int( indices[ i + 1 ] * 2	);
			
			
			viewport.graphics.lineStyle( 0, colors[ Std.int( i / 2 ) ] );
			viewport.graphics.moveTo( projection[ id1 ], projection[ id1 + 1 ] ); 
			viewport.graphics.lineTo( projection[ id2 ], projection[ id2 + 1 ] );
			
			i += 2;
		}
		
		model.material.clearContext( viewport );
		
	}
	
	//PROPERTIES
	private function get_culling():TriangleCulling 
	{
		return _culling;
	}
	
	private function set_culling(value:TriangleCulling):TriangleCulling 
	{
		return _culling = value;
	}
	
	public var culling(get_culling, set_culling):TriangleCulling;
	
}