package h3d.utils;
import nme.display.TriangleCulling;
import nme.geom.Matrix3D;
import h3d.core.Object3D;
import nme.geom.Vector3D;
import nme.Vector;
using nme.Vector;
/**
 * @author Nicolas Barradeau 
 */

class Constants3D 
{

	static public inline var EPSILON:Float = 0.00000001;
	
	static public inline var DEG_TO_RAD:Float = Math.PI / 180;
	static public inline var RAD_TO_DEG:Float = 180 / Math.PI;
	
	static public inline var ZERO:Vector3D = new Vector3D(0, 0, 0);
	static public inline var X_AXIS:Vector3D = new Vector3D(1, 0, 0);
	static public inline var Y_AXIS:Vector3D = new Vector3D(0, -1, 0);
	static public inline var Z_AXIS:Vector3D = new Vector3D(0, 0, 1);
	
	static public inline var CULLING_POSITIVE:TriangleCulling 	= TriangleCulling.POSITIVE;
	static public inline var CULLING_NONE:TriangleCulling 		= TriangleCulling.NONE;
	static public inline var CULLING_NEGATIVE:TriangleCulling 	= TriangleCulling.NEGATIVE;
	
	
	static public inline function vectorFromArray( a:Array<Float> ) :Vector<Float>
	{
		var v:Vector<Float> = new Vector<Float>();
		for ( f in a ) v.push( f );
		return v;
	}
	
	static public function lookAt( eye:Vector3D, at:Vector3D = null, up:Vector3D = null ):Matrix3D
	{
		
		if ( at == null ) at = Z_AXIS;
		if ( up == null ) up = Vector3D.Y_AXIS;
		
		var F:Vector3D = at.subtract( eye );
		F.normalize();
		
		var U:Vector3D = up.clone();
		U.normalize();

		var S:Vector3D = F.crossProduct( U );
		S.normalize();
		
		U = S.crossProduct( F );
		
		var m:Matrix3D = new Matrix3D( vectorFromArray(
		[ 
			S.x,  	S.y, 	S.z,  0,
			U.x,  	U.y, 	U.z,  0,
			-F.x, 	-F.y, 	-F.z, 0,
			0,0,0,1
		]));
		
		return m;
	}
	
	
	//http://code.google.com/p/nekonme/issues/detail?id=187
	static public function decompose( matrix:Matrix3D ):Vector<Vector3D>
	{
		var vec:Vector<Vector3D> = new Vector<Vector3D>();
		var m:Matrix3D = matrix.clone();
		var mr:Vector<Float> = m.rawData;
		
		var pos = new Vector3D(mr[12], mr[13], mr[14]);
		mr[12] = 0;
		mr[13] = 0;
		mr[14] = 0;
		
		
		var scale = new Vector3D();
		scale.x = Math.sqrt(mr[0]*mr[0]+mr[1]*mr[1]+mr[2]*mr[2]);
		scale.y = Math.sqrt(mr[4]*mr[4]+mr[5]*mr[5]+mr[6]*mr[6]);
		scale.z = Math.sqrt(mr[8]*mr[8]+mr[9]*mr[9]+mr[10]*mr[10]);
		
		if (mr[0]*(mr[5]*mr[10]-mr[6]*mr[9])-mr[1]*(mr[4]*mr[10]-mr[6]*mr[8])+mr[2]*(mr[4]*mr[9]-mr[5]*mr[8]) < 0) scale.z = -scale.z;
		
		mr[0] /= scale.x;
		mr[1] /= scale.x;
		mr[2] /= scale.x;
		mr[4] /= scale.y;
		mr[5] /= scale.y;
		mr[6] /= scale.y;
		mr[8] /= scale.z;
		mr[9] /= scale.z;
		mr[10] /= scale.z;
		
		
		var rot = new Vector3D();
		rot.y = Math.asin(-mr[2]);		
		var C = Math.cos(rot.y);		
		if (C > 0) {
			
			rot.x = Math.atan2(mr[6], mr[10]);
			rot.z = Math.atan2(mr[1], mr[0]);
		} 
		else
		{
			rot.z = 0;
			rot.x = Math.atan2(mr[4], mr[5]);
		}  
		
		vec.push(pos);
		vec.push(rot);
		vec.push(scale);
		
		return vec;
	}
	
	static public function recompose( scale:Vector3D, rotation:Vector3D, translation:Vector3D, matrix:Matrix3D = null ):Matrix3D
	{
		
		if ( matrix == null )
		{
			matrix = new Matrix3D();
		}
		else
		{
			matrix.identity();
		}
		
		matrix.appendScale( scale.x * X_AXIS.x, 
							scale.y * Y_AXIS.y, 
							scale.z * Z_AXIS.z ); 
		
		matrix.appendRotation( rotation.x, X_AXIS);
		matrix.appendRotation( rotation.y, Y_AXIS);
		matrix.appendRotation( rotation.z, Z_AXIS);
		
		matrix.appendTranslation( 	translation.x * X_AXIS.x, 
									translation.y * Y_AXIS.y, 
									translation.z * Z_AXIS.z ); 
		
		return matrix;
		
	}
	
	
}