/*
Copyright (c) 2011, Adobe Systems Incorporated
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the 
documentation and/or other materials provided with the distribution.

* Neither the name of Adobe Systems Incorporated nor the names of its 
contributors may be used to endorse or promote products derived from 
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package h3d.utils;

import h3d.utils.Constants3D;
import nme.geom.Matrix3D;
import nme.geom.Vector3D;
import nme.Vector;
using nme.Vector;

class PerspectiveMatrix3D extends Matrix3D
{
	public function new(v:Vector<Float>=null)
	{
		
		super( v );
		_x = new Vector3D();
		_y = new Vector3D();
		_z = new Vector3D();
		_w = new Vector3D();
		
	}
	
	private function customCopyFrom( to:Vector3D, from:Vector3D ):Void
	{
		to.x = from.x;
		to.y = from.y;
		to.z = from.z;
		to.w = from.w;
	}
	
	private function customCopyRowFrom(id:Int, from:Vector3D):Void 
	{
		
		rawData[ id * 4 	] 	=   from.x;
		rawData[ id * 4 + 1 ] 	=   from.y;
		rawData[ id * 4 + 2 ] 	=   from.z;
		rawData[ id * 4 + 3 ] 	=   from.w;
		
		//rawData[ Std.int( id / 4 ) 	] 	=   from.x;
		//rawData[ Std.int( id / 4 ) + 1 ] 	=   from.y;
		//rawData[ Std.int( id / 4 ) + 2 ] 	=   from.z;
		//rawData[ Std.int( id / 4 ) + 3 ] 	=   from.w;
		
	}
	
	public function lookAtLH(eye:Vector3D, at:Vector3D, up:Vector3D):Void
	{
		customCopyFrom(_z, at);
		
		trace( _z.equals( at ) );
		
		_z = _z.subtract(eye);
		_z.normalize();
		_z.w = 0.0;
		
		customCopyFrom(_x,up);
		_crossProductTo(_x,_z);
		
		trace( _x.equals( up.crossProduct( _z ) ) );
		
		_x = up.crossProduct( _z );
		_x.normalize();
		_x.w = 0.0;
		
		customCopyFrom(_y,_z);
		_crossProductTo(_y,_x);
		
		trace( _y.equals( _z.crossProduct( _x ) ) );

		_y = _z.crossProduct( _x );
		//_y.normalize();
		_y.w = 0.0;
		
		_w.x = _x.dotProduct(eye);
		_w.y = _y.dotProduct(eye);
		_w.z = _z.dotProduct(eye);
		_w.w = 1.0;
		
		trace( "x:" + _x );
		trace( "y:" + _y );
		trace( "z:" + _z );
		trace( "w:" + _w );
		
		//customCopyRowFrom(0,_x);
		//customCopyRowFrom(1,_y);
		//customCopyRowFrom(2,_z);
		//customCopyRowFrom(3,_w);
		
		rawData = Constants3D.vectorFromArray( [
												
												_x.x, _x.y, _x.z, 0,
												_y.x, _y.y, _y.z, 0,
												_z.x, _z.y, _z.z, 0,
												0, 0, 0, 1
												
												] );
		
	}

	public function lookAtRH(eye:Vector3D, at:Vector3D, up:Vector3D):Void
	{
		customCopyFrom(_z, eye);
		_z = _z.subtract(at);
		_z.normalize();
		_z.w = 0.0;
		
		customCopyFrom( _x,up );
		_crossProductTo(_x,_z);
		_x.normalize();
		_x.w = 0.0;
		
		customCopyFrom(_y,_z);
		_crossProductTo(_y,_x);
		_y.w = 0.0;
		
		_w.x = _x.dotProduct(eye);
		_w.y = _y.dotProduct(eye);
		_w.z = _z.dotProduct(eye);
		_w.w = 1.0;
		
		var dpx:Float = -_w.x;
		var dpy:Float = -_w.y;
		var dpz:Float = -_w.z;
		
		//customCopyRowFrom(0,_x);
		//customCopyRowFrom(1,_y);
		//customCopyRowFrom(2,_z);
		//customCopyRowFrom(3,_w);
		
		rawData = Constants3D.vectorFromArray( [
												
												_x.x * dpx, _x.y * dpy, _x.z * dpz, 0,
												_y.x * dpx, _y.y * dpy, _y.z * dpz, 0,
												_z.x * dpx, _z.y * dpy, _z.z * dpz, 0,
												0, 0, 0, 1
												
												] );
	}
	
	public function perspectiveLH(width:Float, 
								  height:Float, 
								  zNear:Float, 
								  zFar:Float):Void {
		rawData = Constants3D.vectorFromArray( 
		[
			2.0*zNear/width, 0.0, 0.0, 0.0,
			0.0, 2.0*zNear/height, 0.0, 0.0,
			0.0, 0.0, zFar/(zFar-zNear), 1.0,
			0.0, 0.0, zNear*zFar/(zNear-zFar), 0.0
		]);
	}

	public function perspectiveRH(width:Float, 
								  height:Float, 
								  zNear:Float, 
								  zFar:Float):Void {
		rawData = Constants3D.vectorFromArray 	([
			2.0*zNear/width, 0.0, 0.0, 0.0,
			0.0, 2.0*zNear/height, 0.0, 0.0,
			0.0, 0.0, zFar/(zNear-zFar), -1.0,
			0.0, 0.0, zNear*zFar/(zNear-zFar), 0.0
		]);
	}

	public function perspectiveFieldOfViewLH(fieldOfViewY:Float, 
											 aspectRatio:Float, 
											 zNear:Float, 
											 zFar:Float):Void {
		var yScale:Float = 1.0 / Math.tan(fieldOfViewY / 2.0);
		
		var xScale:Float = yScale / aspectRatio; 
		
		rawData = Constants3D.vectorFromArray 	(
		[
			xScale, 0.0, 0.0, 0.0,
			0.0, yScale, 0.0, 0.0,
			0.0, 0.0, zFar/(zFar-zNear), 1.0,
			0.0, 0.0, -(zNear*zFar)/(zNear-zFar), 0.0
		]);
		
	}

	public function perspectiveFieldOfViewRH(fieldOfViewY:Float, 
											 aspectRatio:Float, 
											 zNear:Float, 
											 zFar:Float):Void {
		var yScale:Float = 1.0/Math.tan(fieldOfViewY/2.0);
		var xScale:Float = yScale / aspectRatio; 
		rawData = Constants3D.vectorFromArray 	([
			xScale, 0.0, 0.0, 0.0,
			0.0, yScale, 0.0, 0.0,
			0.0, 0.0, zFar/(zNear-zFar), -1.0,
			0.0, 0.0, (zNear*zFar)/(zNear-zFar), 0.0
		]);
	}

	public function perspectiveOffCenterLH(left:Float, 
										   right:Float,
										   bottom:Float,
										   top:Float,
										   zNear:Float, 
										   zFar:Float):Void {
		rawData = Constants3D.vectorFromArray 	([
			2.0*zNear/(right-left), 0.0, 0.0, 0.0,
			0.0, -2.0*zNear/(bottom-top), 0.0, 0.0,
			-1.0-2.0*left/(right-left), 1.0+2.0*top/(bottom-top), -zFar/(zNear-zFar), 1.0,
			0.0, 0.0, (zNear*zFar)/(zNear-zFar), 0.0
		]);
	}

	public function perspectiveOffCenterRH(left:Float, 
										   right:Float,
										   bottom:Float,
										   top:Float,
										   zNear:Float, 
										   zFar:Float):Void {
		rawData = Constants3D.vectorFromArray 	([
			2.0*zNear/(right-left), 0.0, 0.0, 0.0,
			0.0, -2.0*zNear/(bottom-top), 0.0, 0.0,
			1.0+2.0*left/(right-left), -1.0-2.0*top/(bottom-top), zFar/(zNear-zFar), -1.0,
			0.0, 0.0, (zNear*zFar)/(zNear-zFar), 0.0
		]);
	}
	
	public function orthoLH(width:Float,
							height:Float,
							zNear:Float,
							zFar:Float):Void {
		rawData = Constants3D.vectorFromArray 	([
			2.0/width, 0.0, 0.0, 0.0,
			0.0, 2.0/height, 0.0, 0.0,
			0.0, 0.0, 1.0/(zFar-zNear), 0.0,
			0.0, 0.0, zNear/(zNear-zFar), 1.0
		]);
	}

	public function orthoRH(width:Float,
							height:Float,
							zNear:Float,
							zFar:Float):Void {
		rawData = Constants3D.vectorFromArray 	([
			2.0/width, 0.0, 0.0, 0.0,
			0.0, 2.0/height, 0.0, 0.0,
			0.0, 0.0, 1.0/(zNear-zNear), 0.0,
			0.0, 0.0, zNear/(zNear-zFar), 1.0
		]);
	}

	public function orthoOffCenterLH(left:Float, 
									 right:Float,
									 bottom:Float,
									 top:Float,
									 zNear:Float, 
									 zFar:Float):Void {
		rawData = Constants3D.vectorFromArray 	([
			2.0/(right-left), 0.0, 0.0, 0.0,
			0.0, 2.0*zNear/(top-bottom), 0.0, 0.0,
			-1.0-2.0*left/(right-left), 1.0+2.0*top/(bottom-top), 1.0/(zFar-zNear), 0.0,
			0.0, 0.0, zNear/(zNear-zFar), 1.0
		]);
	}

	public function orthoOffCenterRH(left:Float, 
									 right:Float,
									 bottom:Float,
									 top:Float,
									 zNear:Float, 
									 zFar:Float):Void {
		rawData = Constants3D.vectorFromArray 	([
			2.0/(right-left), 0.0, 0.0, 0.0,
			0.0, 2.0*zNear/(top-bottom), 0.0, 0.0,
			-1.0-2.0*left/(right-left), 1.0+2.0*top/(bottom-top), 1.0/(zNear-zFar), 0.0,
			0.0, 0.0, zNear/(zNear-zFar), 1.0
		]);
	}
	
	public function perspective(	fov:Float, 
									width:Float, 
									height:Float, 
									near:Float, 
									far:Float) :Void
	{
		
		/*
		var e:Float = 1 / Math.tan( fov * .5 );
		var f_add_n:Float = f + n;
		var f_min_n:Float = f - n;
		rawData = Constants3D.vectorFromArray 	([	
			e,		0,		0,		0,
			0,	  e,		0,		0,
			0,		0,		n/f,	0,
			0,		0,		0,		0
		] );
		//*/
		
		
		/*
		var focalLength:Float = Math.abs( width / 2 * ( Math.cos( fov / 2) / Math.sin( fov / 2 ) ) );
		var vMatrix:Vector<Vector3D> = new Vector<Vector3D>();
		vMatrix[0] = new Vector3D();// -projectionCenter.x, -projectionCenter.y);
		vMatrix[1] = new Vector3D();
		vMatrix[2] = new Vector3D(1+1/focalLength,1+1/focalLength,1);
		
		var matrix:Matrix3D = new Matrix3D();
		recompose(vMatrix);
		//*/
		
		var w:Float = ( 2 * near) / width;
		var h:Float = 1;// ( 2 * near) / height;
		var q:Float = far / (far - near);

		rawData = Constants3D.vectorFromArray 	([
		  w, 0, 0, 0,
		  0, h, 0, 0,
		  0, 0, q, 1,
		  0, 0, -(near * far) / (near - far), 0
		]);
		
		
	}

	private var _x:Vector3D;
	private var _y:Vector3D;
	private var _z:Vector3D;
	private var _w:Vector3D;
	
	private function _crossProductTo(a:Vector3D,b:Vector3D):Void
	{
		_w.x = a.y * b.z - a.z * b.y;
		_w.y = a.z * b.x - a.x * b.z;
		_w.z = a.x * b.y - a.y * b.x;
		_w.w = 1.0;
		customCopyFrom(a,_w);
	}
}

