package nape.phys;
import zpp_nape.ID;
import zpp_nape.Config;
import zpp_nape.Const;
import zpp_nape.dynamics.SpaceArbiterList;
import zpp_nape.dynamics.InteractionGroup;
import zpp_nape.dynamics.InteractionFilter;
import zpp_nape.dynamics.Contact;
import zpp_nape.space.SweepPhase;
import zpp_nape.dynamics.Arbiter;
import zpp_nape.space.DynAABBPhase;
import zpp_nape.space.Broadphase;
import zpp_nape.shape.Polygon;
import zpp_nape.shape.Circle;
import zpp_nape.shape.Edge;
import zpp_nape.shape.Shape;
import zpp_nape.util.Circular;
import zpp_nape.util.Flags;
import zpp_nape.util.Names;
import zpp_nape.space.Space;
import zpp_nape.util.HaxeMacros;
import zpp_nape.util.FastHash;
import zpp_nape.util.Array2;
import zpp_nape.util.Queue;
import zpp_nape.util.RBTree;
import zpp_nape.util.WrapLists;
import zpp_nape.util.Debug;
import zpp_nape.util.Math;
import zpp_nape.util.UserData;
import zpp_nape.util.DisjointSetForest;
import zpp_nape.util.Pool;
import zpp_nape.geom.MatMath;
import zpp_nape.geom.Vec3;
import zpp_nape.geom.Mat23;
import zpp_nape.geom.Convex;
import zpp_nape.util.Lists;
import zpp_nape.geom.Simple;
import zpp_nape.geom.Triangular;
import zpp_nape.geom.Geom;
import zpp_nape.geom.VecMath;
import zpp_nape.geom.PartitionedPoly;
import zpp_nape.geom.Cutter;
import zpp_nape.geom.Simplify;
import zpp_nape.geom.Ray;
import zpp_nape.geom.Distance;
import zpp_nape.geom.Monotone;
import zpp_nape.geom.PolyIter;
import zpp_nape.geom.AABB;
import zpp_nape.geom.MarchingSquares;
import zpp_nape.geom.Vec2;
import zpp_nape.geom.MatMN;
import zpp_nape.geom.GeomPoly;
import zpp_nape.callbacks.CbSetPair;
import zpp_nape.callbacks.CbSet;
import zpp_nape.geom.Collide;
import zpp_nape.callbacks.Callback;
import zpp_nape.callbacks.Listener;
import zpp_nape.callbacks.CbType;
import zpp_nape.phys.FluidProperties;
import zpp_nape.callbacks.OptionType;
import zpp_nape.phys.FeatureMix;
import zpp_nape.phys.Compound;
import zpp_nape.phys.Interactor;
import zpp_nape.phys.Material;
import zpp_nape.constraint.DistanceJoint;
import zpp_nape.phys.Body;
import zpp_nape.constraint.WeldJoint;
import zpp_nape.constraint.MotorJoint;
import zpp_nape.constraint.Constraint;
import zpp_nape.constraint.AngleJoint;
import zpp_nape.constraint.LinearJoint;
import zpp_nape.constraint.UserConstraint;
import zpp_nape.constraint.LineJoint;
import nape.dynamics.FluidArbiter;
import nape.dynamics.ContactList;
import zpp_nape.constraint.PivotJoint;
import nape.dynamics.Arbiter;
import nape.dynamics.InteractionGroupList;
import nape.dynamics.ArbiterType;
import nape.dynamics.ArbiterIterator;
import nape.dynamics.ContactIterator;
import nape.dynamics.InteractionGroupIterator;
import nape.dynamics.InteractionGroup;
import nape.dynamics.InteractionFilter;
import nape.dynamics.ArbiterList;
import nape.dynamics.CollisionArbiter;
import nape.dynamics.Contact;
import nape.space.Broadphase;
import nape.shape.ShapeType;
import nape.shape.ShapeList;
import nape.space.Space;
import nape.shape.Polygon;
import nape.shape.EdgeList;
import nape.shape.ShapeIterator;
import nape.shape.Circle;
import nape.shape.Edge;
import nape.shape.EdgeIterator;
import nape.shape.Shape;
import nape.util.Debug;
import nape.util.BitmapDebug;
import nape.geom.GeomPolyIterator;
import nape.util.ShapeDebug;
import nape.geom.Vec3;
import nape.geom.RayResultIterator;
import nape.geom.GeomPolyList;
import nape.geom.Winding;
import nape.geom.RayResultList;
import nape.geom.Geom;
import nape.geom.Vec2Iterator;
import nape.geom.GeomVertexIterator;
import nape.geom.Mat23;
import nape.geom.Ray;
import nape.geom.RayResult;
import nape.geom.MarchingSquares;
import nape.geom.AABB;
import nape.geom.MatMN;
import nape.geom.Vec2;
import nape.geom.Vec2List;
import nape.callbacks.ListenerList;
import nape.callbacks.ListenerType;
import nape.callbacks.InteractionType;
import nape.callbacks.ConstraintCallback;
import nape.callbacks.CbTypeIterator;
import nape.callbacks.BodyCallback;
import nape.geom.GeomPoly;
import nape.callbacks.Callback;
import nape.callbacks.OptionType;
import nape.callbacks.ListenerIterator;
import nape.callbacks.PreFlag;
import nape.callbacks.Listener;
import nape.callbacks.PreCallback;
import nape.callbacks.CbType;
import nape.callbacks.ConstraintListener;
import nape.callbacks.PreListener;
import nape.callbacks.InteractionCallback;
import nape.callbacks.BodyListener;
import nape.callbacks.CbEvent;
import nape.callbacks.CbTypeList;
import nape.phys.InertiaMode;
import nape.phys.FluidProperties;
import nape.phys.CompoundIterator;
import nape.phys.BodyType;
import nape.callbacks.InteractionListener;
import nape.phys.BodyList;
import nape.phys.GravMassMode;
import nape.phys.MassMode;
import nape.phys.Interactor;
import nape.phys.CompoundList;
import nape.phys.InteractorIterator;
import nape.phys.Material;
import nape.phys.InteractorList;
import nape.phys.BodyIterator;
import nape.constraint.DistanceJoint;
import nape.constraint.WeldJoint;
import nape.constraint.MotorJoint;
import nape.constraint.Constraint;
import nape.constraint.ConstraintIterator;
import nape.constraint.AngleJoint;
import nape.constraint.ConstraintList;
import nape.constraint.LinearJoint;
import nape.constraint.UserConstraint;
import nape.phys.Body;
import nape.constraint.LineJoint;
import nape.constraint.PivotJoint;
#if nape_swc@:keep#end
 class Compound extends Interactor{
    public var zpp_inner:ZPP_Compound;
    public var bodies(get_bodies,never):BodyList;
    inline function get_bodies():BodyList return zpp_inner.wrap_bodies public var constraints(get_constraints,never):ConstraintList;
    inline function get_constraints():ConstraintList return zpp_inner.wrap_constraints public var compounds(get_compounds,never):CompoundList;
    inline function get_compounds():CompoundList return zpp_inner.wrap_compounds public var compound(get_compound,set_compound):Compound;
    inline function get_compound():Compound{
        return if(zpp_inner.compound==null)null else zpp_inner.compound.outer;
    }
    inline function set_compound(compound:Compound):Compound{
        {
            zpp_inner.immutable_midstep("Compound::compound");
            if(this.compound!=compound){
                if(this.compound!=null)this.compound.compounds.remove(this);
                if(compound!=null)compound.compounds.add(this);
            }
        }
        return this.compound;
    }
    public var space(get_space,set_space):Space;
    inline function get_space():Space{
        return if(zpp_inner.space==null)null else zpp_inner.space.outer;
    }
    inline function set_space(space:Space):Space{
        {
            #if(!NAPE_RELEASE_BUILD)if(zpp_inner.compound!=null)throw "Error: Cannot set the space of an inner Compound, only the root Compound space can be set";
            #end
 zpp_inner.immutable_midstep("Compound::space");
            if(this.space!=space){
                if(this.space!=null)this.space.compounds.remove(this);
                if(space!=null)space.compounds.add(this);
            }
        }
        return this.space;
    }
    public function new(){
        #if(!NAPE_RELEASE_BUILD)try{
            super();
        }
        catch(e:Dynamic){}
        #end
#if NAPE_RELEASE_BUILD super();
        #end
 zpp_inner=new ZPP_Compound();
        zpp_inner.outer=this;
        zpp_inner.outer_i=this;
        zpp_inner_i=zpp_inner;
        zpp_inner.insert_cbtype(CbType.ANY_COMPOUND.zpp_inner);
    }
    @:keep public override function toString(){
        return "Compound#"+id;
    }
    public function copy(){
        return zpp_inner.copy();
    }
    public function breakApart(){
        zpp_inner.breakApart();
    }
    public function visitBodies(lambda:Body->Void){
        #if(!NAPE_RELEASE_BUILD)if(lambda==null)throw "Error: lambda cannot be null for Compound::visitBodies";
        #end
 for(b in bodies)lambda(b);
        for(c in compounds)c.visitBodies(lambda);
    }
    public function visitConstraints(lambda:Constraint->Void){
        #if(!NAPE_RELEASE_BUILD)if(lambda==null)throw "Error: lambda cannot be null for Compound::visitConstraints";
        #end
 for(c in constraints)lambda(c);
        for(c in compounds)c.visitConstraints(lambda);
    }
    public function visitCompounds(lambda:Compound->Void){
        #if(!NAPE_RELEASE_BUILD)if(lambda==null)throw "Error: lambda cannot be null for Compound::visitConstraints";
        #end
 for(c in compounds){
            lambda(c);
            c.visitCompounds(lambda);
        }
    }
    public function COM(){
        var ret=new Vec2();
        var total=0.0;
        visitBodies(function(b){
            ret.addeq(b.worldCOM.mul(b.mass,true));
            total+=b.mass;
        });
        #if(!NAPE_RELEASE_BUILD)if(total==0.0)throw "Error: COM of an empty Compound is undefined silly";
        #end
 ret.muleq(1/total);
        return ret;
    }
    public function translate(translation:Vec2){
        {
            #if(!NAPE_RELEASE_BUILD)if(translation!=null&&translation.zpp_disp)throw "Error: "+"Vec2"+" has been disposed and cannot be used!";
            #end

        };
        #if(!NAPE_RELEASE_BUILD)if(translation==null)throw "Error: Cannot translate by null Vec2";
        #end
 var weak=translation.zpp_inner.weak;
        translation.zpp_inner.weak=false;
        visitBodies(function(b)b.position.addeq(translation));
        translation.zpp_inner.weak=weak;
        ({
            if(({
                translation.zpp_inner.weak;
            })){
                translation.dispose();
                true;
            }
            else{
                false;
            }
        });
    }
    public function rotate(centre:Vec2,angle:Float){
        {
            #if(!NAPE_RELEASE_BUILD)if(centre!=null&&centre.zpp_disp)throw "Error: "+"Vec2"+" has been disposed and cannot be used!";
            #end

        };
        #if(!NAPE_RELEASE_BUILD)if(centre==null)throw "Error: Cannot rotate about a null Vec2";
        #end
#if(!NAPE_RELEASE_BUILD)if((angle!=angle))throw "Error: Cannot rotate by NaN radians";
        #end
 var weak=centre.zpp_inner.weak;
        centre.zpp_inner.weak=false;
        visitBodies(function(b)b.rotate(centre,angle));
        centre.zpp_inner.weak=weak;
        ({
            if(({
                centre.zpp_inner.weak;
            })){
                centre.dispose();
                true;
            }
            else{
                false;
            }
        });
    }
}
