package nape.phys;
import zpp_nape.ID;
import zpp_nape.Config;
import zpp_nape.Const;
import zpp_nape.dynamics.SpaceArbiterList;
import zpp_nape.dynamics.InteractionGroup;
import zpp_nape.dynamics.InteractionFilter;
import zpp_nape.dynamics.Contact;
import zpp_nape.space.SweepPhase;
import zpp_nape.dynamics.Arbiter;
import zpp_nape.space.DynAABBPhase;
import zpp_nape.space.Broadphase;
import zpp_nape.shape.Polygon;
import zpp_nape.shape.Circle;
import zpp_nape.shape.Edge;
import zpp_nape.shape.Shape;
import zpp_nape.util.Circular;
import zpp_nape.util.Flags;
import zpp_nape.util.Names;
import zpp_nape.space.Space;
import zpp_nape.util.HaxeMacros;
import zpp_nape.util.FastHash;
import zpp_nape.util.Array2;
import zpp_nape.util.Queue;
import zpp_nape.util.RBTree;
import zpp_nape.util.WrapLists;
import zpp_nape.util.Debug;
import zpp_nape.util.Math;
import zpp_nape.util.UserData;
import zpp_nape.util.DisjointSetForest;
import zpp_nape.util.Pool;
import zpp_nape.geom.MatMath;
import zpp_nape.geom.Vec3;
import zpp_nape.geom.Mat23;
import zpp_nape.geom.Convex;
import zpp_nape.util.Lists;
import zpp_nape.geom.Simple;
import zpp_nape.geom.Triangular;
import zpp_nape.geom.Geom;
import zpp_nape.geom.VecMath;
import zpp_nape.geom.PartitionedPoly;
import zpp_nape.geom.Cutter;
import zpp_nape.geom.Simplify;
import zpp_nape.geom.Ray;
import zpp_nape.geom.Distance;
import zpp_nape.geom.Monotone;
import zpp_nape.geom.PolyIter;
import zpp_nape.geom.AABB;
import zpp_nape.geom.MarchingSquares;
import zpp_nape.geom.Vec2;
import zpp_nape.geom.MatMN;
import zpp_nape.geom.GeomPoly;
import zpp_nape.callbacks.CbSetPair;
import zpp_nape.callbacks.CbSet;
import zpp_nape.geom.Collide;
import zpp_nape.callbacks.Callback;
import zpp_nape.callbacks.Listener;
import zpp_nape.callbacks.CbType;
import zpp_nape.phys.FluidProperties;
import zpp_nape.callbacks.OptionType;
import zpp_nape.phys.FeatureMix;
import zpp_nape.phys.Compound;
import zpp_nape.phys.Interactor;
import zpp_nape.phys.Material;
import zpp_nape.constraint.DistanceJoint;
import zpp_nape.phys.Body;
import zpp_nape.constraint.WeldJoint;
import zpp_nape.constraint.MotorJoint;
import zpp_nape.constraint.Constraint;
import zpp_nape.constraint.AngleJoint;
import zpp_nape.constraint.LinearJoint;
import zpp_nape.constraint.UserConstraint;
import zpp_nape.constraint.LineJoint;
import nape.dynamics.FluidArbiter;
import nape.dynamics.ContactList;
import zpp_nape.constraint.PivotJoint;
import nape.dynamics.Arbiter;
import nape.dynamics.InteractionGroupList;
import nape.dynamics.ArbiterType;
import nape.dynamics.ArbiterIterator;
import nape.dynamics.ContactIterator;
import nape.dynamics.InteractionGroupIterator;
import nape.dynamics.InteractionGroup;
import nape.dynamics.InteractionFilter;
import nape.dynamics.ArbiterList;
import nape.dynamics.CollisionArbiter;
import nape.dynamics.Contact;
import nape.space.Broadphase;
import nape.shape.ShapeType;
import nape.shape.ShapeList;
import nape.space.Space;
import nape.shape.Polygon;
import nape.shape.EdgeList;
import nape.shape.ShapeIterator;
import nape.shape.Circle;
import nape.shape.Edge;
import nape.shape.EdgeIterator;
import nape.shape.Shape;
import nape.util.Debug;
import nape.util.BitmapDebug;
import nape.geom.GeomPolyIterator;
import nape.util.ShapeDebug;
import nape.geom.Vec3;
import nape.geom.RayResultIterator;
import nape.geom.GeomPolyList;
import nape.geom.Winding;
import nape.geom.RayResultList;
import nape.geom.Geom;
import nape.geom.Vec2Iterator;
import nape.geom.GeomVertexIterator;
import nape.geom.Mat23;
import nape.geom.Ray;
import nape.geom.RayResult;
import nape.geom.MarchingSquares;
import nape.geom.AABB;
import nape.geom.MatMN;
import nape.geom.Vec2;
import nape.geom.Vec2List;
import nape.callbacks.ListenerList;
import nape.callbacks.ListenerType;
import nape.callbacks.InteractionType;
import nape.callbacks.ConstraintCallback;
import nape.callbacks.CbTypeIterator;
import nape.callbacks.BodyCallback;
import nape.geom.GeomPoly;
import nape.callbacks.Callback;
import nape.callbacks.OptionType;
import nape.callbacks.ListenerIterator;
import nape.callbacks.PreFlag;
import nape.callbacks.Listener;
import nape.callbacks.PreCallback;
import nape.callbacks.CbType;
import nape.callbacks.ConstraintListener;
import nape.callbacks.PreListener;
import nape.callbacks.InteractionCallback;
import nape.callbacks.BodyListener;
import nape.callbacks.CbEvent;
import nape.callbacks.CbTypeList;
import nape.phys.InertiaMode;
import nape.phys.CompoundIterator;
import nape.phys.BodyType;
import nape.callbacks.InteractionListener;
import nape.phys.BodyList;
import nape.phys.GravMassMode;
import nape.phys.MassMode;
import nape.phys.Interactor;
import nape.phys.Compound;
import nape.phys.CompoundList;
import nape.phys.InteractorIterator;
import nape.phys.Material;
import nape.phys.InteractorList;
import nape.phys.BodyIterator;
import nape.constraint.DistanceJoint;
import nape.constraint.WeldJoint;
import nape.constraint.MotorJoint;
import nape.constraint.Constraint;
import nape.constraint.ConstraintIterator;
import nape.constraint.AngleJoint;
import nape.constraint.ConstraintList;
import nape.constraint.LinearJoint;
import nape.constraint.UserConstraint;
import nape.phys.Body;
import nape.constraint.LineJoint;
import nape.constraint.PivotJoint;
#if nape_swc@:keep#end
 class FluidProperties{
    public var zpp_inner:ZPP_FluidProperties;
    public var userData(get_userData,never):Dynamic<Dynamic>;
    inline function get_userData():Dynamic<Dynamic>{
        if(zpp_inner.userData==null){
            zpp_inner.userData=cast{};
        }
        return zpp_inner.userData;
    }
    public var shapes(get_shapes,never):ShapeList;
    inline function get_shapes():ShapeList{
        if(zpp_inner.wrap_shapes==null)zpp_inner.wrap_shapes=ZPP_ShapeList.get(zpp_inner.shapes,true);
        return zpp_inner.wrap_shapes;
    }
    public function new(density:Float=1,viscosity:Float=1){
        {
            if(ZPP_FluidProperties.zpp_pool==null){
                zpp_inner=new ZPP_FluidProperties();
                #if NAPE_POOL_STATS ZPP_FluidProperties.POOL_TOT++;
                ZPP_FluidProperties.POOL_ADDNEW++;
                #end

            }
            else{
                zpp_inner=ZPP_FluidProperties.zpp_pool;
                ZPP_FluidProperties.zpp_pool=zpp_inner.next;
                zpp_inner.next=null;
                #if NAPE_POOL_STATS ZPP_FluidProperties.POOL_CNT--;
                ZPP_FluidProperties.POOL_ADD++;
                #end

            }
            zpp_inner.alloc();
        };
        zpp_inner.outer=this;
        this.density=density;
        this.viscosity=viscosity;
    }
    #if nape_swc@:keep#end
 public function copy(){
        var ret=new FluidProperties(density,viscosity);
        ret.zpp_inner.userData=zpp_inner.userData;
        return ret;
    }
    public var density(get_density,set_density):Float;
    inline function get_density():Float{
        return zpp_inner.density*("density"=="density"?1000:1);
    }
    inline function set_density(density:Float):Float{
        {
            if(density!=this.density){
                #if(!NAPE_RELEASE_BUILD)if((density!=density))throw "Error: FluidProperties::"+"density"+" cannot be NaN";
                if("density"!="density"&&density<0)throw "Error: FluidProperties::"+"density"+" ("+density+") must be >= 0";
                #end
 zpp_inner.density=density/("density"=="density"?1000:1);
                zpp_inner.invalidate();
            }
        }
        return this.density;
    }
    public var viscosity(get_viscosity,set_viscosity):Float;
    inline function get_viscosity():Float{
        return zpp_inner.viscosity*("viscosity"=="density"?1000:1);
    }
    inline function set_viscosity(viscosity:Float):Float{
        {
            if(viscosity!=this.viscosity){
                #if(!NAPE_RELEASE_BUILD)if((viscosity!=viscosity))throw "Error: FluidProperties::"+"viscosity"+" cannot be NaN";
                if("viscosity"!="density"&&viscosity<0)throw "Error: FluidProperties::"+"viscosity"+" ("+viscosity+") must be >= 0";
                #end
 zpp_inner.viscosity=viscosity/("viscosity"=="density"?1000:1);
                zpp_inner.invalidate();
            }
        }
        return this.viscosity;
    }
    public var gravity(get_gravity,set_gravity):Vec2;
    inline function get_gravity():Vec2{
        return zpp_inner.wrap_gravity;
    }
    inline function set_gravity(gravity:Vec2):Vec2{
        {
            if(gravity==null){
                if(zpp_inner.wrap_gravity!=null){
                    zpp_inner.wrap_gravity.zpp_inner._inuse=false;
                    zpp_inner.wrap_gravity.dispose();
                    zpp_inner.wrap_gravity=null;
                }
            }
            else{
                {
                    #if(!NAPE_RELEASE_BUILD)if(gravity!=null&&gravity.zpp_disp)throw "Error: "+"Vec2"+" has been disposed and cannot be used!";
                    #end

                };
                if(zpp_inner.wrap_gravity==null)zpp_inner.getgravity();
                this.gravity.set(gravity);
            }
        }
        return this.gravity;
    }
    @:keep public function toString(){
        return "{ density: "+density+" viscosity: "+viscosity+" }";
    }
}
