package triga.halfedge;
import nme.Vector;
using nme.Vector;
import nme.display.Graphics;
import nme.display.Shape;
import nme.events.EventDispatcher;
import nme.geom.Point;
import nme.utils.Dictionary;
import nme.utils.setTimeout;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class HEMesh extends EventDispatcher
{
	static public var UNIFY_NORMALS:Bool = false;
	static public var SYNCHRONOUS_FACE_BUILD:Bool = false;
	static public var SYNCHRONOUS_PATCH_FIND:Bool = false;
	static private var meshes:Dictionary = new Dictionary( true );
	static private var vertices:Dictionary = new Dictionary( true );
	static private var indices:Dictionary = new Dictionary( true );
	static private var uvs:Dictionary = new Dictionary( true );
	private var _faces:Vector<HEFace>;
	private var asyncUpdate:Function;
	private var asyncCallBack:Function;
	private var patches:Vector<HEPatch>;
	private var tmpFaces:Vector<HEFace>;
	public var asyncFaceId:Int;
	static public var USE_NORMALS:Bool = false;
	public function new( vertices:Vector<Float>, indices:Vector<Int>, uvs:Vector<Float> = null, unify_normals:Bool = false ) 
	{
		HEMesh.meshes[ this ] = this;
		HEMesh.vertices[ this ] = vertices;
		HEMesh.indices[ this ] = indices;
		if ( uvs != null ) HEMesh.uvs[ this ] = uvs;
		UNIFY_NORMALS = unify_normals;
	}
	public function buildFaces( synchronous:Bool = true ):Void 
	{
		faces = new Vector<HEFace>();
		var indices:Vector<Int> = indices[ this ];
		for ( i in 0 ... indices.length; i += 3 )
		{
			var f:HEFace = new HEFace( indices[ i ], indices[ ( i + 1 ) ], indices[ ( i + 2 ) ] );
			f.id = ( i / 3 );
			faces.push( f );
		}
		SYNCHRONOUS_FACE_BUILD = synchronous;
		findContiguousFaces();
	}
	public function unifyNormals( faces:Vector<HEFace> = null ):Void 
	{
		if ( faces == null ) faces = this.faces;
		var i:Int, j:Int;
		var f:HEFace, _f:HEFace;
		var e:HEEdge, _e:HEEdge, __e:HEEdge;
		for ( i in 0... faces.length ) 
		{
			f = faces[ i ];
			for ( j = i + 1; j < faces.length )
			{
				_f = faces[ j ];
				for( e in f.edges )
				{
					e.pair = null;
					for( _e in _f.edges )
					{
						_e.pair = null;
						if ( e.isPairFlipped( _e ) )
						{
							_f.flipNormals();
						}
					}	
				}
			}
		}
	}
	public function findContiguousFaces():Void 
	{
		//ok for small meshes
		if ( SYNCHRONOUS_FACE_BUILD )
		{
			var i:Int, j:Int;
			var f:HEFace, _f:HEFace;
			var e:HEEdge, _e:HEEdge, __e:HEEdge;
			for ( i in 0... faces.length ) 
			{
				f = faces[ i ];
				search : for( e in f.edges )
				{
					if ( e.pair != null ) break search;
					for ( j = i + 1; j < faces.length )
					{
						_f = faces[ j ];
						for( _e in _f.edges )
						{
							if ( e.isPair( _e ) || ( USE_NORMALS && e.isPairFlipped( _e ) ) )
							{
								e.pair = _e;
								f.contiguous.push( _f );
								_f.contiguous.push( f );
							}
						}	
					}
				}
			}
			dispatchEvent( new HEEvent( HEEvent.ON_CONTIGUOUS_FACE_COMPLETE, null, new HEPatch( this, faces ) ) );
		}
		else
		{
			asyncFaceId = 0;
			asyncFindContiguousFaces();
		}
	}
	private function asyncFindContiguousFaces():Void 
	{
		var i:Int = 10;
		while( i-- > 0 )
		{
			if ( asyncFaceId < faces.length-1 )
			{
				getContiguousFaces( asyncFaceId, faces[ asyncFaceId ] );
				dispatchEvent( new HEEvent( HEEvent.ON_CONTIGUOUS_FACE_UPDATE, faces[ asyncFaceId ] ) );
				asyncFaceId++;
			}
			else
			{
				dispatchEvent( new HEEvent( HEEvent.ON_CONTIGUOUS_FACE_COMPLETE, null, new HEPatch( this, faces ) ) );
				return;
			}
		}
		setTimeout( asyncFindContiguousFaces, 0 );
	}
	private function getContiguousFaces( start:Int, f:HEFace ):Void
	{
		var i:Int;
		var _f:HEFace;
		var e:HEEdge, _e:HEEdge;
		for ( i = start + 1; i < faces.length )
		{
			_f = faces[ i ];
			search : for( e in f.edges )
			{
				if ( e.pair != null ) break search;
				for( _e in _f.edges )
				{
					if ( e.isPair( _e ) || ( USE_NORMALS && e.isPairFlipped( _e ) ) )
					{
						e.pair = _e;
						f.contiguous.push( _f );
						_f.contiguous.push( f );
					}
				}	
			}
		}
	}
	public function getBorder( faces:Vector<HEFace> ):Vector<HEEdge>
	{
		var border:Vector<HEEdge> = new Vector<HEEdge>();
		for( var f:HEFace in faces )
		{
			for( var e:HEEdge in f.edges ) if ( e.pair == null ) border.push( e );
		}
		return border;
	}
	public function getBorderPacth( faces:Vector<HEFace> ):Vector<HEFace>
	{
		var border:Vector<HEFace> = new Vector<HEFace>();
		for( var f:HEFace in faces )
		{
			for( var e:HEEdge in f.edges ) if ( e.pair == null ) border.push( e.face );
		}
		return border;
	}
	public function getLinks( faces:Vector<HEFace> ):Vector<HEEdge>
	{
		var links:Vector<HEEdge> = new Vector<HEEdge>();
		for( var f:HEFace in faces )
		{
			for( var e:HEEdge in f.edges ) if ( e.pair != null ) links.push( e );
		}
		return links;
	}
	/***********************************
	find patches
	***********************************/
	public function findPatches( synchronous:Bool = true, useNormals:Bool = false ):Vector<HEPatch>
	{
		for( var face:HEFace in faces ) face.visited = false;
		USE_NORMALS = useNormals;
		patches = new Vector<HEPatch>();
		tmpFaces = faces.copy();
		if ( synchronous )
		{
			var patches:Vector<HEPatch> = getPatch( tmpFaces, patches );
			dispatchEvent( new HEEvent( HEEvent.ON_PATCH_FINDING_COMPLETE, null, null, patches ) );
			return patches;
		}
		else
		{
			asyncGetPatch( tmpFaces, patches );
			return null;
		}
	}
	private function getNextPatch():Int
	{
		var i:Int = 0;
		while ( i < tmpFaces.length ) 
		{ 
			if ( tmpFaces[ i ] == null )	i++;
			else break;
		}
		return i;
	}
	private function createIsolatedPatch( i:Int ):HEPatch
	{
		var patch:HEPatch = new HEPatch( this );
		patch.faces.push( tmpFaces[ i ] );
		patches.push( patch );
		tmpFaces[ i ].visited = true;
		tmpFaces[ i ] = null;
		return patch;
	}
	private function createContiguousPatch( i:Int ):HEPatch
	{
		var candidates:Vector<HEFace> = tmpFaces[ i ].contiguous;
		var patch:HEPatch = new HEPatch( this );
		patch.faces.push( tmpFaces[ i ] );
		patches.push( patch );
		tmpFaces[ i ].visited = true;
		tmpFaces[ i ] = null;
		for ( i in 0... candidates.length )
		{
			var currentFace:HEFace = candidates[ i ];
			currentFace.visited = true;
			tmpFaces[ currentFace.id ] = null;
			patch.faces.push( currentFace );
			for ( var f:HEFace in currentFace.contiguous )
			{
				if ( !f.visited )
				{
					candidates.push( f );
					f.visited = true;
				}
			}
		}
		return patch;
	}
	private function getPatch( tmpFaces:Vector<HEFace>, patches:Vector<HEPatch> ):Vector<HEPatch> 
	{
		var i:Int = getNextPatch();
		if ( i == tmpFaces.length ) return patches;
		if ( tmpFaces[ i ].contiguous.length == 0 )
		{
			//the face has no neighbours
			//create a patch for a single face
			createIsolatedPatch( i );
		}
		else
		{
			//recursively mark all neighbouring triangles as visited
			//and stores them in the patch
			createContiguousPatch( i );
		}
		return getPatch( tmpFaces, patches );
	}
	private function asyncGetPatch( tmpFaces:Vector<HEFace>, patches:Vector<HEPatch> ):Void
	{
		var i:Int = getNextPatch();
		_trace( 'plop', i );
		if ( i < tmpFaces.length )
		{
			var patch:HEPatch;
			if ( tmpFaces[ i ].contiguous.length == 0 )
			{
				patch = createIsolatedPatch( i );	
			}
			else
			{
				patch = createContiguousPatch( i );
			}
			dispatchEvent( new HEEvent( HEEvent.ON_PATCH_FINDING_UPDATE, null, patch ) );
			setTimeout( asyncGetPatch, 0, tmpFaces, patches );
			return;
		}
		dispatchEvent( new HEEvent( HEEvent.ON_PATCH_FINDING_COMPLETE, null, null, patches ) );
		_trace( 'fin' );
	}
	/*****************************
	end find patches
	*****************************/
	public function flipNormals( faces:Vector<HEFace> = null ):Void
	{
		if ( faces == null ) faces = this.faces;
		for( var f:HEFace in faces ) 
		{
			f.flipNormals();
		}
	}
	public var faces( get_faces, set_faces ):Vector;
	public function get_faces():Vector<HEFace> { return _faces; }
	public function set_faces(value:Vector<HEFace>):Vector 
	{
		_faces = value;
		return faces;
	}
	static public function getVertices( mesh:HEMesh ):Vector<Float>
	{
		return vertices[ mesh ];
	}
	static public function getIndices( mesh:HEMesh ):Vector<Int>
	{
		return indices[ mesh ];
	}
	static public function getUVs( mesh:HEMesh ):Vector<Float>
	{
		return uvs[ mesh ];
	}
}