package triga.lathe;
import nme.Vector;
using nme.Vector;
import away3d.core.base.SubGeometry;
import away3d.entities.Mesh;
import nme.geom.Matrix3D;
import nme.geom.Vector3D;
import triga.utils.VectorUtils;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Lathe extends SubGeometry
{
	private var _axis:Vector3D;
	private var _profile:Profile;
	private var _sides:Int;
	private var _offset:Vector3D;
	private var _turns:Float;
	public function new( axis:Vector3D, profile:Profile, sides:Int = 12, turns:Float = 1, offset:Vector3D = null ) 
	{
		_axis = axis;
		_axis.normalize();
		_profile = profile;
		_sides = sides;
		_turns = turns;
		_offset = offset || new Vector3D();
		compute();
	}
	private function compute():Void 
	{
		var vertices:Vector<Float> = new Vector<Float>();
		var indices:Vector<Int> = new Vector<Int>();
		var uvs:Vector<Float> = new Vector<Float>();
		var step:Int = 0, back:Int = 0, profilePointCount:Int = profile.path.length;
		var i0:Int, i1:Int, i2:Int, i3:Int, currentIndex:Int = 0;
		var m:Matrix3D = new Matrix3D();
		var v:Vector3D, t:Vector3D;
		var angleStep:Float = 360 / ( sides / turns );
		var total:Float = ( 360 * turns ) + angleStep;
		// 0 > i > 360° * turns
		for ( var i:Float = 0; i < total; i+=angleStep )
		{
			m.identity();
			m.appendRotation( i, axis, axis );
			for( v in profile.path )
			{
				t = m.transformVector( v );				
				vertices.push( t.x, t.y, t.z );
			}
			if ( step > 0 )
			{
				back = step - profilePointCount;
				for ( j in 0 ... profilePointCount - 1 )
				{
					i0 = back + j;
					i1 = back + ( j + 1 ) % profilePointCount;
					i2 = step + j;
					i3 = step + ( j + 1 ) % profilePointCount;
					indices.push( i1, i0, i2 );
					indices.push( i1, i2, i3 );
				}
			}
			step += profilePointCount;
		}
		updateVertexData( vertices );
		indices = indices.concat( indices.copy().reverse() );
		updateIndexData( indices );
		updateVertexNormalData( vertices.copy() );
		updateUVData( VectorUtils.expand2D( Vector<Float>([0, 1]), Vector<Float>([0, 1]), profilePointCount, ( sides * turns ) + 1 ) );
	}
	public function update():Void
	{
		var vertices:Vector<Float> = new Vector<Float>();
		var m:Matrix3D = new Matrix3D();
		var v:Vector3D, t:Vector3D;
		var angleStep:Float = 360 / ( sides / turns );
		var total:Float = ( 360 * turns ) + angleStep;
		for ( var i:Float = 0; i < total; i+=angleStep )
		{
			m.identity();
			m.appendRotation( i, axis, axis );
			for( v in profile.path )
			{	
				t = m.transformVector( v );
				vertices.push( t.x, t.y, t.z );
			}
		}
		updateVertexData( vertices );
	}
	public var axis( get_axis, set_axis ):Vector3D;
	public function get_axis():Vector3D 
	{
		return _axis;
	}
	public function set_axis(value:Vector3D):Vector3D 
	{
		_axis = value;
		update();
		return axis;
	}
	public var profile( get_profile, set_profile ):Profile;
	public function get_profile():Profile 
	{
		return _profile;
	}
	public function set_profile(value:Profile):Profile 
	{
		_profile = value;
		compute();
		return profile;
	}
	public var sides( get_sides, set_sides ):Int;
	public function get_sides():Int 
	{
		return _sides;
	}
	public function set_sides(value:Int):Int 
	{
		_sides = value;
		compute();
		return sides;
	}
	public var offset( get_offset, set_offset ):Vector3D;
	public function get_offset():Vector3D 
	{
		return _offset;
	}
	public function set_offset(value:Vector3D):Vector3D 
	{
		_offset = value;
		compute();
		return offset;
	}
	public var turns( get_turns, set_turns ):Float;
	public function get_turns():Float 
	{
		return _turns;
	}
	public function set_turns(value:Float):Float 
	{
		_turns = value;
		compute();
		return turns;
	}
	public var mesh( get_mesh, null ):Mesh;
	public function get_mesh():Mesh
	{
		var mesh:Mesh = new Mesh();
		mesh.geometry.addSubGeometry( this );
		return mesh;
	}
}