package triga.maths.shapes;
import nme.geom.Vector3D;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Plane
{
	static public var EPSILON:Float = .0001;
	public var a:Float, b:Float, c:Float, d:Float, abc:Float, sqr_abc:Float;
	private var _normal:Vector3D;
	public function new( a:Float = 0, b:Float = 0, c:Float = 0, d:Float = 0 ) 
	{
		normal = new Vector3D( a,b,c );
		this.a = a;
		this.b = b;
		this.c = c;
		this.d = d;
		abc = a * a + b * b + c * c;
		sqr_abc = Math.sqrt( abc );
		//x = ( a * d ) / abc;
		//y = ( b * d ) / abc;
		//z = ( c * d ) / abc;
	}
	public function normalizePlane():Void
	{
		var len : Number = 1 / sqr_abc;
		a *= len;
		b *= len;
		c *= len;
		d *= len;
	}
	public function determinant( _x:Float, _y:Float, _z:Float ):Float
	{
		return ( a * _x + b * _y + c * _z - d );
	}
	public function isCoplanar( plane:Plane ):Bool 
	{
		return ( 	a * ( plane.a * plane.d ) + 
					b * ( plane.b * plane.d ) + 
					c * ( plane.c * plane.d ) - d ) == 0;
	}
	public function isPointCoplanar( p:Vector3D ):Bool 
	{
		var det:Float = a * p.x + b * p.y + c * p.z - d;
		return ( det >= -EPSILON && det <= EPSILON );
	}
	public var normal( get_normal, set_normal ):Vector3D;
	public function get_normal():Vector3D 
	{
		return _normal;
	}
	public function set_normal(value:Vector3D):Vector3D 
	{
		_normal = value;
		return normal;
	}
}