package triga.shapes;
import nme.Vector;
using nme.Vector;
import away3d.core.base.data.Vertex;
import nme.geom.Vector3D;
import triga.utils.ConvexHull;
import triga.utils.PlaneUtils;
import triga.utils.TrigaSphereUtils;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Tetrahedron 
{
	public var i0:Int;
	public var i1:Int;
	public var i2:Int;
	public var i3:Int;
	public var centroid:Vector3D;
	public var sphere:TrigaSphere;
	public var inSphere:TrigaSphere;
	public var faces:Vector<TrigaFace>;
	private var _uvData:Vector<Float> = Vector.ofArray([0,0,0,1,1,1,1,0]);
	private var _vertexData:Vector<Float>;
	public var _dihedralAngles:Vector<Float> = Vector<Float>([0,0,0,0]);
	public function new( vertices:Vector<Float>, i0:Int, i1:Int, i2:Int, i3:Int ) 
	{
		this.i0 = i0;
		this.i1 = i1;
		this.i2 = i2;
		this.i3 = i3;
		var v0:Vector3D = new Vector3D( vertices[ i0 * 3 ], vertices[ i0 * 3 + 1 ], vertices[ i0 * 3 + 2 ] );
		var v1:Vector3D = new Vector3D( vertices[ i1 * 3 ], vertices[ i1 * 3 + 1 ], vertices[ i1 * 3 + 2 ] );
		var v2:Vector3D = new Vector3D( vertices[ i2 * 3 ], vertices[ i2 * 3 + 1 ], vertices[ i2 * 3 + 2 ] );
	 var v3:Vector3D = new Vector3D( vertices[ i3 * 3 ], vertices[ i3 * 3 + 1 ], vertices[ i3 * 3 + 2 ] );
		faces = Vector<TrigaFace>( [	new TrigaFace( vertices, i0, i1, i2 ),
										new TrigaFace( vertices, i0, i2, i3 ),
										new TrigaFace( vertices, i0, i3, i1 ),
										new TrigaFace( vertices, i1, i2, i3 )	] );
		centroid = new Vector3D( 	( vertices[ i0 * 3 ] 	+ vertices[ i1 * 3 ] + vertices[ i2 * 3 ]+ vertices[ i3 * 3 ] ) * .25,
									( vertices[ i0 * 3 + 1 ]+ vertices[ i1 * 3 + 1 ]+ vertices[ i2 * 3 + 1 ]+ vertices[ i3 * 3 + 1 ] ) * .25,
									( vertices[ i0 * 3 + 2 ]+ vertices[ i1 * 3 + 2 ]+ vertices[ i2 * 3 + 2 ]+ vertices[ i3 * 3 + 2 ] ) * .25	);
		_vertexData = Vector<Float>([ vertices[ i0 * 3 ] - centroid.x, vertices[ i0 * 3 + 1 ] - centroid.y, vertices[ i0 * 3 + 2 ] - centroid.z,
										vertices[ i1 * 3 ] - centroid.x, vertices[ i1 * 3 + 1 ] - centroid.y, vertices[ i1 * 3 + 2 ] - centroid.z,
										vertices[ i2 * 3 ] - centroid.x, vertices[ i2 * 3 + 1 ] - centroid.y, vertices[ i2 * 3 + 2 ] - centroid.z,
										vertices[ i3 * 3 ] - centroid.x, vertices[ i3 * 3 + 1 ] - centroid.y, vertices[ i3 * 3 + 2 ] - centroid.z]);
		sphere = TrigaSphereUtils.circumSphere( vertices[ i0 * 3 ], vertices[ i0 * 3 + 1 ], vertices[ i0 * 3 + 2 ],
												vertices[ i1 * 3 ], vertices[ i1 * 3 + 1 ], vertices[ i1 * 3 + 2 ],
												vertices[ i2 * 3 ], vertices[ i2 * 3 + 1 ], vertices[ i2 * 3 + 2 ],
												vertices[ i3 * 3 ], vertices[ i3 * 3 + 1 ], vertices[ i3 * 3 + 2 ]		);
		inSphere = TrigaSphereUtils.inSphere( v0, v1, v2, v3 );
		var normals:Vector<Vector3D> = new Vector<Vector3D>();
		for( var f:TrigaFace in faces )
		{
			if ( f.isFacing( centroid ) ) 
			{
				f.flip( _uvData );
			}
			f.buildVector3D();
			var n:Vector3D = f._v1.subtract( f._v0 ).crossProduct( f._v2.subtract( f._v0 ) );
			n.normalize();
			normals.push( n );
		}
		_dihedralAngles = Vector<Float>([	Math.acos( normals[ 0 ].dotProduct( normals[ 1 ] ) ),
											Math.acos( normals[ 1 ].dotProduct( normals[ 2 ] ) ),
											Math.acos( normals[ 2 ].dotProduct( normals[ 3 ] ) ),
											Math.acos( normals[ 3 ].dotProduct( normals[ 0 ] ) ) ] );
	}
	public var uvData( get_uvData, null ):Vector;
	public function get_uvData():Vector<Float>
	{
		return _uvData;
	}
	public var vertexData( get_vertexData, null ):Vector;
	public function get_vertexData():Vector<Float>
	{
		return _vertexData;
	}
	public var vertexNormalData( get_vertexNormalData, null ):Vector;
	public function get_vertexNormalData():Vector<Float>
	{
		return Vector<Float>( [0,0,1,0,0,1,0,0,1,0,0,1] );
	}
	public function makeUnique():Void
	{
		faces = Vector<TrigaFace>( [	new TrigaFace( _vertexData, 0, 1, 2 ),
										new TrigaFace( _vertexData, 0, 2, 3 ),
										new TrigaFace( _vertexData, 0, 3, 1 ),
										new TrigaFace( _vertexData, 1, 2, 3 )	] );
		for( var f:TrigaFace in faces )
		{
			if ( f.isFacing( centroid ) ) 
			{
				f.flip();
			}
		}
	}
	public var indexData( get_indexData, null ):Vector;
	public function get_indexData():Vector<Int>
	{
		return Vector<Int>( [	faces[ 0 ].i0, faces[ 0 ].i1, faces[ 0 ].i2, 
								faces[ 1 ].i0, faces[ 1 ].i1, faces[ 1 ].i2, 
								faces[ 2 ].i0, faces[ 2 ].i1, faces[ 2 ].i2,
								faces[ 3 ].i0, faces[ 3 ].i1, faces[ 3 ].i2		] );
	}
	/*
	public function get centroid():Vertex 
	{
		if ( _centroid == null )
		{
			_centroid = new Vertex( ( vertices[ i0 * 3 ] 	+ vertices[ i1 * 3 ] + vertices[ i2 * 3 ]+ vertices[ i3 * 3 ] ) * .25,
									( vertices[ i0 * 3 + 1 ]+ vertices[ i1 * 3 + 1 ]+ vertices[ i2 * 3 + 1 ]+ vertices[ i3 * 3 + 1 ] ) * .25,
									( vertices[ i0 * 3 + 2 ]+ vertices[ i1 * 3 + 2 ]+ vertices[ i2 * 3 + 2 ]+ vertices[ i3 * 3 + 2 ] ) * .25	);
		}
		return _centroid;
	}
	*/
	public function shareFace( t:Tetrahedron ):TrigaFace
	{
		for( var f:TrigaFace in faces ) for( var _f:TrigaFace in t.faces ) if ( f.isPairOrEqual( _f ) ) return f;
		return null;
	}
	public function hasCommonFace( face:TrigaFace ):TrigaFace
	{
		for( var f:TrigaFace in faces ) if ( f.isPairOrEqual( face ) ) return f;
		return null;
	}
	public function hasEqualFace( face:TrigaFace ):TrigaFace
	{
		for( var f:TrigaFace in faces ) if ( f.equals( face ) ) return f;
		return null;
	}
	public function shareVertex( index:Int ):Bool
	{
		if ( i0 == index || i1 == index || i2 == index || i3 == index ) return true;
		return false;
	}
}