package triga.spline.curves;
import nme.Vector;
using nme.Vector;
import nme.geom.Vector3D;
// forked from makc3d's Cardinal Splines Part 4
 /**
 * Exploring formula in Jim Armstrong "Cardinal Splines Part 4"
 * @see http://algorithmist.wordpress.com/2009/10/06/cardinal-splines-part-4/
 */
 class Cardinal implements ICurve3D
{
	private var _precision:Float = .1;
	private var _tension:Float = 1;
	private var _loop:Bool = false;
	public function new( precision:Float = .1, tension:Float = .1, loop:Bool = false )
	{
		_precision = precision;
		_tension = tension;
		_loop = loop;
	}
 static public function compute( vertices:Vector<Vector3D>, precision:Float = .1, tension:Float = 1, loop:Bool = false, target:Vector<Vector3D> = null):Vector<Vector3D>
	{
		precision = Math.max( .01, Math.min( 1, precision ) );
		tension = Math.max( -3, Math.min( 3, tension ) );
		if(  target  == null )  target = new Vector<Vector3D>();
		var p0:Vector3D, p1:Vector3D, p2:Vector3D, p3:Vector3D;
 var i:Int, t:Float;
 for (i in 0... vertices.length - ( loop ? 0 : 1 ))
		{
 p0 = (i < 1) ? vertices [vertices.length - 1] : vertices [i - 1];
			p1 = vertices [i];
			p2 = vertices [(i +1 + vertices.length) % vertices.length];
			p3 = vertices [(i +2 + vertices.length) % vertices.length];
			//target.push( p1 );
			for ( t= 0; t < 1; t += precision ) 
			{
 target.push( new Vector3D 	(
											// x
											tension * ( -t * t * t + 2 * t * t - t) * p0.x +
											tension * ( -t * t * t + t * t) * p1.x +
											(2 * t * t * t - 3 * t * t + 1) * p1.x +
											tension * (t * t * t - 2 * t * t + t) * p2.x +
											( -2 * t * t * t + 3 * t * t) * p2.x +
											tension * (t * t * t - t * t) * p3.x,
											// y
											tension * ( -t * t * t + 2 * t * t - t) * p0.y +
											tension * ( -t * t * t + t * t) * p1.y +
											(2 * t * t * t - 3 * t * t + 1) * p1.y +
											tension * (t * t * t - 2 * t * t + t) * p2.y +
											( -2 * t * t * t + 3 * t * t) * p2.y +
											tension * (t * t * t - t * t) * p3.y,
											// z
											tension * ( -t * t * t + 2 * t * t - t) * p0.z +
											tension * ( -t * t * t + t * t) * p1.z +
											(2 * t * t * t - 3 * t * t + 1) * p1.z +
											tension * (t * t * t - 2 * t * t + t) * p2.z +
											( -2 * t * t * t + 3 * t * t) * p2.z +
											tension * (t * t * t - t * t) * p3.z
										) );
 }
 }
		if ( loop )
		{
			target.push( target[0] );
		}
		else {
			//target.push( vertices[vertices.length -1 ] );
		}
		return target;
 }
	/* INTERFACE triga.curves.shapes.ICurve3D */
	public function set_precision(value:Float):Float 
	{
		_precision = value;
		return precision;
	}
	public var precision( get_precision, set_precision ):Float;
	public function get_precision():Float 
	{
		return _precision;
	}
	public var tension( get_tension, set_tension ):Float;
	public function get_tension():Float 
	{
		return _tension;
	}
	public function set_tension(value:Float):Float 
	{
		_tension = value;
		return tension;
	}
	public function set_loop(value:Bool):Bool 
	{
		_loop = value;
		return loop;
	}
	public var loop( get_loop, set_loop ):Bool;
	public function get_loop():Bool 
	{
		return _loop;
	}
	public function compute( vertices:Vector<Vector3D>, target:Vector<Vector3D> = null ):Vector<Vector3D> 
	{
		return Cardinal.compute( vertices, precision, tension, loop, target );
	}
 }