package triga.spline.curves;
import nme.Vector;
using nme.Vector;
import nme.geom.Vector3D;
 /** 
 * Catmull-Rom spline through N vertices.
 * @author makc
 * @license WTFPLv2, http://sam.zoy.org/wtfpl/
 */ 
 class CatmullRom implements ICurve3D
 {
	private var _precision:Float = .1;
	private var _loop:Bool = false;
	public function new( precision:Float = .1, loop:Bool = false )
	{
		_precision = precision;
		_loop = loop;
	}
 static public function compute( vertices:Vector<Vector3D>, precision:Float = .1, loop:Bool = false, target:Vector<Vector3D> = null ):Vector<Vector3D>
	{
		precision = Math.max( .01, Math.min( .99, precision ) );
		var i:Int, t:Float;
		var p0:Vector3D, p1:Vector3D, p2:Vector3D, p3:Vector3D;
		if(  target  == null )  target = new Vector<Vector3D>();
		//target.push( vertices[0] );
 for (i in 0... vertices.length - ( loop ? 0 : 1 )) 
		{
			p0 = vertices [(i -1 + vertices.length) % vertices.length];
			p1 = vertices [i];
			p2 = vertices [(i +1 + vertices.length) % vertices.length];
			p3 = vertices [(i +2 + vertices.length) % vertices.length];
			target.push( p1 );
			for ( t = precision; t < 1; t+= precision ) 
			{
				target.push( new Vector3D (	0.5 * (( 2*p1.x) +
											t * (( -p0.x +p2.x) +
											t * ((2*p0.x -5*p1.x +4*p2.x -p3.x) +
											t * ( -p0.x +3 * p1.x -3 * p2.x +p3.x)))),
											0.5 * (( 2*p1.y) +
											t * (( -p0.y +p2.y) +
											t * ((2*p0.y -5*p1.y +4*p2.y -p3.y) +
											t * ( -p0.y +3 * p1.y -3 * p2.y +p3.y)))),
											0.5 * (( 2*p1.z) +
											t * (( -p0.z +p2.z) +
											t * ((2*p0.z -5*p1.z +4*p2.z -p3.z) +
											t * ( -p0.z +3 * p1.z -3 * p2.z +p3.z))))		) );
			}
		}
		if ( loop )
		{
			target.push( target[0] );
		}
		else 
		{
		//	target.push( vertices[vertices.length -1 ] );
		}
		return target;
 }
	/* INTERFACE triga.curves.shapes.ICurve3D */
	public function set_precision(value:Float):Float 
	{
		_precision = value;
		return precision;
	}
	public var precision( get_precision, set_precision ):Float;
	public function get_precision():Float 
	{
		return _precision;
	}
	public function set_loop(value:Bool):Bool 
	{
		_loop = value;
		return loop;
	}
	public var loop( get_loop, set_loop ):Bool;
	public function get_loop():Bool 
	{
		return _loop;
	}
	public function compute( vertices:Vector<Vector3D>, target:Vector<Vector3D> = null ):Vector<Vector3D> 
	{
		return CatmullRom.compute( vertices, precision, loop, target );
	}
 }