package triga.spline.utils;
import nme.Vector;
using nme.Vector;
import nme.display.Graphics;
import nme.geom.Point;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Edge 
{
	static private var ip:Point = new Point;
	static public var edges:Vector<Edge> = new Vector<Edge>();
	static public var buffer:Vector<Point>;
	public var active:Bool = true;
	public var start:Int;
	public var end:Int;
	public var edgeId:Int;
	public function new( start:Int, end:Int ) 
	{
		this.start = start;
		this.end = end;
		edgeId = edges.length;
		edges.push( this );
	}
	public function subdivide( ip:Point, start:Int, end:Int ):Void
	{
		active = false;
		new Edge( start, buffer.length );
		new Edge( buffer.length, end );
		buffer.push( ip.clone() );
	}
	public function intersect( other:Edge ):Point
	{
		if ( !active || !other.active ) return null;
		var ip:Point = lineIntersectLine( buffer[ start ], buffer[ end ], buffer[ other.start ], buffer[ other.end ] )
		if ( 	ip != null 
		&& 		!ip.equals( buffer[ start ] )
		&& 		!ip.equals( buffer[ end ] )
		&& 		!ip.equals( buffer[ other.start ] )
		&& 		!ip.equals( buffer[ other.end ] )	)
		{
			return ip;
		}
		return null;
	}
	public function isLeft( e:Edge ):Bool 
	{
		var a:Point = buffer[ start ];
		var b:Point = buffer[ end ];
		var p:Point = buffer[ e.end ];
		return ( ( a.x - b.x ) * ( b.y - b.y ) ) - ( ( p.x - b.x ) * ( a.y - b.y ) ) > 0;
	}
	static public function lineIntersectLine ( A : Point, B : Point, E : Point, F : Point, ABasSeg : Boolean = true, EFasSeg : Boolean = true ) : Point
	{
			var a1:Float, a2:Float, b1:Float, b2:Float, c1:Float, c2:Float;
			a1= B.y-A.y;
			b1= A.x-B.x;
			a2= F.y-E.y;
			b2= E.x-F.x;
			var denom:Float=a1*b2 - a2*b1;
			if (denom == 0) return null;
			c1= B.x*A.y - A.x*B.y;
			c2 = F.x * E.y - E.x * F.y;
			ip.x=(b1*c2 - b2*c1)/denom;
			ip.y=(a2*c1 - a1*c2)/denom;
			if ( A.x == B.x )ip.x = A.x;
			else if ( E.x == F.x ) ip.x = E.x;
			if ( A.y == B.y )ip.y = A.y;
			else if ( E.y == F.y )ip.y = E.y;
			if ( ABasSeg )
			{
					if ( ( A.x < B.x ) ? ip.x < A.x || ip.x > B.x : ip.x > A.x || ip.x < B.x )return null;
					if ( ( A.y < B.y ) ? ip.y < A.y || ip.y > B.y : ip.y > A.y || ip.y < B.y )return null;
			}
			if ( EFasSeg )
			{
					if ( ( E.x < F.x ) ? ip.x < E.x || ip.x > F.x : ip.x > E.x || ip.x < F.x )return null;
					if ( ( E.y < F.y ) ? ip.y < E.y || ip.y > F.y : ip.y > E.y || ip.y < F.y )return null;
			}
			return ip;
	}
	/**
	 * 
	 **/
	public function render( graphics:Graphics, _angle:Float = 20, _length:Float = 15 ):Void
	{
		var p0:Point = buffer[ start ];
		var p1:Point = buffer[ end ];
		graphics.moveTo( 	p0.x, p0.y );
		graphics.lineTo( 	p1.x, p1.y );
		var dx:Float = p1.x - p0.x;
		var dy:Float = p1.y - p0.y;
		var angle:Float = Math.atan2( dy, dx );
		var len:Float = Math.sqrt( dx * dx + dy * dy );
		var rad:Float = Math.PI / 180 * _angle;
		graphics.moveTo( 	p1.x - Math.cos( angle - rad ) * _length,
							p1.y - Math.sin( angle - rad ) * _length	);
		graphics.lineTo( 	p1.x, p1.y ); 
		graphics.lineTo( 	p1.x - Math.cos( angle + rad ) * _length,
							p1.y - Math.sin( angle + rad ) * _length	);
		graphics.lineTo( 	p1.x - Math.cos( angle - rad ) * _length,
							p1.y - Math.sin( angle - rad ) * _length	);
	}
}