﻿package systems.traer 
import nme.Vector;
using nme.Vector;
{
import nme.display.Sprite;
package systems.traer;
class ParticleSystem
{
	public static var RUNGE_KUTTA:Int = 0;
	public static var MODIFIED_EULER:Int = 1;
	private static var DEFAULT_GRAVITY:Float = 0;
	private static var DEFAULT_DRAG:Float = 0.01;
	private var _particles:Vector<Particle>;
	private var _springs:Vector<Spring>;
	private var _attractions:Vector<Attraction>;
	private var _customForces:Vector<Force> = new Vector<Force>();
	private var _integrator:Integrator;
	private var _gravity:Vector3D;
	private var _drag:Float;
	public function new( gravity:Vector3D, drag:Float = .01 )
	{
		integrator = new RungeKuttaIntegrator(this);
		particles = new Vector<Particle>();
		springs = new Vector<Spring>();
		attractions = new Vector<Attraction>();
		this.gravity = gravity.clone();
		this.drag = drag;
	}
	public function setGravity3D(gx:Float, gy:Float, gz:Float):Void 
	{
		gravity.x = gx;
		gravity.y = gy;
		gravity.z = gz;
	}
	public function setGravity2D(gy:Float):Void {
		gravity.y = gy;
	}
	public function tick(t:Float = 1):Void 
	{
		integrator.step(t);
	}
	public function makeParticle(mass:Float = 1, x:Float = 0, y:Float = 0, z:Float = 0):Particle
	{
		var p:Particle = new Particle(mass);
		p.position.x = x
		p.position.y = y
		p.position.z = z
		particles.push(p);
		return p;
	}
	public function makeSpring(particleA:Particle, particleB:Particle, ks:Float, d:Float, r:Float):Spring
	{
		var s:Spring = new Spring(particleA, particleB, ks, d, r);
		springs.push(s);
		return s;
	}
	public function makeAttraction(particleA:Particle, particleB:Particle, k:Float, minDistance:Float):Attraction
	{
		var a:Attraction = new Attraction(particleA, particleB, k,minDistance);
		attractions.push(a);
		return a;
	}
	public function clear():Void 
	{
		particles = new Vector<Particle>();
		springs = new Vector<Spring>();
		attractions = new Vector<Attraction>();
	}
	public function applyForces():Void
	{
		//var time:Int = getTimer();
		var i:Int;
		var len:Int = particles.length;
		var p:Particle;
		var isZero:Bool = ( gravity.x == 0 && gravity.y == 0 && gravity.z == 0 );
		if ( !isZero )
		 {
			 for ( i in 0... len; ++i )
			 {
				 particles[i].force = particles[i].force.add( gravity );//TODO retourner un Vcetor3D
			 }
		 }
		for ( i in 0... len; ++i )
		{
			p = particles[i];
			//p.force.addFromValues( p.velocity.x * -drag, p.velocity.y * -drag, p.velocity.z * -drag );
			p.force.x += p.velocity.x * -drag;
			p.force.y += p.velocity.y * -drag;
			p.force.z += p.velocity.z * -drag;
		}
		 len = springs.length;
		 for ( i in 0... len )
		 {
			 springs[i].apply();
		 }
		 len = attractions.length;
		 for ( i in 0... len )
		 {
			 attractions[i].apply();
		 }
		 len = customForces.length;
		 for (i in 0... len )
		 {
			 customForces[i].apply();
		 }
		 //trace("time = "+(getTimer()-time))
	 }
	 public function clearForces():Void
	 {
		 var i:Int;
		 var len:Int = particles.length;
		 for (i in 0... len) 
		 {
			particles[i].clearForce();
		 }
	 }
	 public function numberOfParticles():Int {
		 return particles.length;
	 }
	 public function numberOfSprings():Int {
		 return springs.length;
	 }
	 public function numberOfAttractions():Int {
		 return attractions.length;
	 }
	 public function getParticleById(id:Int):Particle {
		 return particles[id];
	 }
	 public function getSpringById(id:Int):Spring {
		 return springs[id];
	 }
	 public function getAttractionById(id:Int):Attraction
	 {
		 return attractions[id];
	 }
	 public function addCustomForce(f:Force):Void 
	 {
		 customForces.push(f);
	 }
	 public function numberOfCustomForces():Int
	 {
		 return customForces.length;
	 }
	 public function getCustomForceById(id:Int):Force 
	 {
		 return customForces[id];
	 }
	 public function removeCustomForceById(id:Int):Force
	 {
		 var f:Force = customForces[id];
		 customForces.splice(id, 1);
		 return f;
	 }
	 public function removeCustomForce(f:Force):Void 
	 {
		 customForces.splice(customForces.lastIndexOf(f), 1);
	 }
	 public function removeParticleById(id:Int):Particle 
	 {
		 //var f:Particle = particles[id];
		 var i:Int
		 //for( var p:Particle in particles ) 
		 for ( i in 0... particles.length ) 
		 {
			 if ( particles[ i ].id == id )
			 {
				particles.splice(i, 1);
			 }
		 }
		 return null;
	 }
	 public function removeParticle(p:Particle):Void
	 {
		 particles.splice(particles.lastIndexOf(p), 1);
	 }
	 public function removeSpringById(id:Int):Spring
	 {
		 var f:Spring = springs[id];
		 springs.splice(id, 1);
		 return f;
	 }
	 public function removeSpring(s:Spring):Void
	 {
		 springs.splice(springs.lastIndexOf(s), 1);
	 }
	 public function removeAttractionById(id:Int):Attraction
	 {
		 var f:Attraction = attractions[id];
		 attractions.splice(id, 1);
		 return f;
	 }
	 public function removeAttraction(a:Attraction):Void 
	 {
		 attractions.splice( attractions.lastIndexOf(a), 1 );
	 }
	public var particles( get_particles, set_particles ):Vector;
	 public function get_particles():Vector<Particle> { return _particles; }
	 public function set_particles(value:Vector<Particle>):Vector 
	 {
		 _particles = value;
	 	return particles;
	}
	public var gravity( get_gravity, set_gravity ):Vector3D;
	 public function get_gravity():Vector3D { return _gravity; }
	 public function set_gravity(value:Vector3D):Vector3D 
	 {
		 _gravity = value;
	 	return gravity;
	}
	public var drag( get_drag, set_drag ):Float;
	 public function get_drag():Float { return _drag; }
	 public function set_drag(value:Float):Float 
	 {
		 _drag = value;
	 	return drag;
	}
	public var springs( get_springs, set_springs ):Vector;
	 public function get_springs():Vector<Spring> { return _springs; }
	 public function set_springs(value:Vector<Spring>):Vector 
	 {
		 _springs = value;
	 	return springs;
	}
	public var attractions( get_attractions, set_attractions ):Vector;
	 public function get_attractions():Vector<Attraction> { return _attractions; }
	 public function set_attractions(value:Vector<Attraction>):Vector 
	 {
		 _attractions = value;
	 	return attractions;
	}
	public var customForces( get_customForces, set_customForces ):Vector;
	 public function get_customForces():Vector<Force> { return _customForces; }
	 public function set_customForces(value:Vector<Force>):Vector 
	 {
		 _customForces = value;
	 	return customForces;
	}
	public var integrator( get_integrator, set_integrator ):Integrator;
	 public function get_integrator():Integrator { return _integrator; }
	 public function set_integrator(value:Integrator):Integrator 
	 {
		 _integrator = value;
	 	return integrator;
	}
}