﻿package systems.traer
import nme.Vector;
using nme.Vector;
{
import nme.display.Shape;
package systems.traer;
import nme.display.StageDisplayState;
import nme.display.StageScaleMode;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.geom.Point;
import nme.geom.Vector3D;
import shapes.delaunay.Delaunay;
import shapes.delaunay.DelaunayTriangle;
import shapes.shapes2D.polygon.ConvexHull;
import shapes.shapes2D.polygon.Triangulation;
/**
 * @author Nicolas Barradeau
 */
class Arboreum extends Sprite
{
	private var NODE_SIZE:Float = 1;
	private var EDGE_LENGTH:Float = 10;
	private var EDGE_STRENGTH:Float = 0.2;
	private var SPACER_STRENGTH:Float = 10000;
	//ParticleSystem physics;
	private var scale:Float = 1;
	private var centroidX:Float = 0;
	private var centroidY:Float = 0;
	public function new() 
	{
		if (stage) init();
		else addEventListener(Event.ADDED_TO_STAGE, init);
	}
	private var canvas:Shape = new Shape();
	private var physics:ParticleSystem;
	private var particles:Vector<Particle>;
	private function init(e:Event = null):Void 
	{
		removeEventListener(Event.ADDED_TO_STAGE, init);
		stage.align = StageAlign.TOP_LEFT;
		stage.scaleMode = StageScaleMode.NO_SCALE;
		stage.displayState = StageDisplayState.FULL_SCREEN;
		physics = new ParticleSystem( new Vector3D( 0.00000000001, 0, 0 ), 0.99 );
		particles = new Vector<Particle>();
		addChild( canvas );
		// Now try this to see make it more damped, but stable.
		//physics.drag = 0.001;
		initialize();
		addEventListener( Event.ENTER_FRAME, oef );
		stage.addEventListener( MouseEvent.MOUSE_DOWN, mHandler );
	}
	private function mHandler( e:Event ):Void
	{
		addNode();
	}
	private function oef( e:Event ):Void
	{
		physics.tick(); 
		if ( physics.numberOfParticles() > 1 ) 
		{
			updateCentroid();
			x = stage.stageWidth / 2 - centroidX;
			y = stage.stageHeight / 2 - centroidY;
		}
		scaleX = scaleY = scale;
		draw(); 
	}
	private function addSpacersToNode( p:Particle, r:Particle ):Void
	{
		var i:Int;
		var q:Particle;
		for ( i in 0... physics.numberOfParticles(); ++i )
		{
			q = physics.getParticleById( i );
			if ( p != q && p != r )	physics.makeAttraction( p, q, -SPACER_STRENGTH, 20 );
		}
	}
	private function makeEdgeBetween( a:Particle, b:Particle ):Void
	{
		physics.makeSpring( a, b, EDGE_STRENGTH, EDGE_STRENGTH, EDGE_LENGTH );
	}
	private function initialize():Void
	{
		physics.clear();
		physics.makeParticle();
	}
	private function addNode():Void
	{ 
		var q:Particle = physics.getParticleById( Math.floor( Math.random() * physics.numberOfParticles() ) );
		var p:Particle = physics.makeParticle();
		p.position.x = q.position.x + Math.random() - .5;// ? -1:1;
		p.position.y = q.position.y + Math.random() - .5;// ? -1:1; 
		p.position.z = q.position.z + Math.random() - .5;// ? -1:1;
		addSpacersToNode( p, q );
		makeEdgeBetween( p, q );
	}
	private function updateCentroid():Void
	{
		var xMax:Float = Math.NEGATIVE_INFINITY;
		var xMin:Float = Math.POSITIVE_INFINITY;
		var yMax:Float = Math.NEGATIVE_INFINITY; 
		var yMin:Float = Math.POSITIVE_INFINITY;
		var i:Int;
		var p:Particle;
		for ( i in 0... physics.numberOfParticles(); ++i )
		{
			p = physics.getParticleById( i );
			if ( xMin > p.position.x ) xMin = p.position.x;
			if ( xMax < p.position.x ) xMax = p.position.x;
			if ( yMin > p.position.y ) yMin = p.position.y;
			if ( yMax < p.position.y ) yMax = p.position.y;
		}
		var deltaX:Float = xMax-xMin;
		var deltaY:Float = yMax-yMin;
		centroidX = xMin + 0.5 * deltaX;
		centroidY = yMin + 0.5 * deltaY;
		if ( deltaY > deltaX )
		{
			scale = canvas.height/deltaY;
		}
		else 
		{
			scale = canvas.width/deltaX;
		}
	}
	private function draw():Void
	{ 
		canvas.graphics.clear();
		canvas.graphics.lineStyle( 2, 0xFFFFFF );
		var v:Particle;
		var s:Spring;
		var points:Vector<Point> = new Vector<Point>();
		var i:Int;
		// draw vertices
		for ( i in 0... physics.numberOfParticles(); ++i )
		{
			v = physics.getParticleById( i );
			v.draw( canvas.graphics, 5 );
			points.push( new Point( v.position.x, v.position.y ) );
		}
		// draw edges 
		//for ( i in 0... physics.numberOfSprings(); ++i )
		//{
			//s = physics.getSpringById( i );
			//s.draw( canvas.graphics );
		//}
		canvas.graphics.lineStyle( 4, 0xFFFF00 );
		plotLines( ConvexHull.process( points ) );
		canvas.graphics.lineStyle( 2, 0xFF0000 );
		points = Triangulation.process( points );
		var delaunay:Vector<DelaunayTriangle> = Delaunay.Triangulate( points );
		if ( delaunay != null ) plotTriss( delaunay );
	}
	private function plotTriss(delaunay:Vector<DelaunayTriangle> = null ):Void 
	{
		for( var t:DelaunayTriangle in delaunay )
		{
			t.draw( canvas.graphics );
		}
	}
	private function plotLines( points:Vector<Point>):Void
	{
		if ( points.length == 0 ) return;
		var p:Point = points[ 0 ];
		canvas.graphics.moveTo( p.x, p.y );
		for( p in points )
		{
			canvas.graphics.lineTo( p.x, p.y );
		}
		canvas.graphics.lineTo( points[ 0 ].x, points[ 0 ].y );
	}
}