﻿package systems.traer
import nme.Vector;
using nme.Vector;
{
import nme.display.Sprite;
package systems.traer;
import nme.display.StageScaleMode;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.geom.Point;
import nme.geom.Vector3D;
import utils.GeomUtils;
/**
 * ...
 * @author tlecoz
 */
class Cloth extends Sprite 
{
	private var physics:ParticleSystem;
	private var particles:Vector<Vector<Particle>>;
	private var gridSize:Int = 20;
	private var SPRING_STRENGTH:Float = 0.2;
	private var SPRING_DAMPING:Float = 0.1;
	private var isDown:Bool = false;
	private var old:Point = new Point();
	private var corners:Vector<Particle>;
	public function new():Void 
	{
		stage.align = StageAlign.TOP_LEFT;
		stage.scaleMode = StageScaleMode.NO_SCALE;
		stage.displayState = StageDisplayState.FULL_SCREEN;
		stage.frameRate = 48
		physics = new ParticleSystem( new Vector3D( 0, 0, 0.01 ), 0.01 );
		particles = new Vector<Vector<Particle>>(gridSize, true);
		var gridStepX:Float = (stage.stageWidth / 2) / gridSize;
		var gridStepY:Float = (stage.stageHeight / 2) / gridSize;
		var i:Int,j:Int;
		var left:Float = stage.stageWidth / 2 - ( ( gridStepX * gridSize ) / 2 );
		var top:Float = stage.stageHeight / 2 - ( ( gridStepY * gridSize ) / 2 );
		for (i in 0... gridSize) 
		{
			particles[i] = new Vector<Particle>(gridSize, true);
			for (j in 0... gridSize) 
			{
				particles[i][j] = physics.makeParticle( 1, left + j * gridStepX + (width / 4), top + i * gridStepY + 20, 0.0);
				if (j > 0) 
				{
					physics.makeSpring(particles[i][j - 1], particles[i][j], SPRING_STRENGTH, SPRING_DAMPING, gridStepX);
				}
			}
		}
		for (j in 0... gridSize) 
		{
			for (i = 1; i < gridSize) 
			{
				 physics.makeSpring(particles[i - 1][j], particles[i][j], SPRING_STRENGTH, SPRING_DAMPING, gridStepY);
			}
		}
		corners = Vector.ofArray( [
											particles[0][0], 
											particles[0][gridSize - 1],
											particles[gridSize - 1][0], 
											particles[gridSize - 1][gridSize - 1] 
										] );
		for( var p:Particle in corners )
		{
			p.fixed = true;
		}
		stage.addEventListener(MouseEvent.MOUSE_DOWN, onDown);
		stage.addEventListener(MouseEvent.MOUSE_UP, onUp);
		addEventListener(Event.ENTER_FRAME, draw);
	}
	private function onDown(e:MouseEvent):Void {
		isDown = true;
	}
	private function onUp(e:MouseEvent):Void
	{
		isDown = false;
		//particles[0][gridSize - 1].velocity.setFromValues( (mouseX - old.x), (mouseY - old.y), 0 );
	}
	private function draw(e:Event = null):Void
	{
		physics.tick();
		if (isDown)
		{
			var p:Particle = getClosestParticle();
			//particles[0][gridSize - 1].position.setFromValues( mouseX, mouseY, 0);
			p.position.x = mouseX;
			p.position.y = mouseY;
			p.clearVelocity();
		}
		graphics.clear();
		graphics.lineStyle(0, 0xffffff);
		var i:Int, j:Int;
		for (i in 0... gridSize)
		{
			for (j in 0... gridSize)
			{
				particles[i][j].draw( graphics );
			}
		}
		for (i in 0... gridSize)
		 {
			graphics.moveTo(particles[i][0].position.x, particles[i][0].position.y);
			for (j in 0... gridSize)
			{
			 graphics.lineTo( particles[i][j].position.x, particles[i][j].position.y);
			}
			graphics.lineTo(particles[i][gridSize - 1].position.x, particles[i][gridSize - 1].position.y);
		 }
		 for (j in 0... gridSize)
		 {
			graphics.moveTo(particles[0][j].position.x, particles[0][j].position.y);
			for (i in 0... gridSize)
			{
			 graphics.lineTo(particles[i][j].position.x, particles[i][j].position.y);
			}
			graphics.lineTo(particles[gridSize - 1][j].position.x, particles[gridSize - 1][j].position.y);
		 }
		 old.x = mouseX;
		 old.y = mouseY;
	}
	private function getClosestParticle():Particle 
	{
		var distMax:Float = Math.POSITIVE_INFINITY;
		var dist:Float;
		var _p:Particle;
		for( var p:Particle in corners )
		{
			dist = GeomUtils.distance( new Point( p.position.x, p.position.y ), new Point( mouseX, mouseY ) );
			if ( dist < distMax )
			{
				distMax = dist;
				_p = p;
			}
		}
		return _p;
	}
}