﻿package systems.traer
import nme.Vector;
using nme.Vector;
{
import nme.display.BitmapData;
package systems.traer;
import nme.display.TriangleCulling;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.filters.BlurFilter;
import nme.geom.Point;
import nme.geom.Vector3D;
import utils.GeomUtils;
class ClothImage extends Sprite 
{
	private var physics:ParticleSystem;
	private var particles:Vector<Vector<Particle>>;
	private var gridWidth:Int = 15;
	private var gridHeight:Int = 15;
	private var SPRING_STRENGTH:Float = .11;
	private var SPRING_DAMPING:Float = 0.2;
	private var isDown:Bool = false;
	private var old:Point = new Point();
	private var corners:Vector<Particle>;
	private var srcBd:BitmapData = new BitmapData( 400,400, true,0 );
	private var indices:Vector<Int>;
	private var uvs:Vector<Float>;
	public function new():Void 
	{
		//stage.frameRate = 42;
		srcBd.perlinNoise( 20, 100, 3, 0, true, true, 1, true );
		srcBd.copyChannel( srcBd, srcBd.rect, new Point(), BitmapDataChannel.RED, BitmapDataChannel.ALPHA );
		//srcBd.applyFilter( srcBd, srcBd.rect, new Point(), new BlurFilter( 12, 12, 3 ) );
		srcBd.threshold( srcBd, srcBd.rect, new Point(), '>=', 0x80000000, 0 );
		srcBd.threshold( srcBd, srcBd.rect, new Point(), '!=', 0, 0xFFFFFFFF );
		physics = new ParticleSystem( new Vector3D( 0, 0.1, 0 ), 0.01 );
		particles = new Vector<Vector<Particle>>( gridHeight, true);
		var gridStepX:Float = (stage.stageWidth / 2) / gridWidth;
		var gridStepY:Float = (stage.stageHeight / 2) / gridHeight;
		var left:Float = stage.stageWidth / 2 - ( ( gridStepX * gridWidth ) / 2 );
		var top:Float = stage.stageHeight / 2 - ( ( gridStepY * gridHeight ) / 2 );
		uvs 		= new Vector<Float>();
		indices 		= new Vector<Int>();
		var i:Int, j:Int, k:Int= 0;
		for (i in 0... gridHeight) 
		{
			particles[i] = new Vector<Particle>( gridWidth, true);
			for (j in 0... gridWidth) 
			{
				particles[ i ][ j ] = physics.makeParticle( 1, left + i * gridStepX, top + j * gridStepY );
				particles[ i ][ j ].id = k;
				k++;
				if( i > 0 ) physics.makeSpring(particles[i - 1][j], particles[i][j], SPRING_STRENGTH, SPRING_DAMPING, gridStepX);
				if( j > 0 ) physics.makeSpring(particles[i][j - 1], particles[i][j], SPRING_STRENGTH, SPRING_DAMPING, gridStepY);
				uvs.push( j / gridWidth, i / gridHeight );
			}
		}
		k = 0;
		for ( i in 0... gridHeight - 1 )
		{
			for ( j in 0... gridWidth - 1 )
			{
				k = i * gridHeight + j;
				indices.push( k, k + 1, ( k + gridWidth ) );
				indices.push( k + 1, ( k + gridWidth ) + 1, ( k + gridWidth ) );
			}
		}
		corners = Vector<Particle>( 	[
										particles[ 0 ][ 0 ], 
										particles[ 0 ][ gridWidth - 1 ],
										particles[ gridHeight - 1 ][ 0 ], 
										particles[ gridHeight - 1 ][ gridWidth - 1] 
										] );
		for( var p:Particle in corners )
		{
			p.fixed = true;
		}
		stage.addEventListener(MouseEvent.MOUSE_DOWN, onDown);
		stage.addEventListener(MouseEvent.MOUSE_UP, onUp);
		addEventListener(Event.ENTER_FRAME, draw);
	}
	private function onDown(e:MouseEvent):Void 
	{
		isDown = true;
	}
	private function onUp(e:MouseEvent):Void
	{
		isDown = false;
	}
	private function draw(e:Event = null):Void
	{
		physics.tick();
		var p:Particle;
		if (isDown)
		{
			p = getClosestParticle();
			p.position.x = mouseX;
			p.position.y = mouseY;
			p.clearVelocity();
		}
		graphics.clear();
		var i:Int, j:Int, i0:Int, i1:Int, i2:Int, i3:Int;
		var v:Vector3D;
		var vertices:Vector<Float> 	= new Vector<Float>();
		for (i in 0... gridHeight)
		{
			for (j in 0... gridWidth)
			{
				v = particles[ i ][ j ].position;
				vertices.push( v.x, v.y );
			}
		}
		graphics.beginBitmapFill( srcBd, null, true, true );
		graphics.drawTriangles( vertices, indices, uvs, TriangleCulling.NONE );
		old.x = mouseX;
		old.y = mouseY;
	}
	private function getClosestParticle():Particle 
	{
		var distMax:Float = Math.POSITIVE_INFINITY;
		var dist:Float;
		var _p:Particle;
		for( var p:Particle in corners )
		{
			dist = GeomUtils.distance( new Point( p.position.x, p.position.y ), new Point( mouseX, mouseY ) );
			if ( dist < distMax )
			{
				distMax = dist;
				_p = p;
			}
		}
		return _p;
	}
}