﻿// Clumper - Actionscript 3 Version by Jim Bumgardner 9/6/2008
package triga.systems.verlet3d
{
import nme.display.DisplayObjectContainer;
import nme.events.Event;
import nme.display.Sprite;
import nme.display.DisplayObject;
import nme.display.GradientType;
import nme.events.MouseEvent;
import nme.geom.Matrix;
import nme.geom.Point;
import nme.text.TextField;
class ClumpyClip3d extends VerletPoint3d
{
	static private var count:Float = 50; // Number of balls
	static public var balls:Array = [];
	static private var minRadius:Float = 10; // Radius of balls
	static private var maxRadius:Float = 10; // Radius of balls
	static private var interSpace:Float = 1.1; // Desired personal space 1.0 means balls are touching
												// Used to extend collision radius beyond visible radius
	static private var speed:Float = 1.0; // Controls how quickly balls move out of the way
	static private var centerWeight:Float; // Controls how strong the force towards the center is
	static private var damping:Float = .1; // Decreasing this makes the balls act like they are in thick oil
	static public var boundingBox:BoundingBox;
	static public var OX:Float, OY:Float, OZ:Float;
	static public var SW:Float, SH:Float, SD:Float;
	private var lx:Float, ly:Float, lz:Float;
	public var baseRadius:Float;
	static private var radToDeg:Float = 180/Math.PI;
	static private var degToRad:Float = Math.PI/180;
	public static function InitBalls( _boundingBox:BoundingBox, _count:Float = 50 ):Void
	{
		boundingBox = _boundingBox;
		OX = boundingBox.x;
		OY = boundingBox.y;
		OZ = boundingBox.z;
		SW = boundingBox.width;
		SH = boundingBox.height;
		SD = boundingBox.depth;
		count = _count;
		centerWeight = 1 / _count;
		while( _count-- ) 
		{
			new ClumpyClip3d();
		}
	}
	public function new()
	{
		x = Math.random() - .5;// Math.random() * SW;
		y = Math.random() - .5;// Math.random() * SH;
		z = Math.random() - .5;// Math.random() * SH;
		//x = OX + Math.random() * SW;
		//y = OY + Math.random() * SH;
		//z = OZ + Math.random() * SD;
		vx = vy = vz = 0;
		color = 0xFFFFFF;
		baseRadius = radius = minRadius + Math.random() * maxRadius;
		balls.push(this);
	}
	public function myHitTest(mc:ClumpyClip3d):Bool
	{
		var dx:Float = mc.x - x;
		var dy:Float = mc.y - y;
		var dz:Float = mc.z - z;
		var minDist:Float = (radius + mc.radius);
		return (dx * dx + dy * dy + dz * dz) < minDist * minDist * 3;
	}
	override public function update():Void
	{
		if ( !fixed )
package triga.systems.verlet3d;
		lx = x;
		ly = y;
		lz = z;
		vx *= damping;
		vy *= damping;
		vz *= damping;
		// Collision detection with balls (Avoidance)
		var weight:Float = 0;
		var fx:Float = 0;
		var fy:Float = 0;
		var fz:Float = 0;
		var dx:Float, dy:Float, dz:Float, dist:Float, scale:Float;
		for (i in 0 ... balls.length ) 
		{
			if (balls[i] != this && myHitTest(balls[i])) 
			{
				dx = x - balls[i].x;
				dy = y - balls[i].y;
				dz = z - balls[i].z;
				dist = Math.sqrt(dx * dx + dy * dy + dz * dz);
				var maxDist:Float = ( this.radius + balls[i].radius ) * interSpace;
				var diff:Float = maxDist - dist;
				if (diff > 0) 
				{
					scale = diff / maxDist;
					scale = scale * scale;
					weight += scale;
					scale *= speed / dist;
					fx += dx*scale;
					fy += dy*scale;
					fz += dz*scale;
				}
			}
		}
		// Attraction to center
		//dx = SW / 2 - x;
		//dy = SH / 2 - y;
		//dz = SD / 2 - z;
		dx = 0 - x;
		dy = 0 - y;
		dz = 0 - z;
		dist = Math.sqrt(dx * dx + dy * dy + dz * dz);
		weight += centerWeight;
		scale = centerWeight / dist;
		fx += dx * scale;
		fy += dy * scale;
		fz += dz * scale;
		vx += fx / weight;
		vy += fy / weight;
		vz += fz / weight;
		x += vx;
		y += vy;
		z += vz;
		}
	}
}