package triga.systems.verlet3d.deflectors;
import nme.Vector;
using nme.Vector;
import away3d.core.base.SubGeometry;
import away3d.entities.Mesh;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class MeshDeflector extends Deflector 
{
	private var faces:Vector<FaceDeflector> = new Vector<FaceDeflector>();
	public function new( damping:Float = .9, friction:Float = 1, mesh:Mesh = null ) 
	{
		super( damping, friction );
		var f:FaceDeflector;
		var vertices:Vector<Float>, indices:Vector<Int>;
		var id0:Int, id1:Int, id2:Int;
		for ( var sub:SubGeometry in mesh.geometry.subGeometries )
		{
			vertices = sub.vertexData;
			indices = sub.indexData;
			for (i in 0 ... indices.length; i+= 3 ) 
			{
				id0 = indices[ i ] * 3;
				id1 = indices[ i + 1 ] * 3;
				id2 = indices[ i + 2 ] * 3;
				f = new FaceDeflector( damping, friction );
				f.initFromValues( 	vertices[ id0 ], vertices[ id0 + 1 ], vertices[ id0 + 2 ],
									vertices[ id1 ], vertices[ id1 + 1 ], vertices[ id1 + 2 ],
									vertices[ id2 ], vertices[ id2 + 1 ], vertices[ id2 + 2 ]	); 
				faces.push( f );
			}
		}
	}
	override public function apply( id:Int ) :Bool
	{
		for( var f:FaceDeflector in faces ) 
		{
			if ( f.positions_x == null ) 
			{
				f.positions_x = positions_x;
				f.positions_y = positions_y;
				f.positions_z = positions_z;
				f.velocities_x = velocities_x;
				f.velocities_y = velocities_y;
				f.velocities_z = velocities_z;
			}
			if( f.apply( id ) ) return true;
		}
		return false;
	}
	public var damping( null, set_damping ):Float;
	override public function set_damping( value:Float ):Float
	{ 
		for( var f:FaceDeflector in faces ) f.damping = value;
		return damping;
	}
	public var friction( null, set_friction ):Float;
	override public function set_friction( value:Float ):Float
	{ 
		for( var f:FaceDeflector in faces ) f.friction = value;
		return friction;
	}
}