package triga.systems.verlet3d.deflectors;
import nme.Vector;
using nme.Vector;
import nme.geom.Vector3D;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class PlaneDeflector extends Deflector
{
	public var a:Float;
	public var b:Float;
	public var c:Float;
	public var d:Float;
	private var dx:Float;
	private var dy:Float;
	private var dz:Float;
	private var _v0:Vector3D, _v1:Vector3D, _v2:Vector3D;
	private var vectors:Vector<Float> = new Vector<Float>( 9 );
	private var u:Float;
	private var det:Float;
	private var dot:Float;
	public function new( damping:Float = .5, friction:Float = 1, v0:Vector3D = null, v1:Vector3D = null, v2:Vector3D = null )
	{
		super( damping, friction );
		if ( v0 != null && v1 != null && v2 != null ) initFromVecors( v0, v1, v2 );
	}
	public function initFromValues( v0x:Float, v0y:Float, v0z:Float, v1x:Float, v1y:Float, v1z:Float, v2x:Float, v2y:Float, v2z:Float	):IDeflector
	{
		return initFromVecors( 	new Vector3D( v0x, v0y, v0z ),
								new Vector3D( v1x, v1y, v1z ),
								new Vector3D( v2x, v2y, v2z ) );
	}
	public function initFromVecors( v0:Vector3D, v1:Vector3D, v2:Vector3D ):PlaneDeflector
	{
		this.v0 = v0;
		this.v1 = v1;
		this.v2 = v2;
		a = v0.y * (v1.z - v2.z) + v1.y * (v2.z - v0.z) + v2.y * (v0.z - v1.z);
		b = v0.z * (v1.x - v2.x) + v1.z * (v2.x - v0.x) + v2.z * (v0.x - v1.x);
		c = v0.x * (v1.y - v2.y) + v1.x * (v2.y - v0.y) + v2.x * (v0.y - v1.y);
		d = a * v0.x + b * v0.y + c * v0.z;
		normalize();
		return this;
	}
	private function normalize():Void
	{
		var len : Number = 1 / Math.sqrt(a*a + b*b + c*c);
		a *= len;
		b *= len;
		c *= len;
		d *= len;
	}
	override public function apply( id:Int ):Bool
	{
		det =( a * ( positions_x[ id ] + velocities_x[ id ] ) + 
				b * ( positions_y[ id ] + velocities_y[ id ] ) + 
				c * ( positions_z[ id ] + velocities_z[ id ] ) - d );
		if ( det < 0 ) 
		{
			var _o:Vector3D = new Vector3D( 	positions_x[ id ], 
												positions_y[ id ], 
												positions_z[ id ] );
			var _d:Vector3D = new Vector3D( 	_o.x + velocities_x[ id ], 
												_o.y + velocities_y[ id ],
												_o.z + velocities_z[ id ] );
			//var ip:Vector3D = getIntersection( o, d, this );
			dx = velocities_x[ id ];
			dy = velocities_y[ id ];
			dz = velocities_z[ id ];
			dot = ( a * dx + b * dy + c * dz );
			u = -( a * positions_x[ id ] + b * positions_y[ id ] + c * positions_z[ id ] - d ) / dot;
			var ip:Vector3D = new Vector3D( positions_x[ id ] + u * dx, positions_y[ id ] + u * dy, positions_z[ id ] + u * dz );
			//var rd:Vector3D = getReflection( _o, _d, this );
			dot *= 2;
			var rd:Vector3D = new Vector3D( -( dx - ( a * dot ) ), -( dy - ( b * dot ) ), -( dz - ( c * dot ) ) );
			var t:Vector3D = setResponseLength( _d, ip, rd );
			velocities_x[ id ] = t.x * damping;
			velocities_y[ id ] = t.y * damping;
			velocities_z[ id ] = t.z * damping;
			if ( abs( velocities_x[ id ] ) < 1 ) velocities_x[ id ] = 0;
			if ( abs( velocities_y[ id ] ) < 1 ) velocities_y[ id ] = 0;
			if ( abs( velocities_z[ id ] ) < 1 ) velocities_z[ id ] = 0;
			velocities_x[ id ] *= friction;
			velocities_y[ id ] *= friction;
			velocities_z[ id ] *= friction;
			positions_x[ id ] = ip.x;
			positions_y[ id ] = ip.y;
			positions_z[ id ] = ip.z;
			return true;
		}
		return false;
	}
	//private function getIntersection( o:Vector3D, d:Vector3D):Vector3D 
	//{
		//var dx:Float = ( d.x - o.x );
		//var dy:Float = ( d.y - o.y );
		//var dz:Float = ( d.z - o.z );
		//
		//var u:Float = -( a * o.x + b * o.y + c * o.z - d ) / ( a * dx + b * dy + c * dz );
		//
		//return new Vector3D( o.x + u * dx, o.y + u * dy, o.z + u * dz );
	//}
	//private function getReflection( o:Vector3D, d:Vector3D, plane:PlaneDeflector):Vector3D 
	//{
		//var v:Vector3D = o.subtract( d );
		//
		//var n:Vector3D = new Vector3D( plane.a, plane.b, plane.c );
		//n.scaleBy( n.dotProduct( v ) * 2 );
		//
		//return new Vector3D( -( v.x - n.x ), -( v.y - n.y ), -( v.z - n.z ) );
	//}
	private function setResponseLength( d:Vector3D, ip:Vector3D, rd:Vector3D):Vector3D 
	{
		var t:Vector3D = ip.subtract( rd );
		t.normalize();
		t.scaleBy( -Vector3D.distance( ip, d ) );
		t = t.add( ip );
		return t;
	}
	private function abs( value:Float ):Float 
	{
		if( value < 0 ) return ( value * -1 );
		return value;
	}
	public var v0( get_v0, set_v0 ):Vector3D;
	public function get_v0():Vector3D 
	{
		return _v0;
	}
	public function set_v0(value:Vector3D):Vector3D 
	{
		_v0 = value;
		vectors[ 0 ] = value.x;
		vectors[ 1 ] = value.y;
		vectors[ 2 ] = value.z;
		if( v0 != null && v1 != null && v2 != null ) invalidateGeometry();
		return v0;
	}
	public var v1( get_v1, set_v1 ):Vector3D;
	public function get_v1():Vector3D 
	{
		return _v1;
	}
	public function set_v1(value:Vector3D):Vector3D 
	{
		_v1 = value;
		vectors[ 3 ] = value.x;
		vectors[ 4 ] = value.y;
		vectors[ 5 ] = value.z;
		if( v0 != null && v1 != null && v2 != null ) invalidateGeometry();
		return v1;
	}
	public var v2( get_v2, set_v2 ):Vector3D;
	public function get_v2():Vector3D 
	{
		return _v2;
	}
	public function set_v2(value:Vector3D):Vector3D 
	{
		_v2 = value;
		vectors[ 6 ] = value.x;
		vectors[ 7 ] = value.y;
		vectors[ 8 ] = value.z;
		if( v0 != null && v1 != null && v2 != null ) invalidateGeometry();
		return v2;
	}
	public function invalidateGeometry():Void 
	{
		//to be overridden
	}
}