package triga.systems.verlet3d.deflectors;
import nme.geom.Vector3D;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class SphereDeflector extends Deflector 
{
	private var _radius:Float;
	private var radius2:Float;
	private var _position:Vector3D;
	private var pp:Vector3D;
	private var dx:Float, dy:Float, dz:Float;
	public function new( position:Vector3D, radius:Float, damping:Float = 1, friction:Float = 1 ) 
	{
		super( damping, friction );
		this.position = position;
		this.radius = radius;
		radius2 = radius * radius;
	}
	override public function apply( id:Int ):Bool
	{
		dx = position.x - (positions_x[ id ] + velocities_x[ id ] );
		dy = position.y - (positions_y[ id ] + velocities_y[ id ] );
		dz = position.z - (positions_z[ id ] + velocities_z[ id ] );
		var dist:Float = dx * dx + dy * dy + dz * dz;
		if ( dist < radius2 ) 
		{
			pp = new Vector3D( -dx, -dy, -dz );
			pp.normalize();
			velocities_x[ id ] = pp.x * damping;
			velocities_y[ id ] = pp.y * damping;
			velocities_z[ id ] = pp.z * damping;
			velocities_x[ id ] *= friction;
			velocities_y[ id ] *= friction;
			velocities_z[ id ] *= friction;
			pp.scaleBy( radius );
			pp = pp.add( position );
			positions_x[ id ] = pp.x;
			positions_y[ id ] = pp.y;
			positions_z[ id ] = pp.z;
			return true;
		}
		return false;
	}
	private function IntersectRaySphere( p:Vector3D, d:Vector3D ):Vector3D
	{
		var ip:Vector3D = new Vector3D();
		var m:Vector3D = p.subtract( position );
		var b:Float = m.dotProduct( d );
		var c:Float = m.dotProduct( m ) - radius2;
		// Exit if r’s origin outside s (c > 0) and r pointing away from s (b > 0)
		if (c > 0 && b > 0 ) return null;
		var discr:Float = b*b - c;
		// A negative discriminant corresponds to ray missing sphere
		if (discr < 0 ) return null;
		// Ray now found to intersect sphere, compute smallest t value of intersection
		var t:Float = -b - Math.sqrt(discr);
		// If t is negative, ray started inside sphere so clamp t to zero
		if (t < 0 ) t = 0;
		ip.x = p.x + t * d.x;
		ip.y = p.y + t * d.y;
		ip.z = p.z + t * d.z;
		return ip;
	}
	public var position( get_position, set_position ):Vector3D;
	public function get_position():Vector3D 
	{
		return _position;
	}
	public function set_position(value:Vector3D):Vector3D 
	{
		_position = value;
		return position;
	}
	public var radius( get_radius, set_radius ):Float;
	public function get_radius():Float 
	{
		return _radius;
	}
	public function set_radius(value:Float):Float 
	{
		_radius = value;
		radius2 = value * value;
		return radius;
	}
}