package triga.systems.verlet3d.forces;
import nme.geom.Matrix3D;
import nme.geom.Vector3D;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Motor extends Force
{
	private var _matrix:Matrix3D;
	private var axis:Vector3D;
	private var _origin:Vector3D;
	private var _direction:Vector3D;
	private var _radius:Float;
	private var _strength:Float;
	private var p:Vector3D = new Vector3D();
	private var tmp:Vector3D = new Vector3D();
	private var dir:Vector3D = new Vector3D();
	private var pos:Vector3D = new Vector3D();
	public function new( origin:Vector3D, direction:Vector3D, radius:Float = 10, strength:Float = 5 ) 
	{
		_matrix = new Matrix3D();
		this.direction = direction;
		this.origin = origin;
		this.radius = radius;
		this.strength = strength;
	}
	override public function apply( id:Int ):Void
	{
		//get distance to the axis
		p.x = positions_x[ id ];
		p.y = positions_y[ id ];
		p.z = positions_z[ id ];
		pos.x = p.x - _origin.x;
		pos.y = p.y - _origin.y;
		pos.z = p.z - _origin.z;
		dir.normalize();
		dir.scaleBy( dir.dotProduct( pos ) );
		tmp = origin.add( dir );
		var dist:Float = Vector3D.distance( p, tmp );
		if ( dist < radius )
		{
			_matrix.identity();
			_matrix.appendRotation( strength, axis, tmp );
			p = _matrix.deltaTransformVector( p );
			dist = 1 / dist;
			forces_x[ id ] += abs( p.x - positions_x[ id ] ) * dist;
			forces_y[ id ] += abs( p.y - positions_y[ id ] ) * dist;
			forces_z[ id ] += abs( p.z - positions_z[ id ] ) * dist;
		}
	}
	private function abs( value:Float ):Float 
	{
		if( value < 0 ) return ( value * -1 );
		return value;
	}
	private function resetAxis():Void 
	{
		if ( _origin != null && _direction != null )
		{
			axis = origin.subtract( direction );
			dir = axis.clone();
		}
	}
	public var origin( get_origin, set_origin ):Vector3D;
	public function get_origin():Vector3D 
	{
		return _origin;
	}
	public function set_origin(value:Vector3D):Vector3D 
	{
		_origin = value;
		resetAxis();
		return origin;
	}
	public var direction( get_direction, set_direction ):Vector3D;
	public function get_direction():Vector3D 
	{
		return _direction;
	}
	public function set_direction(value:Vector3D):Vector3D 
	{
		_direction = value;
		resetAxis();
		return direction;
	}
	public var radius( get_radius, set_radius ):Float;
	public function get_radius():Float 
	{
		return _radius;
	}
	public function set_radius(value:Float):Float 
	{
		_radius = value;
		return radius;
	}
	public var strength( get_strength, set_strength ):Float;
	public function get_strength():Float 
	{
		return _strength;
	}
	public function set_strength(value:Float):Float 
	{
		_strength = value;
		return strength;
	}
}