package triga.utils;
import nme.Vector;
using nme.Vector;
import nme.events.Event;
import nme.events.EventDispatcher;
import nme.geom.Point;
import nme.geom.Vector3D;
import triga.shapes.Tetrahedron;
import triga.shapes.TrigaFace;
import triga.shapes.TrigaRay;
/**
 * @author nicoptere
 * http://en.nicoptere.net/
 */
class Delaunay
{
	static private var mesh:MeshUtils;
	/**
	 * performs a Delaunay tetrahedralization on a set of vertices 
	 * @param vertices the original Vector of vertices
	 * @return a Vector<Tetrahedron> of delaunay tetras
	 * 
	 */
	public static function compute( vertices:Vector<Float>, indices:Vector<Int> = null ):Vector<Tetrahedron>
	{
		//those will be used quite everywhere so I am storing them here not to declare them x times
		var i:Int, j:Int, k:Int;
		var nv:Int = vertices.length;
		if ( nv < 4 ) return null;
		if ( indices != null ) mesh = new MeshUtils( vertices, indices );
		else mesh = null
		var xmin:Float = Math.POSITIVE_INFINITY;
		var xmax:Float = Math.NEGATIVE_INFINITY;
		var ymin:Float = Math.POSITIVE_INFINITY;
		var ymax:Float = Math.NEGATIVE_INFINITY;
		var zmin:Float = Math.POSITIVE_INFINITY;
		var zmax:Float = Math.NEGATIVE_INFINITY;
		for ( i in 0... nv; i+=3 )
		{
			//vertices[ i ] = new Vector3D( vertices[i].x, vertices[i].y, vertices[i].z );
			if (vertices[i] < xmin) xmin = vertices[i];
			if (vertices[i] > xmax) xmax = vertices[i];
			if (vertices[i+1] < ymin) ymin = vertices[i+1];
			if (vertices[i+1] > ymax) ymax = vertices[i+1];
			if (vertices[i+2] < zmin) zmin = vertices[i+2];
			if (vertices[i+2] > zmax) zmax = vertices[i+2];
		}
		var dmax:Float = Math.max( xmax - xmin, ymax - ymin, zmax - zmin ) * 2;
		//building a super tetrahedron
		var tetras:Vector<Tetrahedron> = new Vector<Tetrahedron>();
		vertices.push( -0.798892958090 * dmax, 	-0.274518427439	* dmax, 	0.535172563978 * dmax,
						0.337674633600 * dmax, 	-0.783974919468 * dmax, 	-0.520921458490 * dmax,
						0.717804165557 * dmax, 	0.325344236568 	* dmax, 	0.615555282682 * dmax,
					 -0.256585841067 * dmax, 	0.733149110340 	* dmax, 	-0.629806388170 * dmax 	);
		var index:Int = nv / 3;
		var superTetra:Tetrahedron = new Tetrahedron( vertices, index, index + 1, index + 2, index + 3 );
		tetras.push( superTetra ); 
		//some vars 
		var faces:Vector<TrigaFace> = new Vector<TrigaFace>();
		var tetra:Tetrahedron, _tetra:Tetrahedron;
		var vertex:Vector3D = new Vector3D();
		var f:TrigaFace, _f:TrigaFace;
		// Include each point one at a time into the existing mesh
		for ( i in 0... vertices.length; i+=3 )
		{
			index = i / 3;
			vertex.x = vertices[ i ];
			vertex.y = vertices[ i + 1 ];
			vertex.z = vertices[ i + 2 ];
			for ( j in 0... tetras.length )
			{
				tetra = tetras[ j ];
				//si le point est dans la circumSphere
				if ( TrigaSphereUtils.contains( tetra.sphere, vertex ) )
				{
					//push the tetra's faces 
					for( f in tetra.faces ) faces.push( f );
					//removes the tetra from list
					tetras.splice( j,1 );
					j--;
					continue;
				}
			}
			for ( j = faces.length - 2; j >= 0; j--)
			{
				for ( k = faces.length - 1; k >= j + 1; k--)
				{
					if ( faces[ j ].isPairOrEqual( faces[ k ] ) )
					{
						faces.splice( k, 1 );
						faces.splice( j, 1 );
						k--;
						continue;
					}
				}
			}
			// Form new tetras for the current point
			// Skipping over any tagged faces.
			for ( f in faces )
			{
				if ( f != null ) 
				{
					_tetra = new Tetrahedron( vertices, index, f.i0, f.i1, f.i2 );
				//	if ( mesh != null && mesh.pointInPolyhedra( tetra.centroid ) ) tetras.push( _tetra );
					//else 
					tetras.push( _tetra );
				}
			}
		}
		// Remove tetras sharing supertriangle vertices
		for ( i = tetras.length - 1; i >= 0; i--)
		{
			tetra = tetras[ i ];
			if ( 	tetra.shareVertex( superTetra.i0 ) 
			||		tetra.shareVertex( superTetra.i1 ) 
			||		tetra.shareVertex( superTetra.i2 ) 
			||		tetra.shareVertex( superTetra.i3 ) )
			{
				tetras.splice(i, 1);
			}
		}
		return tetras;
	}
}