package triga.utils;
import nme.Vector;
using nme.Vector;
import away3d.tools.utils.Ray;
import nme.events.Event;
import nme.events.EventDispatcher;
import nme.geom.Vector3D;
import nme.utils.setTimeout;
import triga.shapes.Tetrahedron;
import triga.shapes.TrigaFace;
import triga.shapes.TrigaSphere;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class DelaunayCleaner extends EventDispatcher 
{
	static public var tetras:Vector<Tetrahedron>;
	static private var vertices:Vector<Float>;
	static private var indices:Vector<Int>;
	static private var currentTetra:Int;
	static private var faces:Vector<TrigaFace>;
	static private var tmp:Vector<Tetrahedron>;
	static private var face:TrigaFace;
	static private var tetra:Tetrahedron;
	static public var lastTetra:Tetrahedron;
	private var ray:Ray	= new Ray();
	private var axis:Vector3D;
	static private var AXES:Vector<Vector3D> = Vector<Vector3D>(	[
																	new Vector3D( 10000000000,0,0 ),
																	new Vector3D( 0, 10000000000,0 ),
																	new Vector3D( 0, 0, 10000000000 ),
																	new Vector3D( -10000000000,0,0 ),
																	new Vector3D( 0, -10000000000,0 ),
																	new Vector3D( 0, 0, -10000000000 )
																	]);
	public function new() 
	{
	}
	/**
	 * removes tetrahedra lying outside the volume.
	 * very costly operation, Delaunay.getInstance() will dispatch an Event.COMPLETE when done
	 * @param	tetras
	 * @param	vertices
	 * @param	indices
	 */
	public function cleanUp( tetras:Vector<Tetrahedron>, vertices:Vector<Float>, indices:Vector<Int> ):Void
	{
		DelaunayCleaner.tetras = tetras;
		DelaunayCleaner.currentTetra = 0;
		var i:Int;
		faces = new Vector<TrigaFace>();
		tmp = new Vector<Tetrahedron>();
		//builds the mesh's faces
		for ( i in 0... indices.length; i+=3 )
		{
			face = new TrigaFace( vertices, indices[ i ], indices[ i + 1 ], indices[ i + 2 ] );
			faces.push( face );
			face.buildVector3D();
		}
		_trace( 'faces:', faces.length );
		_trace( 'tetras:', tetras.length );
		recursiveTest();
	}
	private function recursiveTest():Void
	{
		//_trace( 'recursive test' );
		var minAngle:Float = Math.PI / 180 * 5;
		GlobalTextField.instance.text = '';
		if ( currentTetra < tetras.length )
		{
			_trace( 'testing tetra # ' + currentTetra, ' / ', tetras.length );
			lastTetra = tetras[ currentTetra ];
			var ratio:Float = 3 * ( lastTetra.inSphere.radius / lastTetra.sphere.radius );
			//_trace( 'ratio ' + ratio );
			if ( ratio >= .3 )
			{
				_trace( lastTetra._dihedralAngles[ 0 ] );
				_trace( lastTetra._dihedralAngles[ 1 ] );
				_trace( lastTetra._dihedralAngles[ 2 ] );
				_trace( lastTetra._dihedralAngles[ 3 ] );
				if ( 
						lastTetra._dihedralAngles[ 0 ] > minAngle
				&&		lastTetra._dihedralAngles[ 1 ] > minAngle
				&&		lastTetra._dihedralAngles[ 2 ] > minAngle
				&&		lastTetra._dihedralAngles[ 3 ] > minAngle
				)
				{
					if ( isTetraInside( lastTetra ) )
					{
						//if ( sharesFace( lastTetra ) ){
							tmp.push( lastTetra );
							dispatchEvent( new Event( Event.CHANGE ) );
						//}
					}
				}
			}
			currentTetra++;
			setTimeout( recursiveTest, 0 );
		}
		else
		{
			tetras = tmp.copy();
			dispatchEvent( new Event( Event.COMPLETE ) );
		}
	}
	private function sharesFace( tetra:Tetrahedron ):Bool
	{
		//var f:TrigaFace;
		for( face in faces ) if( tetra.hasCommonFace( face ) )return true
		//{
			//f = tetra.hasCommonFace( face );
			//if ( f != null ) 
			//{
				//if ( f.isFacing( tetra.centroid ) )
				//{
					//return false;
				//}
				//else
				//{
					//return true;
				//}
			//}
		//}
		return false;
	}
	private function isTetraInside( tetra:Tetrahedron ):Bool
	{
		var count0:Int = 0, count1:Int = 0;
		var p:Vector3D = tetra.centroid;
		var shareFace:Bool = false;
		for( face in faces ) 
		{
			//front
			axis = AXES[0];
			axis.y = p.y;
			axis.z = p.z;
			if ( ray.getRayToTriangleIntersection( p, axis, face._v0, face._v1, face._v2 ) != null )
			{
				count0++;
			}
			axis = AXES[1];
			axis.x = p.x;
			axis.z = p.z;
			if ( ray.getRayToTriangleIntersection( p, axis, face._v0, face._v1, face._v2 ) != null )
			{
				count0++;
			}
			axis = AXES[2];
			axis.x = p.x;
			axis.y = p.y;
			if ( ray.getRayToTriangleIntersection( p, axis, face._v0, face._v1, face._v2 ) != null )
			{
				count0++;
			}
			//back
			/*
			axis = AXES[3];
			axis.y = p.y;
			axis.z = p.z;
			if ( ray.getRayToTriangleIntersection( p, axis, face._v0, face._v1, face._v2 ) != null )
			{
				count1++;
			}
			axis = AXES[4];
			axis.x = p.x;
			axis.z = p.z;
			if ( ray.getRayToTriangleIntersection( p, axis, face._v0, face._v1, face._v2 ) != null )
			{
				count1++;
			}
			axis = AXES[5];
			axis.x = p.x;
			axis.y = p.y;
			if ( ray.getRayToTriangleIntersection( p, axis, face._v0, face._v1, face._v2 ) != null )
			{
				count1++;
			}
			*/
		}
		//_trace( 'faces hit:', hitFace.length, 'count:', count );
		//if ( !shareFace ) return false;
		return ( count0 % 2 == 1 );// || ( count1 % 2 == 1 );
	}
}