package triga.utils;
import nme.Vector;
using nme.Vector;
import away3d.containers.View3D;
import away3d.entities.Mesh;
import away3d.materials.ColorMaterial;
import away3d.primitives.Cube;
import nme.geom.Vector3D;
import triga.spline.curves.Cubic;
import triga.spline.curves.ICurve3D;
import triga.utils.MeshUtils;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Subdivision 
{
	static private var subdivisions:Vector<Vector3D> = new Vector<Vector3D>();
	static public function compute( mesh:Mesh, method:ICurve3D = null, offset:Float = .5, doubleSided:Bool = false, inner:Bool = false ):Mesh
	{
		if ( method == null ) method = new Cubic( .5, true );
		var vertices:Vector<Float> = mesh.geometry.subGeometries[ 0 ].vertexData;
		var indices:Vector<Int> = mesh.geometry.subGeometries[ 0 ].indexData;
		var vs:Vector<Float> = new Vector<Float>();
		var ind:Vector<Int> = new Vector<Int>();
		var tv0:Vector3D = new Vector3D(), tv1:Vector3D = new Vector3D(), tv2:Vector3D = new Vector3D();
		var triangleVectors:Vector<Vector3D> = Vector<Vector3D>([tv0, tv1, tv2]);
		var edgeVectors:Vector<Vector3D> = new Vector<Vector3D>();
		var ev:Vector3D = new Vector3D();
		var splineVectors:Vector<Vector3D> = new Vector<Vector3D>();
		var sv:Vector3D = new Vector3D();
		var c:Vector3D = new Vector3D();
		var i:Int, j:Int;
		var third:Float = 1 / 3;
		var segments:Int = 1 / method.precision;
		var innerOffset:Float = offset;
		var i0:Int, i1:Int, i2:Int, ic:Int;
		var n0:Int, n1:Int, n2:Int;
		for ( i in 0... indices.length; i += 3 )
		{
			//current indices
			i0 = indices[ i ];
			i1 = indices[ i + 1 ];
			i2 = indices[ i + 2 ];
			//vertices
			i0 *= 3;
			i1 *= 3;
			i2 *= 3;
			//face center
			c.x = ( vertices[ i0 ] + vertices[ i1 ] + vertices[ i2 ] ) * third;
			c.y = ( vertices[ i0 + 1 ] + vertices[ i1 + 1 ] + vertices[ i2 + 1 ] ) * third;
			c.z = ( vertices[ i0 + 2 ] + vertices[ i1 + 2 ] + vertices[ i2 + 2 ] ) * third;
			//
			//if ( offset == -1 ) innerOffset = Math.random() < .25 ? 1 : ( .1 + Math.random() * .8 );
			tv0.x = vertices[ i0 ] + ( c.x - vertices[ i0 ] ) * innerOffset;
			tv0.y = vertices[ i0 + 1 ] + ( c.y - vertices[ i0 + 1 ] ) * innerOffset;
			tv0.z = vertices[ i0 + 2 ] + ( c.z - vertices[ i0 + 2 ] ) * innerOffset;
			tv1.x = vertices[ i1 ] + ( c.x - vertices[ i1 ] ) * innerOffset;
			tv1.y = vertices[ i1 + 1 ] + ( c.y - vertices[ i1 + 1 ] ) * innerOffset;
			tv1.z = vertices[ i1 + 2 ] + ( c.z - vertices[ i1 + 2 ] ) * innerOffset;
			tv2.x = vertices[ i2 ] + ( c.x - vertices[ i2 ] ) * innerOffset;
		 tv2.y = vertices[ i2 + 1 ] + ( c.y - vertices[ i2 + 1 ] ) * innerOffset;
			tv2.z = vertices[ i2 + 2 ] + ( c.z - vertices[ i2 + 2 ] ) * innerOffset;
			splineVectors = method.compute( triangleVectors );
			if ( inner )
			{
				n0 = vs.length / 3;
				ic = n0 + splineVectors.length;
				for ( j in 0... splineVectors.length )
				{
					ind.push( ic, n0 + j, n0 + ( j + 1 ) % splineVectors.length ); 
					vs.push( splineVectors[ j ].x, splineVectors[ j ].y, splineVectors[ j ].z );
				}
				vs.push( c.x, c.y, c.z );
			}
			else
			{
				tv0.x = vertices[ i0 ];
				tv1.x = vertices[ i1 ];
				tv2.x = vertices[ i2 ];
				tv0.y = vertices[ i0 + 1 ];
				tv1.y = vertices[ i1 + 1 ];
				tv2.y = vertices[ i2 + 1 ];
				tv0.z = vertices[ i0 + 2 ];
				tv1.z = vertices[ i1 + 2 ];
				tv2.z = vertices[ i2 + 2 ];
				edgeVectors = subdiv( triangleVectors, segments );
				j = 0;
				while ( splineVectors.length > edgeVectors.length ) splineVectors.splice ( j++*segments, 1 );
				n0 = vs.length / 3;
				n1 = n0 + 1;
				n2 = n0 + 2;
				vs.push( 	vertices[ i0 ],vertices[ i0 + 1 ],vertices[ i0 + 2 ],
							vertices[ i1 ],vertices[ i1 + 1 ],vertices[ i1 + 2 ],
							vertices[ i2 ],vertices[ i2 + 1 ],vertices[ i2 + 2 ]	);
				i1 = vs.length / 3;
				var oid:Int, nid:Int, id:Int
				for ( j in 0... splineVectors.length )
				{
					if ( ( j % segments ) == 0 )
					{
						id = Math.floor( j / segments );
						i0 = id == 0 ? n0 : id == 1 ? n1 : n2;
						i2 = id == 0 ? n2 : id == 1 ? n0 : n1;
						ind.push( i0, i2, i1 + j );
					}
					i2 = i1 + ( j + 1 ) % splineVectors.length;
					ind.push( i0, i1 + j, i2 );
					vs.push( splineVectors[ j ].x, splineVectors[ j ].y, splineVectors[ j ].z );
				}
			}
		}
		if ( doubleSided ) ind = ind.concat( ind.copy().reverse() );
		return MeshUtils.buildMesh( vs, ind );
	}
	static private function subdiv( vertices:Vector<Vector3D>, segments:Int ):Vector<Vector3D>
	{
		subdivisions = new Vector<Vector3D>();
		var i:Int, j:Int, k:Int = 0;
		var v:Vector3D, n:Vector3D;
		var step:Float = 1 / segments;
		for ( i in 0... vertices.length )
		{
			v = vertices[ i ];
			n = vertices[ (i + 1) % vertices.length ];
			for ( j in 0... segments )
			{
				subdivisions.push( new Vector3D( 	v.x + ( n.x - v.x ) * ( j * step ),
													v.y + ( n.y - v.y ) * ( j * step ),
													v.z + ( n.z - v.z ) * ( j * step ) ) );
			}
		}
		return subdivisions;
	}
	static public function wireframe( mesh:Mesh, thickness:Float, doubleSided:Bool = false ):Mesh
	{
		var vertices:Vector<Float> = mesh.geometry.subGeometries[ 0 ].vertexData;
		var indices:Vector<Int> = mesh.geometry.subGeometries[ 0 ].indexData;
		var i:Int; 
		var i0:Int, i1:Int, i2:Int;
		var n0:Int, n1:Int, n2:Int;
		var third:Float = 1 / 3;
		var c:Vector3D = new Vector3D();
		var vn0:Vector3D, vn1:Vector3D, vn2:Vector3D;
		var vs:Vector<Float> = vertices.copy();
		var ind:Vector<Int> = new Vector<Int>();
		for ( i in 0... indices.length; i += 3)
		{
			//current indices
			i0 = indices[ i ];
			i1 = indices[ i + 1 ];
			i2 = indices[ i + 2 ];
			//new indices
			n0 = vs.length / 3;
			n1 = n0 + 1;
			n2 = n0 + 2;
			ind.push( 	i0, i1, n0,
						i1, n1, n0,
						i1, i2, n1, 
						i2, n2, n1,
						i2, i0, n2, 
						i0, n0, n2	);
			//vertices
			i0 *= 3;
			i1 *= 3;
			i2 *= 3;
			//face center
			c.x = ( vertices[ i0 ] + vertices[ i1 ] + vertices[ i2 ] ) * third;
			c.y = ( vertices[ i0 + 1 ] + vertices[ i1 + 1 ] + vertices[ i2 + 1 ] ) * third;
			c.z = ( vertices[ i0 + 2 ] + vertices[ i1 + 2 ] + vertices[ i2 + 2 ] ) * third;
			//new Vertices
			vs.push( 	vertices[ i0 ] + ( c.x - vertices[ i0 ] ) * thickness,vertices[ i0 + 1 ] + ( c.y - vertices[ i0 + 1 ] ) * thickness,vertices[ i0 + 2 ] + ( c.z - vertices[ i0 + 2 ] ) * thickness,
						vertices[ i1 ] + ( c.x - vertices[ i1 ] ) * thickness,vertices[ i1 + 1 ] + ( c.y - vertices[ i1 + 1 ] ) * thickness,vertices[ i1 + 2 ] + ( c.z - vertices[ i1 + 2 ] ) * thickness,
						vertices[ i2 ] + ( c.x - vertices[ i2 ] ) * thickness,vertices[ i2 + 1 ] + ( c.y - vertices[ i2 + 1 ] ) * thickness,vertices[ i2 + 2 ] + ( c.z - vertices[ i2 + 2 ] ) * thickness	);
		}
		if ( doubleSided ) ind = ind.concat( ind.copy().reverse() );
		mesh.geometry.subGeometries[ 0 ].updateVertexData( vs );
		mesh.geometry.subGeometries[ 0 ].updateIndexData( ind );
		return mesh;
	}
}