package net.nicoptere.ca.ca;
import haxe.Timer;
import nme.display.Bitmap;
import nme.geom.Point;
import nme.geom.Rectangle;
import nme.text.TextField;
import nme.text.TextFieldAutoSize;
import nme.display.BitmapData;
import nme.display.Sprite;

/**
 * @author Nicolas Barradeau 
 */

class CellularAutomata_1 extends Sprite
{
	private var w:Float;
	private var h:Float;
	
	private var bd:BitmapData;
	private var bmp:Bitmap;
	private var tf:TextField;
	
	private var rules:Array<Array<Int>>;
	private var ruleset:Array<Int>;
	private var ruleId:Int = 0;
	
	public function new( w:Float, h:Float ) 
	{
		super();
		
		this.w = w;
		this.h = h;
		
		bd = new BitmapData( Std.int( w ), Std.int( h ), false, 0xFFFFFF );
		bmp = new Bitmap( bd );
		addChild( bmp );
		
		tf = new TextField();
		tf.background = true;
		tf.autoSize = TextFieldAutoSize.LEFT;
		addChild( tf );
		
		rules = new Array<Array<Int>>();
		ruleset = new Array<Int>();
		
		/*
			rule ID					0		...		255
			rules[ 0 ] | 000 = 	[	?		...		 ?	]
			rules[ 1 ] | 001 = 	[	?       ...    	 ?	]
			rules[ 2 ] | 010 = 	[	?       ...    	 ?	]
			rules[ 3 ] | 011 = 	[	?       ...    	 ?	]
			rules[ 4 ] | 100 = 	[	?       ...    	 ?	]
			rules[ 5 ] | 101 = 	[	?       ...    	 ?	]
			rules[ 6 ] | 110 = 	[	?       ...    	 ?	]
			rules[ 7 ] | 111 = 	[	?       ...    	 ?	]
			for ( i in 0...8  )
			{
				
				trace( '\t\t rules[ ' + i + ' ] \t\t | \t ' + int_to_bin( i, 3 ).join('') +'  = 	[	?		...		 ?	]' );
				
			}
		*/ 
		
		for ( i in 0...8 )
		{
			
			ruleset = new Array<Int>();
			rules.push( ruleset );
			
		}
		
		/*
			table de correspondance:
				
			rule ID					0		1		2		3		...		255
			rules[ 0 ] | 000 = 	[	0		1		0       1				 1	]
			rules[ 1 ] | 001 = 	[	0       0       1       1           	 1	]
			rules[ 2 ] | 010 = 	[	0       0       0       0           	 1	]
			rules[ 3 ] | 011 = 	[	0       0       0       0           	 1	]
			rules[ 4 ] | 100 = 	[	0       0       0       0           	 1	]
			rules[ 5 ] | 101 = 	[	0       0       0       0           	 1	]
			rules[ 6 ] | 110 = 	[	0       0       0       0           	 1	]
			rules[ 7 ] | 111 = 	[	0       0       0       0           	 1	]
			
			
		*/
		
		for ( i in 0...0xFF + 1  )
		{
			
			var binaryValues:Array<Int> = int_to_bin( i, 8 );
			
			for ( i in 0...8 )
			{
				rules[ i ].push( binaryValues[ i ] );
			}
			
		}
		
		//juste pour vérifier:
		
		/*
		var ruleId:Int = 30;
		for ( i in 0...8 )
		{
			
			trace( '\t RULE '+ ruleId +' :' + i + ' | ' + int_to_bin( i, 3 ).join('') + ' = ' + rules[ i ][ ruleId ] );
			
		}
		return;
		//*/
		
		
		
		//retrouver un index à partir de 3 Int 0 | 1 
		
		/*
		
		trace( bin_to_int( 0, 0, 0 ) );// 0
		trace( bin_to_int( 0, 0, 1 ) );// 1
		trace( bin_to_int( 0, 1, 0 ) );// 2
		trace( bin_to_int( 0, 1, 1 ) );// 3
		trace( bin_to_int( 1, 0, 0 ) );// 4
		trace( bin_to_int( 1, 0, 1 ) );// 5
		trace( bin_to_int( 1, 1, 0 ) );// 6
		trace( bin_to_int( 1, 1, 1 ) );// 7
		
		return;
		//*/
		
		//ou avec des boolean
		
		/*
		trace( bool_to_int( false, false, false ) );// 0
		trace( bool_to_int( false, false, true 	) );// 1
		trace( bool_to_int( false, true, false 	) );// 2
		trace( bool_to_int( false, true, true 	) );// 3
		trace( bool_to_int( true, false, false 	) );// 4
		trace( bool_to_int( true, false, true 	) );// 5
		trace( bool_to_int( true, true, false 	) );// 6
		trace( bool_to_int( true, true, true 	) );// 7
		return;
		//*/
		
		var timer:Timer = new Timer( 1000 );
		timer.run = reset;
		reset();
		
	}
	
	private function reset() :Void
	{
		
		bd.fillRect( bd.rect, 0 );
		bd.setPixel( Std.int( bd.width / 2 ), 0, 0xFFFFFF );
		
		tf.text = 'rule id: ' + ruleId;
		for ( i in 0...bd.height ) 
		{
			scanLine( ruleId, i );
		}
		ruleId++;
		ruleId %= 0xFF;
		
	}
	
	private function int_to_bin( k:Int, base:Int = 3 ):Array<Int>
	{
		
		var bin:Array<Int> = [];
		
		while ( k > 0 )
		{
			
			bin.push( k % 2 );
			k = Std.int( k / 2 );
			
		}
		
		while ( bin.length < base ) bin.push( 0 );
		
		bin.reverse();
		return bin;
		
	}
	
	private inline function bin_to_int( a:Int, b:Int, c:Int ):Int
	{
		
		return a << 2 | b << 1 | c;
		
	}
	
	private inline function bool_to_int( a:Bool, b:Bool, c:Bool ):Int
	{
		
		return Std.int( a ? 1 : 0 ) << 2 | Std.int( b ? 1 : 0 ) << 1 | Std.int( c ? 1 : 0 );
		
	}
	
	private function scanLine( ruleId:Int, line:Int ):Void
	{
		
		
		for ( i in 1...bd.width-1 )
		{
			
			var output:Int = applyRule( i, line, ruleId );
			
			bd.setPixel( i, line + 1, output );
			
		}
		
	}
	
	private function applyRule( x:Int, y:Int, ruleId:Int ) :Int
	{
		
		var id:Int = bool_to_int( 	bd.getPixel( x - 1	, y ) == 0,
									bd.getPixel( x 		, y ) == 0,
									bd.getPixel( x + 1	, y ) == 0	);
		
		return rules[ id ][ ruleId ] * 0xFFFFFF; 
		
	}
	
}