package net.nicoptere.ca.gol;
import flash.display.BitmapData;

/**
 * @author Nicolas Barradeau 
 */

class GOLCell extends Cell
{

	/**
		rules:
		1 - Any live cell with fewer than two live neighbours dies, as if caused by under-population.
		2 - Any live cell with two or three live neighbours lives on to the next generation.
		3 - Any live cell with more than three live neighbours dies, as if by overcrowding.
		4 - Any dead cell with exactly three live neighbours becomes a live cell, as if by reproduction.
	 */
	
	public function new( x:Int, y:Int, state:Int = 0 ) 
	{
		super( x, y, state );
	}
	
	override public function evaluate():Void 
	{
		
		//is cell dead or alive?
		
		//count the alive neighbors around the cell
		
		var count:Int = 0;
		
		for ( c in neighbors ) 
		{
			if ( c.state == 1 )	count++;
		}
		
		//rule 1 : underpopulation
		
		if ( count < 2 )
		{
			nextState = 0;
		}
		
		//rule 2 : cool spot
		
		if ( count == 2 || count == 3 )
		{
			if( state == 1 ) nextState = 1;
		}
		
		//rule 3 : overpopulation
		
		if ( count > 3 )
		{
			if( state == 1 ) nextState = 0;
		}
		
		//rule 4: it's alive... ALIVE!!!
		
		if ( count == 3 )
		{
			if( state == 0 ) nextState = 1;
		}
		
	}
	
	override public function render(bd:BitmapData):Void 
	{
		
		state = nextState;
		var color:Int = ( state == 1 ) ? 0 : 0xFFFFFF;
		bd.setPixel( x, y, color );
		
	}
}