package net.nicoptere.ca.gol;
import haxe.Timer;
import nme.display.BitmapData;
import nme.display.Bitmap;
import nme.display.Sprite;

/**
 * @author Nicolas Barradeau 
 */

class NeuronTest extends Sprite
{
	
	private var w:Int;
	private var h:Int;
	
	private var bd:BitmapData;
	private var bmp:Bitmap;
	
	private var cells:Array<Array<NeuronCell>>;
	
	public function new( width:Float, height:Float ) 
	{
		
		super();
		
		var scale:Float = 10;
		this.w = Std.int( width / scale );
		this.h = Std.int( height / scale );
		
		//init cell pool:
		cells = new Array<Array<NeuronCell>>();
		
		for ( x in 0...w )
		{
			
			cells.push( new Array<NeuronCell>() );
			
			for ( y in 0...h )
			{
				//3 possible states 
				var state:Int = Std.int( Math.random() * 3 );
				
				var cell:NeuronCell = new NeuronCell( x, y, state );
				cells[ x ].push( cell );
				
			}
		}
		
		//adds the neighbors to each cell
		for ( x in 0...w )
		{
			for ( y in 0...h )
			{
				
				var cell:NeuronCell = cells[ x ][ y ];
				
				var top:Int 	= ( y - 1 );  
				var right:Int 	= ( x + 1 );  
				var bottom:Int 	= ( y + 1 );  
				var left :Int	= ( x - 1 );  
				
				if( top < 0 	) 	top 		+= h;
				if( right >= w 	)	right 		-= w;
				if( left < 0	) 	left 		+= w;  
				if( bottom >= h ) 	bottom 		-= h;  
				
				//up, right, bottom, left
				cell.addNeighbor( 	cells[x][top]			);
				cell.addNeighbor( 	cells[right][y]			);
				cell.addNeighbor( 	cells[x][bottom]		);
				cell.addNeighbor( 	cells[left][y]			);
				
				//diagonals
				cell.addNeighbor( 	cells[right][top]		);
				cell.addNeighbor( 	cells[right][bottom]	);
				cell.addNeighbor( 	cells[left][bottom]		);
				cell.addNeighbor( 	cells[left][top]		);
				
			}
		}
		
		bd = new BitmapData( w, h, false, 0xFFFFFF );
		bmp = new Bitmap( bd );
		bmp.scaleX = bmp.scaleY = scale;
		addChild( bmp );
		
		var timer:Timer = new Timer( 100 );
		timer.run = update;
		
	}
	
	private function update() 
	{
		
		bd.fillRect( bd.rect, 0xFFFFFF );
		
		bd.lock();
		
		//evaluate simulation
		for ( array in cells )
		{
			for ( c in array )
			{
				c.evaluate();
			}
		}
		
		//render
		for ( array in cells )
		{
			for ( c in array )
			{
				c.render( bd );
			}
		}
		
		bd.unlock();
		
	}
}