package net.nicoptere.ca.gol;
import haxe.Timer;
import nme.display.BitmapData;
import nme.display.Bitmap;
import nme.display.PixelSnapping;
import nme.display.Sprite;

/**
 * @author Nicolas Barradeau 
 */

class WaveTest extends Sprite
{
	
	private var w:Int;
	private var h:Int;
	
	private var bd:BitmapData;
	private var bmp:Bitmap;
	
	private var cells:Array<Array<WaveCell>>;
	
	public function new( width:Float, height:Float ) 
	{
		
		super();
		
		var scale:Float = 4;
		this.w = Std.int( width / scale );
		this.h = Std.int( height / scale );
		
		//init cell pool:
		cells = new Array<Array<WaveCell>>();
		
		for ( x in 0...w )
		{
			
			cells.push( new Array<WaveCell>() );
			
			for ( y in 0...h )
			{
				
				var cell:WaveCell = new WaveCell( x, y, 0 );
				cells[ x ].push( cell );
				
			}
		}
		
		//adds the neighbors to each cell
		for ( x in 0...w )
		{
			for ( y in 0...h )
			{
				
				var cell:WaveCell = cells[ x ][ y ];
				
				var top:Int 	= ( y - 1 );  
				var right:Int 	= ( x + 1 );  
				var bottom:Int 	= ( y + 1 );  
				var left :Int	= ( x - 1 );  
				
				if( top < 0 	) 	top 		+= h;
				if( right >= w 	)	right 		-= w;
				if( left < 0	) 	left 		+= w;  
				if( bottom >= h ) 	bottom 		-= h;  
				
				//up, right, bottom, left
				cell.addNeighbor( 	cells[x][top]			);
				cell.addNeighbor( 	cells[right][y]			);
				cell.addNeighbor( 	cells[x][bottom]		);
				cell.addNeighbor( 	cells[left][y]			);
				
				//diagonals
				cell.addNeighbor( 	cells[right][top]		);
				cell.addNeighbor( 	cells[right][bottom]	);
				cell.addNeighbor( 	cells[left][bottom]		);
				cell.addNeighbor( 	cells[left][top]		);
				
			}
		}
		
		//points at random
		
		//center
		//cells[ Std.int( w * .5 ) ][ Std.int( h * .5 ) ].state = 0xFF;
		
		for ( i in 0...10 )
		{
			cells[ Std.int( Math.random() * w ) ][ Std.int( Math.random() * h ) ].state = 0xFF;
		}
		
		bd = new BitmapData( w, h, false, 0xFFFFFF );
		bmp = new Bitmap( bd );
		bmp.scaleX = bmp.scaleY = scale;
		addChild( bmp );
		
		var timer:Timer = new Timer( 10 );
		timer.run = update;
		
	}
	
	private function update() 
	{
		
		bd.fillRect( bd.rect, 0xFFFFFF );
		
		bd.lock();
		
		//evaluate simulation
		for ( array in cells )
		{
			for ( c in array )
			{
				c.evaluate();
			}
		}
		
		//render
		for ( array in cells )
		{
			for ( c in array )
			{
				c.render( bd );
			}
		}
		
		bd.unlock();
		
	}
}