package net.nicoptere.ca.pattern;
import biga.utils.GeomUtils;
import com.bit101.components.HUISlider;
import haxe.Timer;
import nme.display.Bitmap;
import nme.display.PixelSnapping;
import nme.events.Event;
import nme.geom.Point;
import nme.geom.Rectangle;
import nme.display.BitmapData;
import nme.display.Sprite;

/**
 * @author Nicolas Barradeau 
 */

class BelousovZhabotinsky extends Sprite
{
	/*
	//http://eprints.ucl.ac.uk/17241/1/17241.pdf
	// Idealised Belousov-Zhabotinsky reaction
	float [][][] a;
	float [][][] b;
	float [][][] c;
	int p = 0, q = 1;
	void setup ()
	{
		size (400 ,400);
		colorMode(HSB ,1.0);
		a = new float [width ][ height ][2];
		b = new float [width ][ height ][2];
		c = new float [width ][ height ][2];
		for (int x = 0; x < width; x++) 
		{
			for (int y = 0; y < height; y++)
			{
				a[x][y][p] = random (0.0 ,1.0);
				b[x][y][p] = random (0.0 ,1.0);
				c[x][y][p] = random (0.0 ,1.0);
			}
		}
	}
	void draw ()
	{
		for (int x = 0; x < width; x++)
		{
			for (int y = 0; y < height; y++)
			{
				float c_a = 0.0;
				float c_b = 0.0;
				float c_c = 0.0;
				for (int i = x - 1; i <= x + 1; i++)
				{
					for (int j = y - 1; j <= y + 1; j++)
					{
						c_a += a[(i+width )% width ][(j+height )% height ][p];
						c_b += b[(i+width )% width ][(j+height )% height ][p];
						c_c += c[(i+width )% width ][(j+height )% height ][p];
					}
				}
				c_a /= 9.0;
				c_b /= 9.0;
				c_c /= 9.0;
				
				a[x][y][q] = constrain(	c_a + c_a * (c_b - c_c), 0, 1);
				b[x][y][q] = constrain(	c_b + c_b * (c_c - c_a), 0, 1);
				c[x][y][q] = constrain(	c_c + c_c * (c_a - c_b), 0, 1);
				
				set(x,y,color (0.5 ,0.7 ,a[x][y][q]));
				
			}
		}
		if (p == 0)
		{
			p = 1; q = 0;
		}
		else
		{
			p = 0; q = 1;
		}
	}
	//*/

	private var w:Int;
	private var h:Int;
	private var bd:BitmapData;
	private var bmp:Bitmap;
	
	private var pattern:BitmapData;
	
	private var a:Array<Array<Array<Float>>>;
	private var b:Array<Array<Array<Float>>>;
	private var c:Array<Array<Array<Float>>>;
	
	private var p:Int = 0;
	private var q:Int = 1;
	
	public function new( w:Float, h:Float ) 
	{
		super();
		
		var scale:Float = 10;
		this.w = Std.int( w / scale );
		this.h = Std.int( h / scale );
		
		bd = new BitmapData( this.w, this.h, false, 0xFFFFFF );
		bmp = new Bitmap( bd );
		bmp.scaleX = bmp.scaleY = scale;
		addChild( bmp );
		
		pattern = new BitmapData( 100, 1, false, 0 );
		pattern.noise( 123 );
		
		a = new Array<Array<Array<Float>>>();
		b = new Array<Array<Array<Float>>>();
		c = new Array<Array<Array<Float>>>();
		
		for ( i in 0...bd.width )
		{
			a.push( new Array<Array<Float>>() );
			b.push( new Array<Array<Float>>() );
			c.push( new Array<Array<Float>>() );
			
			for ( j in 0...bd.height )
			{
				
				a[ i ].push( new Array<Float>() );
				b[ i ].push( new Array<Float>() );
				c[ i ].push( new Array<Float>() );
				
				//p
				a[ i ][ j ].push( Math.random() );
				b[ i ][ j ].push( Math.random() );
				c[ i ][ j ].push( Math.random() );
				
				//q
				a[ i ][ j ].push( 0 );
				b[ i ][ j ].push( 0 );
				c[ i ][ j ].push( 0 );
				
			}			
		}
		p = 0;
		q = 1;
		
		var timer:Timer = new Timer( 10 );
		timer.run = update;
		
	}
	///*
	private function update():Void
	{
		
		var c_a:Float, c_b:Float, c_c:Float;
		
		var ra:Float = .95;
		var rb:Float = 1.1;
		var rc:Float = .8;
		
		//ra = rb = rc = 1;
		
		bd.lock();
		for ( x in 0...w ) 
		{
			
			for ( y in 0...h ) 
			{
				
				c_a = 0;
				c_b = 0;
				c_c = 0;
				
				var ox:Int = ( x - 1);
				var dx:Int = ( x + 2);
				
				var oy:Int = ( y - 1);
				var dy:Int = ( y + 2);
				
				for ( i in ox...dx ) 
				{
					for ( j in oy...dy ) 
					{
						
						c_a += a[ ( i + w ) % w ][ ( j + h ) % h ][ p ];
						c_b += b[ ( i + w ) % w ][ ( j + h ) % h ][ p ];
						c_c += c[ ( i + w ) % w ][ ( j + h ) % h ][ p ];
						
					}
				}
				
				c_a /= 9.0;
				c_b /= 9.0;
				c_c /= 9.0;
				
				a[x][y][q] = constrain(	c_a + c_a * ( c_b - c_c ), 0, 1);
				b[x][y][q] = constrain(	c_b + c_b * ( c_c - c_a ), 0, 1);
				c[x][y][q] = constrain(	c_c + c_c * ( c_a - c_b ), 0, 1);
				
				//a[x][y][q] = constrain(c_a + c_a * ( ra * ( rc * c_b ) - c_c ), 0, 1);
				//b[x][y][q] = constrain(c_b + c_b * ( rb * ( ra * c_c ) - c_a ), 0, 1);
				//c[x][y][q] = constrain(c_c + c_c * ( rc * ( rb * c_a ) - c_b ), 0, 1);
				
				var px:Int = Std.int( a[x][y][q] * 0xFF ) << 16 | Std.int( b[x][y][q] * 0xFF ) << 8 | Std.int( c[x][y][q] * 0xFF );
				
				//var px:Int = Std.int( a[x][y][q] * 0xCC ) << 16 | Std.int( b[x][y][q] * 0xCC ) << 8 | 0;
				bd.setPixel( x, y, px );
				
			}
		}
		bd.unlock();
		
		if (p == 0) 
		{
			p = 1; q = 0;
		}
		else 
		{
			p = 0; q = 1;
		}
	}
	
	private function constrain (value : Float, minValue : Float, maxValue : Float) : Float
	{
		return Math.min(Math.max(value, minValue),maxValue); 
	}
	
}