package net.nicoptere.ca.pattern;
import haxe.Timer;
import nme.display.PixelSnapping;
import nme.display.Sprite;
import nme.events.MouseEvent;
import nme.display.Bitmap;
import nme.display.BitmapData;
import nme.geom.Rectangle;
import nme.events.Event;
import nme.Vector;
using nme.Vector;

/**
//http://softologyblog.wordpress.com/2011/07/05/multi-scale-turing-patterns/
//http://hellopixel.net/blog/2012/02/25/organic-turing-pattern/
 * @author simoe
 */
class TuringPattern extends Sprite
{
	
	private var WIDTH : Int;
	private var HEIGHT : Int;

	private var _grids : Vector<Float>;
	private var _diffLeft : Vector<Float>;
	private var _diffRight : Vector<Float>;
	private var _blurBuffer : Vector<Float>;
	private var _variations : Vector<Float>;
	private var _bestVariation : Vector<Float>;
	private var _bestLevel : Vector<Int>;
	private var _directions : Vector<Bool>;
	private var _stepSizes : Vector<Float>;
	private var _radii : Vector<Float>;

	private var n : Int;
	private var levels : Int;

	private var _bmpData : BitmapData;
	private var _bmp : Bitmap;
	private var _firstPixel : Particle3D;

	
	public function new( w:Float, h:Float ) 
	{
		super();
		
		var scale:Float = 1;
		WIDTH 	= HEIGHT = 100;// Std.int( Math.max( w, h ) );
		
		var base : Float 		= random(1.5, 2.4);
		var stepScale : Float 	= random(.005, .06);
		var stepOffset : Float  = random(.0001, .02);
		
		n = WIDTH * HEIGHT;
		levels = Std.int( Math.log( WIDTH ) / Math.log( base ) );
		
		_radii 			= new Vector<Float>();
		_stepSizes 		= new Vector<Float>();
		_grids 			= new Vector<Float>();
		_diffLeft 		= new Vector<Float>();
		_diffRight 		= new Vector<Float>();
		_blurBuffer 	= new Vector<Float>();
		_bestLevel 		= new Vector<Int>();
		_variations 	= new Vector<Float>();
		_bestVariation 	= new Vector<Float>();
		_directions 	= new Vector<Bool>();
		
		_bmpData = new BitmapData( WIDTH, HEIGHT, false, 0x000000);
		_bmp = new Bitmap(_bmpData, PixelSnapping.AUTO, false );
		_bmp.scaleX = _bmp.scaleY = 5;// Math.min( w / WIDTH, h / HEIGHT );
		
		addChild(_bmp);
		
		//determines the shape of the pattern
		for ( i in 0...levels )
		{
			var radius : Int = Std.int( Math.floor( Math.pow(base, i) ) );
			_radii.push( radius );
			_stepSizes.push( Math.log(radius) * stepScale + stepOffset );
		}
		
		//initialise the grid with random point between -1 and +1
		for ( i in 0...n )
		{
			_grids.push( random( -1, 1) );
			//init tableaux
			_diffLeft.push( 0 );
			_diffRight.push( 0 );
			_blurBuffer.push( 0 );
			_bestLevel.push( 0 );
			_variations.push( 0 );
			_bestVariation.push( 0 );
			_directions.push( false );
		}
		
		getPixels();
		var timer:Timer = new Timer( 1 );
		timer.run = update;
		
		//addEventListener(Event.ENTER_FRAME, update);
		//stage.addEventListener(MouseEvent.CLICK, click);
	}

    private function step() : Void
	{
		var activator : Vector<Float> = _grids;
		var inhibitor : Vector<Float> = _diffRight;
		
		//for ( l in 0...( levels - 1 ) )
		for ( l in 0...levels )
		{
			var radius : Int = Std.int(_radii[l]);
			fastBlur(activator, inhibitor, _blurBuffer, WIDTH, HEIGHT, radius);
			
			//calculates the absolute difference between activator and inhibitor

			for (  i in 0...n )
			{
				_variations[i] = activator[i] - inhibitor[i];
				if( _variations[i] < 0 )
				{
					_variations[i] = -_variations[i];
				}
			}
			
			//if first level then set some initial values for bestLevel and bestVariation 
			if (l == 0)
			{
				for ( i in 0...n )
				{
					_bestVariation[i] = _variations[i];
					_bestLevel[i] = l;
					_directions[i] = activator[i] > inhibitor[i];
				}
				activator = _diffRight;
				inhibitor = _diffLeft;
			} 
			else 
			{
				for ( i in 0...n )
				{
					if (_variations[i] < _bestVariation[i])
					{
						_bestVariation[i] = _variations[i];
						_bestLevel[i] = l;
						_directions[i] = activator[i] > inhibitor[i];
					}
				}
				var swap : Vector<Float> = activator;
				activator = inhibitor;
				inhibitor = swap;
			}
		}
		
		var smallest : Float = Math.POSITIVE_INFINITY;
		var largest : Float = Math.NEGATIVE_INFINITY;
		
		for ( i in 0...n )
		{
			var curStep : Float = _stepSizes[_bestLevel[i]];
			if (_directions[i])
			{
				_grids[i] += curStep;
			} 
			else 
			{
				_grids[i] -= curStep;
			}
			smallest = Math.min(smallest, _grids[i]);
			largest = Math.max(largest, _grids[i]);
		}
		
		var range : Float = (largest - smallest) / 2;
		
		for ( i in 0...n )
		{
			_grids[i] = ((_grids[i] - smallest) / range) - 1;
		}
	}

	private function click(event : MouseEvent) : Void
	{
		var i : Int;
		
		for ( i in 0...n )
		{
			_grids[i] = random(-1, 1);
			//_diffLeft[i] = undefined;
			//_diffRight[i] = undefined;
			//_blurBuffer[i] = undefined;
			//_bestLevel[i] = undefined;
			//_variations[i] = undefined;
			//_bestVariation[i] = undefined;
			//_directions[i] = undefined;
		}
	}

	private function update() : Void
	{
		var p : Particle3D = _firstPixel;
		var i:Int = 0;
		step();
		_bmpData.lock();
		do 
		{
			var color : Int = Std.int( 0x80 + ( 0x80 * _grids[i++] ) );
			_bmpData.setPixel( Std.int( p.x ), Std.int( p.y ), color << 16 | color << 8 | color);
			p = p.next;
		} while (p != null);
		_bmpData.unlock();
		
	}

	
	
	private function random(min : Float, max : Float = 0 ) : Float
	{
		if ( max == 0 )
		{
			max = min;
			min = 0;
		}
		return Math.random() * (max - min) + min;
	}

	private function getPixels() : Void
	{
		var x : Float;
		var y : Float;
		var prevPixel : Particle3D = null;
		var pixel : Particle3D = null;
		
		_bmpData.lock();
		//for (x = 0;x < WIDTH;x++) 
		for (x in 0...WIDTH ) 
		{
			//for (y = 0;y < HEIGHT;y++) 
			for( y in 0...HEIGHT )
			{
				var color : Int = _bmpData.getPixel(x, y);
				pixel = new Particle3D(color);
				pixel.x = x; 
				pixel.y = y;
				
				pixel.next = prevPixel;
				prevPixel = pixel;
			}
		}
		_firstPixel = pixel;
		_bmpData.unlock();
	}

	/*
	 * Blur the image using sequential two step bluring technique
	 * First blur the image vertically then horizontally.   
	 */

	private function fastBlur( source : Vector<Float>, dest : Vector<Float>, buffer : Vector<Float>, 
							  w : Int, h : Int, radius : Float) : Void
	{
		var y : Int;
		var x : Int;
		//for (y = 0; y < h; y++)
		for( y in 0...h )
		{
			//for (x = 0;x < w;x++)
			for( x in 0...w )
			{
				var i : Int = y * w + x;
				if(y == 0 && x == 0)
				{
					buffer[i] = source[i];
				} else if (y == 0)
				{
					buffer[i] = buffer[i - 1] + source[i];
				} else if (x == 0)
				{
					buffer[i] = buffer[i - w] + source[i];
				} 
				else
				{
					buffer[i] = buffer[i - 1] + buffer[i - w] - buffer[i - w - 1] + source[i];
				}
			}
		}
		
		//for (y = 0; y < h; y++)
		for( y in 0...h )
		{
			//for (x = 0;x < w;x++)
			for( x in 0...w )
			{
				var minx : Int = Std.int( Math.max(0, x - radius)		);
				var maxx : Int = Std.int( Math.min(x + radius, w - 1)	);
				var miny : Int = Std.int( Math.max(0, y - radius)		);
				var maxy : Int = Std.int( Math.min(y + radius, h - 1)	);
				var area : Int = (maxx - minx) * (maxy - miny);
				
				var nw : Int = miny * w + minx;
				var ne : Int = miny * w + maxx;
				var sw : Int = maxy * w + minx;
				var se : Int = maxy * w + maxx;
				
				var n : Int = y * w + x;
				dest[n] = (buffer[se] - buffer[sw] - buffer[ne] + buffer[nw]) / area; 
			}
		}
	}
}

class Point3D
{
    public var x : Float;
    public var y : Float;
    public var z : Float;

    public function new(x : Float = 0, y : Float = 0, z : Float = 0)
    {
        this.x = x;
        this.y = y;
        this.z = z;
    }
}

class Particle3D extends Point3D
{
    public var red : Int;
    public var green : Int;
    public var blue : Int;
    public var color : Int;

    public var projX : Float;
    public var projY : Float;

    public var u : Float;
    public var v : Float;
    public var w : Float;

    public var next : Particle3D;
    public var onScreen : Bool;

    public function new(color : Int = 0xFFFFFF)
    {
		super();
        this.color = color;
        this.red = getRed(color);
        this.green = getGreen(color);
        this.blue = getBlue(color);
    }

    public function getRed(c : Int) : Int
    {
        return c >> 16 & 0xff;
    }

    public function getGreen(c : Int) : Int
    {
        return c >> 8 & 0xff;
    }

    public function getBlue(c : Int) : Int
    {
        return c & 0xff;
    }
}