package net.nicoptere.geometry.interpolation;
import biga.distribution.Star;
import haxe.Timer;
import net.nicoptere.utils.Colors;
import nme.display.Sprite;
import nme.geom.Point;
import nme.Vector;

/**
 * @author Nicolas Barradeau 
 */

class InterpolationPath extends Sprite
{
	private var w:Float;
	private var h:Float;
	private var points:Vector<Point>;
	private var p:Point;
	private var T:Float;
	
	public function new(  w:Float, h:Float ) 
	{
		super();
		
		this.w = w;
		this.h = h;
		
		T = 0;
		
		points = new Star( new Point( w * .5, h * .5 ), 100, 200 ).distribute( 5 );
		
		p = new Point();
		
		var timer:Timer = new Timer( 10 );
		timer.run = update;
		
	}
	
	private function update() 
	{
		
		graphics.clear();
		
		graphics.lineStyle( 0, Colors.WHITE );
		graphics.moveTo( points[0].x, points[0].y );
		for ( p in points )
		{
			
			graphics.lineTo( p.x, p.y );
			graphics.drawCircle( p.x, p.y, 1 );
			graphics.moveTo( p.x, p.y );
		
		}
		
		//auto
		T = .5 + Math.sin( Timer.stamp() ) * .5;
		
		//T = normalize( mouseY, 0, h );
		
		getPointAt( T, points, p );
		
		
		graphics.lineStyle( 0, 0, 0 );
		graphics.beginFill( Colors.ORANGE, .5 );
		graphics.drawCircle( p.x, p.y, 10 );
		graphics.beginFill( Colors.ORANGE );
		graphics.drawCircle( p.x, p.y, 5 );
		
	}
	
	private function getPointAt( t:Float, points:Vector<Point>, p:Point = null , loop:Bool = false ):Point 
	{
		var length:Int = points.length;
		if ( !loop ) length--;
		
		var i:Int = Math.floor( length * t );
		var delta:Float = 1 / ( length );
		t = ( t - ( i * delta ) ) / delta;
		
		if ( p == null )
		{
			p =  new Point( 	lerp( t, points[ i ].x, points[ ( i + 1 ) % points.length ].x ),
								lerp( t, points[ i ].y, points[ ( i + 1 ) % points.length ].y ) );
		}
		else
		{
			p.x = lerp( t, points[ i ].x, points[ ( i + 1 ) % points.length ].x );
			p.y = lerp( t, points[ i ].y, points[ ( i + 1 ) % points.length ].y );
		}
		return p;
	}
	
	static public function lerp( t:Float, a:Float, b:Float ):Float
	{
		return a + ( b - a ) * t;
	}
	
	static public function normalize( value:Float, a:Float, b:Float ):Float
	{
		return ( value - a ) / ( b - a );
	}
	
	static public function map(value:Float, min1:Float, max1:Float, min2:Float, max2:Float):Float
	{
		return lerp( normalize( value, min1, max1 ), min2, max2 );
	}
	
}