package net.nicoptere.geometry.oscillation;
import haxe.Timer;
import net.nicoptere.utils.Colors;
import net.nicoptere.utils.Constants;
import net.nicoptere.geometry.oscillation.Harmonic;
import nme.display.Bitmap;
import nme.display.BitmapData;
import nme.display.Shape;
import nme.display.Sprite;
import nme.events.MouseEvent;
import nme.geom.ColorTransform;
import nme.geom.Point;
import nme.Lib;

/**
 * @author Nicolas Barradeau 
 */

class Oscillation_4 extends Sprite
{

	private var w:Float;
	private var h:Float;
	
	private var canvas:Shape;
	private var bd:BitmapData;
	private var bmp:Bitmap;
	
	private var radius:Float;
	
	private var position:Point;
	private var harmonicX:Harmonic;
	private var harmonicY:Harmonic;
	
	
	public function new( w:Float, h:Float ) 
	{
		super();
		
		this.w = w;
		this.h = h;
		
		
		radius = 50;
		
		harmonicX = new Harmonic( 4, Constants.DEGREES_TO_RADIANS );
		harmonicY = new Harmonic( 4, Constants.DEGREES_TO_RADIANS );
		
		position = new Point( w * .5, h *.5 );
		
		canvas = new Shape();
		bd = new BitmapData( Std.int( w ), Std.int( h ), true, 0 ); 
		bmp = new Bitmap( bd );
		addChild( bmp );
		
		var t:Timer = new Timer( 1 );
		t.run = update;
		
		Lib.stage.addEventListener( MouseEvent.MOUSE_DOWN, onMouseDown );
		
	}
	
	private function onMouseDown(e:MouseEvent):Void 
	{
		harmonicX = new Harmonic( 4, .01 );
		harmonicY = new Harmonic( 4, .01 );
	}
	
	private function update():Void 
	{
		canvas.graphics.clear();
		
		canvas.graphics.lineStyle( 0, Colors.BLUE, .5 );
		canvas.graphics.moveTo( position.x, position.y );
		
			harmonicX.update();
			harmonicY.update();
			
			position.x = w * .5 + Math.cos( harmonicX.angle ) * w * .5;
			position.y = h * .5 + Math.sin( harmonicY.angle ) * h * .5;
		
		canvas.graphics.lineTo( position.x, position.y );
		
		bd.draw( canvas );
		bd.colorTransform( bd.rect, new ColorTransform( 1, 1, 1, ( 1 - ( 1 / 0xFF ) ) ) );
	
		graphics.clear();
		graphics.lineStyle( 0, 0xAAAAAA );
		graphics.drawCircle( w * .5, h * .5, radius );
		graphics.lineStyle( 0, 0, 0 );
		
		graphics.beginFill( Colors.WHITE );
		graphics.drawCircle( position.x, position.y, 3 );
		
		graphics.beginFill( Colors.RED );
		graphics.drawCircle( w * .5 + Math.cos( harmonicX.angle ) * radius,
							 h * .5 + Math.sin( harmonicX.angle ) * radius, 5 );
							 
		graphics.beginFill( Colors.GREEN );
		graphics.drawCircle( w * .5 + Math.cos( harmonicY.angle ) * radius,
							 h * .5 + Math.sin( harmonicY.angle ) * radius, 5 );
	}
	
}