package net.nicoptere.particles.app;
import nme.display.BitmapData;
import net.nicoptere.particles.bodies.Velocity;
import net.nicoptere.utils.Constants;
import nme.display.Bitmap;
import nme.display.Shape;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.geom.ColorTransform;
import nme.Lib;
import nme.Vector;

/**
 * @author Nicolas Barradeau 
 */

class BubbleChamber extends BaseScene
{
	private var velocities:Vector<AngularVElocity>;
	private var bd:BitmapData;
	private var bgColor:Int;
	private var canvas:Shape;

	public function new( w:Float, h:Float ) 
	{
		super( w, h );
		
		velocities = new Vector<AngularVElocity>();
		
		init();
		
		bgColor = 0xFF101010;
		
		canvas = new Shape();
		
		bd = new BitmapData( Std.int( w ), Std.int( h ), true, 0 ); 
		var bmp:Bitmap = new Bitmap( bd );
		addChild( bmp );
		
		Lib.stage.addEventListener( MouseEvent.MOUSE_DOWN, reset );
		
		addEventListener( Event.ENTER_FRAME, oef );
		
	}
	
	private function init() 
	{
		
		for ( i in 0...100 )
		{
			
			var vel:AngularVElocity = new AngularVElocity( 	w * .5, 
															h * .5,
															1 + Math.random() * 10
															);
			
			velocities.push( vel );
			
		}
		
		
	}
	
	private function reset(e:MouseEvent):Void 
	{
		
		//bd.fillRect( bd.rect, bgColor );
		
		for ( v in velocities )
		{
			v.x = w * .5;
			v.y = h * .5;
		}
		
	}
	
	private function oef(e:Event):Void 
	{
		graphics.clear();
		graphics.lineStyle( 0, 0xFFFFFF );
		//canvas.graphics.lineStyle( 0, 0xFFFFFF, 1 );
	
		for ( v in velocities )
		{
			
			v.update( Constants.TIME_STEP );
			
			graphics.moveTo( v.x - v.vx, v.y - v.vy );
			graphics.lineTo( v.x, v.y  );
			
			//canvas.graphics.moveTo( v.x - v.vx, v.y - v.vy );
			//canvas.graphics.lineTo( v.x, v.y  );
			
			checkBounds();
			
		}
		
		//bd.draw( canvas );
		//bd.colorTransform( bd.rect, new ColorTransform( 1,1,1, .99 ) );
		//canvas.graphics.clear();
			
		//for ( i in 0...1000 ){}removeEventListener( Event.ENTER_FRAME, oef );
		
	}
	
	private function checkBounds() 
	{
		var margin:Float = 20;
		for( v in velocities )
		{
			if( v.x < margin ) v.x = w - margin;
			if( v.y < margin ) v.y = h - margin;
			if( v.x > w - margin ) v.x = margin;
			if( v.y > h - margin ) v.y = margin;
		}
	}
}

import net.nicoptere.particles.bodies.Velocity;
class AngularVElocity extends Velocity
{
	private var angle:Float;
	private var angularSpeed:Float;
	private var iterations:Int;
	private var maxIterations:Int;
	
	public function new( x:Float, y:Float, radius:Float )
	{
		
		super( x, y, radius );
		
		angle = Math.random() * Math.PI * 2;
		
		//vitess angulaire
		this.angularSpeed = ( Math.random() - .5 ) * 20 * Constants.DEGREES_TO_RADIANS;
		
		//nombre d'iterations avant de changer de direction
		iterations = 0;
		maxIterations = 10 + Std.int( Math.random() * 30 );
		
	}
	
	override public function update(dt:Float = 1):Void 
	{
		
		/*
		iterations++;
		if ( iterations > maxIterations )
		{
			iterations = 0;
			this.angularSpeed = ( Math.random() - .5 ) * 20 * Constants.DEGREES_TO_RADIANS;
			maxIterations = 10 + Std.int( Math.random() * 30 );
		}
		//*/
		
		angle += angularSpeed;
		
		vx = Math.cos( angle ) * radius;
		vy = Math.sin( angle ) * radius;
		
		super.update(dt);
		
	}
	
}