package net.nicoptere.particles.app;
import haxe.Timer;
import net.nicoptere.particles.app.creature.Creature;
import net.nicoptere.particles.bodies.Body;
import net.nicoptere.particles.forces.Elastic;
import nme.geom.Rectangle;

/**
 * @author Nicolas Barradeau 
 */

class CreatureTest extends BaseScene
{
	private var heart:Body;
	private var head:Body;
	private var limb:Body;
	private var tail:Body;

	private var el2:Elastic;
	private var el1:Elastic;
	private var el0:Elastic;
	private var el3:Elastic;
	
	private var c:Creature;
	
	public function new() 
	{
		super();
		
		var cx:Float = w * .1;
		var cy:Float = h * .5;
		
		head 	= new Body( cx + 10, cy 	 );
		heart	= new Body( cx - 10, cy + 10 );
		limb 	= new Body( cx - 10, cy - 10 );
		tail 	= new Body( cx - 20, cy 	 );
		
		tail.friction = .5;
		
		bodies.push( head );
		bodies.push( heart );
		bodies.push( limb );
		bodies.push( tail );
		
		var strength:Float = 1;
		
		el0 = new Elastic( head, heart );
		forces.push( el0 );
		
		el1 = new Elastic( head, limb );
		forces.push( el1 );
		
		el2 = new Elastic( heart, limb );
		forces.push( el2 );
		
		el3 = new Elastic( tail, limb );
		forces.push( el3 );
		
		el3 = new Elastic( tail, heart );
		forces.push( el3 );
		
		var timer:Timer = new Timer( 30 );
		timer.run = update;
		
		c = new Creature( cx, cy );
		
	}
	
	private function update() 
	{
		
		checkBounds();
		
		updateBodies();
		
		var t:Float = .5 + Math.sin( Timer.stamp() * 10 ) * .5;
		
		c.update( t );
		
		var min:Float = 1;
		var max:Float = 20;
		
		head.mass 	= min + ( 1 - t ) * max;
		heart.mass 	= min + t * max;
		limb.mass 	= min + t * max;
		
		head.friction 	= t;
		heart.friction 	= ( 1 - t );
		limb.friction  	= ( 1 - t );
		
		el0.restLength 	= 1 + t * 50;
		el1.restLength 	= 1 + t * 50;
		
		render();
		c.render( graphics );
		
		/*
		graphics.clear();
		c.render( graphics );
		
		graphics.beginFill( 0xFFCC00, .5 );
		graphics.drawCircle( head.x, head.y, head.friction * 10 );
		
		graphics.beginFill( 0xFF0000, .5 );
		graphics.drawCircle( heart.x, heart.y, heart.friction * 10 );
		
		graphics.beginFill( 0x00FF00, .5 );
		graphics.drawCircle( limb.x, limb.y, limb.friction * 10 );
		
		graphics.beginFill( 0x0000FF, .5 );
		graphics.drawCircle( tail.x, tail.y, tail.friction * 10 );
		
		for( force in forces )
		{
			force.render( graphics, 0xFFCC00 );
		}
		//*/
	}
	
	private function checkBounds() 
	{
		return;
		//for( body in bodies )
		//{
			//if( body.x < 0 ) body.x = 0;
			//if( body.y < 0 ) body.y = 0;
			//if( body.x > w ) body.x = w - 1;
			//if( body.y > h ) body.y = h - 1;
		//}
		//return;
		var hit:Rectangle = new Rectangle();
		
		for( body in bodies )
		{
			if ( body.x < 0 )
			{
				hit.x = 1;
				break;
			}
			if ( body.y < 0 )
			{
				hit.y = 1;
				break;
			}
			if ( body.x > w )
			{
				hit.width = 1;
				break;
			}
			if ( body.y > h )
			{
				hit.height = 1;
				break;
			}
		}
		
		if ( hit.x == 1 )	for ( body in bodies ) body.x += w;
		if ( hit.y == 1 )	for ( body in bodies ) body.y += h;
		if ( hit.width == 1 )	for ( body in bodies ) body.x -= w;
		if ( hit.height == 1 )	for ( body in bodies ) body.y -= h;
		
		
	}
}