package net.nicoptere.particles.bodies;
import net.nicoptere.particles.forces.Force;
import net.nicoptere.utils.Constants;


/**
 * @author Nicolas Barradeau 
 */

class Body extends Friction
{
	
	private var _mass:Float;
	private var invertMass:Float;
	private var forces:Array<Force>;
	
	public function new( x:Float, y:Float, mass:Float = 1 ) 
	{
		
		super( x, y, mass );
		
		this.mass = mass;
		
		forces = new Array<Force>();
		
	}
	
	public function addForce( f:Force ):Void
	{
		
		for ( _f in forces ) 
		{
			//cette force est deja presente
			if ( f == _f ) 
			{ 
				return;
			}
		}
		//sinon, on l'ajoute
		forces.push( f );
		f.addBody( this );
		
	}
	
	public function removeForce( f:Force ):Void
	{
		for ( i in 0...Std.int( forces.length ) )
		{
			//cette force etait presente, elle degage
			if ( f == forces[ i ] )
			{
				forces.splice( i, 1 );
				return;
			}
		}
	}
	
	override public function update( dt:Float = 1 ):Void
	{
		
		//reset acceleration
		ax = 0;
		ay = 0;
		
		//ajoute toutes le forces
		for ( f in forces )
		{
			
			f.evaluate( this );
			
			ax += f.x * invertMass;
			ay += f.y * invertMass;
			
		}
		
		super.update( dt );
		
	}
	
	//SETTER la masse doit recalculer la masse inverse
	
	private function get_mass():Float 
	{
		return _mass;
	}
	
	private function set_mass(value:Float):Float 
	{
		if ( value <= 0 ) 
		{
			//EPSILON sert à eviter une division par 0
			//cet objet ne bougera pas trop trop...
			invertMass = Constants.EPSILON;
		}
		else 
		{
			invertMass = 1 / value;
		}
		radius = value;
		return _mass = value;
	}
	
	public var mass(get_mass, set_mass):Float;
	
}