package net.nicoptere.particles.forces;
import net.nicoptere.particles.bodies.Body;
import nme.display.Graphics;
import nme.geom.Point;

/**
 * ...
 * @author Nicolas Barradeau 
 */

class Attractor extends Force
{
	private var _position:Point;
	private var _radius:Float;
	private var _strength:Float;
	
	private var squareRadius:Float;
	private var invertSquareRadius:Float;

	public function new( x:Float = 0, y:Float = 0, radius:Float = 1, strength:Float = 1 ) 
	{
		
		super();
		bodies = new Array<Body>();
		
		position = new Point( x, y );
		this.radius = radius;
		this.strength = strength;
		
	}
	
	
	override public function evaluate( b:Body = null ):Void 
	{
		
		//calcule la distance entre 2 points
		dx = position.x - b.x;
		dy = position.y - b.y;
		dist = ( dx * dx + dy * dy );
		
		//si le corps est hors de la zone d'influence
		if ( dist > squareRadius )
		{
			//aucune force ne s'applique
			x = 0;
			y = 0;
			return;
		}
		
		//la force diminue quand on s'éloigne du centre
		force = ( 1 - dist * invertSquareRadius ) * strength;
		
		angle = Math.atan2( dy, dx );
		x = Math.cos( angle ) * force;
		y = Math.sin( angle ) * force;
		
		
	}
	
	override public function render(graphics:Graphics, color:Int = 0xFF0000, alpha:Float = 1 ):Void 
	{
		
		graphics.beginFill( color );
		graphics.drawCircle( position.x, position.y, 5 );
		
		graphics.beginFill( color, alpha );
		graphics.drawCircle( position.x, position.y, radius );
		graphics.endFill();
		
	}
	
	
	private function get_position():Point 
	{
		return _position;
	}
	
	private function set_position(value:Point):Point 
	{
		return _position = value;
	}
	
	public var position(get_position, set_position):Point;
	
	private function get_radius():Float 
	{
		return _radius;
	}
	
	private function set_radius(value:Float):Float 
	{
		if ( value == 0 )
		{
			squareRadius = value * value;
		}
		else
		{
			squareRadius = value * value;
		}
		invertSquareRadius = 1 / squareRadius;
		
		return _radius = value;
	}
	
	public var radius(get_radius, set_radius):Float;
	
	private function get_strength():Float 
	{
		return _strength;
	}
	
	private function set_strength(value:Float):Float 
	{
		return _strength = value;
	}
	
	public var strength(get_strength, set_strength):Float;
	
	
}