package net.nicoptere.particles.forces;
import biga.utils.GeomUtils;
import net.nicoptere.particles.bodies.Body;
import nme.display.Graphics;

/**
 * @author Nicolas Barradeau 
 */

class Elastic extends Force
{
	private var b0:Body;
	private var b1:Body;
	public var restLength:Float;
	public var strength:Float;

	public function new( b0:Body, b1:Body, restLength:Float = -1, strength:Float = 1 ) 
	{
		
		super();
		
		this.b0 = b0;
		this.b1 = b1;
		
		addBody( b0 );
		addBody( b1 );
		
		b0.addForce( this );
		b1.addForce( this );
		
		if ( restLength == -1 )
		{
			restLength = GeomUtils.distance( b0, b1 );
		}
		this.restLength = restLength;
		
		this.strength = strength;
		
	}
	
	override public function evaluate( b:Body = null ):Void 
	{
		
		var o:Body = b == b0 ? b1 : b0;
		
		dx = b.x - o.x;
		dy = b.y - o.y;
		dist = Math.sqrt( dx * dx + dy * dy );
		
		//force de repulsion
		force = -strength * ( dist - restLength );
		
		//applique la repulsion dans l'axe anchor->body
		angle = Math.atan2( dy, dx );
		
		x = Math.cos( angle ) * force;
		y = Math.sin( angle ) * force;
		
	}
	
	override public function render(graphics:Graphics, color:Int = 0xFF0000, alpha:Float = 1):Void 
	{
		graphics.lineStyle( 2, color, alpha );
		graphics.moveTo( b0.x, b0.y );
		graphics.lineTo( b1.x, b1.y );
		graphics.lineStyle( 0,0,0 );
	}
	
}