package net.nicoptere.particles.forces;
import biga.color.Color;
import biga.color.HSV;
import biga.utils.GeomUtils;
import net.nicoptere.particles.bodies.Body;
import nme.display.Graphics;
import nme.geom.Point;

/**
 * @author Nicolas Barradeau 
 */

class Spring extends Force
{
	static private var hsv:HSV = new HSV( 0, 100, 100 );
	
	public var anchor:Point;
	public var restLength:Float;
	private var strength:Float;
	
	public function new( anchor:Point, restLength:Float = 1, strength:Float = 1 )// compression:Float = 1, extension:Float = .9 ) 
	{
		super();
		
		if ( anchor == null ) anchor = new Point();
		this.anchor = anchor;
		
		//position au repos
	    this.restLength = restLength;
		
		//force avec laquelle le ressort revient à sa longueur de repos
		this.strength = strength;
		
	}
	
	override public function evaluate( b:Body = null ):Void 
	{
		
		dx = b.x - anchor.x;
		dy = b.y - anchor.y;
		dist = Math.sqrt( dx * dx + dy * dy );
		
		//force de repulsion
		force = -strength * ( dist - restLength );
		
		//applique la repulsion dans l'axe anchor->body
		angle = Math.atan2( dy, dx );
		
		x = Math.cos( angle ) * force;
		y = Math.sin( angle ) * force;
		
	}
	
	override public function render(graphics:Graphics, color:Int = 0xFF0000, alpha:Float = 1):Void 
	{
		
		graphics.lineStyle( 0, color, alpha );
		graphics.drawCircle( anchor.x, 	anchor.y, 2 );
		graphics.drawCircle( anchor.x, 	anchor.y, restLength );
		
		for ( b in bodies )
		{
			
			dx = ( b.x - anchor.x );
			dy = ( b.y - anchor.y );
			dist = Math.sqrt( dx * dx + dy * dy );
			
			hsv.hue = GeomUtils.map( dist / restLength, 0, 1, 0, 180 );
			graphics.lineStyle( 0, Color.hsv_rgb( hsv ), alpha );
			graphics.moveTo( anchor.x, anchor.y );
			graphics.lineTo( b.x, b.y );
			
			graphics.lineStyle( 0, color, alpha );
			graphics.moveTo( anchor.x, anchor.y );
			angle = Math.atan2( dy, dx );
			graphics.lineTo( 	anchor.x + Math.cos( angle ) * restLength,
								anchor.y + Math.sin( angle ) * restLength );
			
		}
		graphics.lineStyle( 0,0,0 );
	}
}
