package net.nicoptere.particles.test;
import net.nicoptere.particles.bodies.Body;
import net.nicoptere.particles.forces.Attractor;
import net.nicoptere.particles.forces.Gravity;
import net.nicoptere.particles.forces.Field;
import nme.display.Sprite;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.Lib;

/**
 * @author Nicolas Barradeau 
 */

class TestAttractor extends Sprite
{
	
	private var w:Float;
	private var h:Float;
	
	private var wind:Field;
	private var attractor:Attractor;
	
	private var bodies:Array<Body>;
	private var body:Body;
	private var mouseDown:Bool;
	private var angle:Float;

	public function new( width:Float = 800, height:Float = 480 ) 
	{
		
		super();
		
		w = width;
		h = height;
		
		wind = new Field();
		wind.strength = 2;
		attractor = new Attractor( w * .5, h * .5, w * .5, 1 );
		
		angle = 0;
		
		bodies = new Array<Body>();
		
		Lib.stage.addEventListener( MouseEvent.MOUSE_DOWN, onMouseDown );
		Lib.stage.addEventListener( MouseEvent.MOUSE_UP, onMouseDown );
		addEventListener( Event.ENTER_FRAME, update );
		
	}
	
	private function onMouseDown(e:MouseEvent):Void 
	{
		
		mouseDown = e.type == MouseEvent.MOUSE_DOWN ? true : false;
		
	}
	
	private function update(e:Event):Void 
	{
		graphics.clear();
		
		
		
		attractor.render( graphics, 0x0000FF, .25 );
		
		
		//update la force
		
		//randomise la direction du vent
		angle += ( Math.random() - .5 ) * Math.PI / 10;
		
		wind.direction.x += ( Math.cos( angle ) - wind.x ) * .01;
		wind.direction.y += ( Math.sin( angle ) - wind.y ) * .01;
		
		wind.render( graphics, 0x00FF00 );
		
		
		if ( mouseDown )
		{
			
			body = new Body( Math.random() * w, Math.random() * h, 1 + Math.random() * 20 );
			
			body.friction = .98;
			body.addForce( wind );//ajoute le vent
			body.addForce( attractor );//ajoute l'attracteur
			
			bodies.push( body );
			
		}
		
		for( body in bodies )
		{
			
			//met a jour
			body.update();
			
			//rendu
			body.render( graphics );
			
			//dessine le vecteur de la velocite en rouge
			body.drawVelocityVector( graphics );
			
			//dessine le vecteur de l'acceleration en jaune
			body.drawAcceleratioVector( graphics, 50 );
			
			graphics.lineStyle( 0, 0, 0 );
			
		}
		
		for ( body in bodies )
		{
			
			if ( body.x < 0 ) body.x += w;
			if ( body.y < 0 ) body.y += h; 
			if ( body.x > w ) body.x = 0; 
			if ( body.y > h	) body.y = 0; 
			
		}
		
	}
	
}