package net.nicoptere.physics;
import nape.common.FixedStep;
import nape.constraint.DistanceJoint;
import nape.constraint.MotorJoint;
import nape.constraint.PivotJoint;
import nape.geom.AABB;
import nape.geom.MarchingSquares;
import nape.geom.Vec2;
import nape.phys.Body;
import nape.phys.BodyType;
import nape.shape.Circle;
import nape.shape.Polygon;
import nape.space.Space;
import nape.util.BitmapDebug;
import nme.display.DisplayObject;
import nme.display.Graphics;
import nme.display.Sprite;
import nme.events.MouseEvent;

/**
 * @author Nicolas Barradeau 
 */

class PhysicsScene extends FixedStep
{
	private var w:Float;
	private var h:Float;
	private var space:Space;
	private var b0:Body;
	private var b1:Body;
	private var b2:Body;

	public function new( w:Float = 800, h:Float = 480 ) 
	{
		super( 1 / 60 );
		this.w = w;
		this.h = h;
		
		space = new Space( new Vec2( 0, 1000 ) );

		//borders
		var border = new Body( BodyType.STATIC );
		border.shapes.add(new Polygon(Polygon.rect(0,0,-50,h)));
		border.shapes.add(new Polygon(Polygon.rect(0,0,w,-50)));
		border.shapes.add(new Polygon(Polygon.rect(w,0,50,h)));
		border.shapes.add(new Polygon(Polygon.rect(0,h,w,50)));
		border.space = space;

		//create some nape bodies generated from display object graphics.
		for (i in 0...5)
		{
			
			var graphic:Sprite = new Sprite();
			
			var g:Graphics = graphic.graphics;
			g.beginFill( 0x808080,1);
			g.drawCircle(0,0,10);
			g.endFill();
			
			var body = circle( i * 100, 100, 10 );
			//var body = new Body();
			//body.position.setxy( i*100 , 100 );
			//body.rotation = Math.random() * Math.PI/180;
			//body.shapes.add( new Circle( 10) );
			//body.space = space;
			body.graphic = graphic;
			addChild( body.graphic );
			

		}
		
		var graphic:Sprite = new Sprite();
		var g:Graphics = graphic.graphics;
		g.beginFill( 0x008080,1);
		g.drawCircle(0,0,20);
		g.endFill();
		
		b0 = circle( w * .5 - 100, h * .5 + 100, 15 );
		b1 = circle( w * .5 - 100, h * .5 + 100, 25 );
		b2 = circle( w * .5 - 100, h * .5 - 50, 15 );
		
		var joint:DistanceJoint = new DistanceJoint( b0, b1,	new Vec2(10,0),	new Vec2( 0,0), 10, 80 );
		joint.space = space;
		
		var joint:DistanceJoint = new DistanceJoint( b0, b2,	new Vec2(0,0),	new Vec2( 0,0), 50, 50 );
		joint.space = space;
		
		var joint:DistanceJoint = new DistanceJoint( b1, b2,	new Vec2(0,0),	new Vec2( 0,0), 50, 50 );
		joint.space = space;
		
		//joint.space = null;
		//overlap debug ontop of everything
		var debug = new BitmapDebug( Std.int(w),Std.int(h),0x333333,true);
		debug.display.alpha = 1;
		debug.drawConstraints = true;
		addChild(debug.display);
		
		//=================================================================================
		
        var fluidBody = new Body(BodyType.STATIC);
        var fluidShape = new Polygon(Polygon.rect(0, h/2, w, h/2));
 
        fluidShape.filter.collisionMask = 0;
        fluidShape.fluidEnabled = true;
        fluidShape.filter.fluidMask = 2;
 
        fluidShape.fluidProperties.density = 3;
        fluidShape.fluidProperties.viscosity = 6;
 
        fluidShape.body = fluidBody;
        fluidBody.space = space;
		
		
        for (i in 0...20) 
		{
			
            var pentagonBody = new Body();
			
            pentagonBody.position.setxy( Math.random() * w, 200 );
			
            var pentagonShape = new Polygon(Polygon.regular(50, 50, 5));
 
            pentagonShape.filter.collisionGroup = ~2;
            //pentagonShape.filter.fluidGroup = 2;
 
            pentagonShape.body = pentagonBody;
            pentagonBody.space = space;
			fix( pentagonBody );
			
        }
		
		
		//=================================================================================
		//grabby grabby

		var hand = new PivotJoint(space.world,null,new Vec2(), new Vec2());
		hand.active = false;
		hand.space = space;
		hand.stiff = false;
		
		stage.addEventListener( MouseEvent.MOUSE_DOWN, function (_) {
			var mp = new Vec2(mouseX,mouseY);
			for(b in space.bodiesUnderPoint(mp)) {
				if(!b.isDynamic()) continue;
				hand.body2 = b;
				hand.anchor2 = b.worldToLocal(mp);
				hand.active = true;
				break;
			}
		});
		stage.addEventListener( MouseEvent.MOUSE_UP, function (_) hand.active = false);

		//=================================================================================
		
		run( function(dt) {
			debug.clear();
			hand.anchor1.setxy(mouseX,mouseY);
			space.step(dt);
			debug.draw(space);
			debug.flush();
			b1.angularVel = 10;

		} );
	}	
	
	private function fix( b:Body ):Void
	{
		var mp = new PivotJoint(space.world, b, b.position, new Vec2());
		mp.space = space;
	}

	private function circle(x, y, r):Body
	{
		var b = new Body();
		b.shapes.add(new Circle(r));
		b.position.setxy(x,y);
		b.space = space;
		return b;
	}
}