package net.nicoptere.planets;
import nme.display.Bitmap;
import nme.display.BitmapData;
import nme.display.Graphics;
import nme.display.PixelSnapping;
import nme.display.Sprite;
import nme.filters.BlurFilter;
import nme.geom.Point;

/**
 * @author Nicolas Barradeau
 */

class AsteroidBelt extends Sprite
{
	private var asteroids:Array<Point>;
	private var angles:Array<Float>;
	private var speeds:Array<Float>;
	private var radii:Array<Float>;
	
	private var inRadius:Float;
	private var outRadius:Float;
	private var bmp:Bitmap;
	private var bd:BitmapData;
	
	public function new( count:Int, inRadius:Float, outRadius:Float ) 
	{
		super();
		this.inRadius = inRadius;
		this.outRadius = outRadius;
		
		asteroids = new Array<Point>();
		
		angles = new Array<Float>();
		speeds = new Array<Float>();
		radii = new Array<Float>();
		
		for ( i in 0...count )
		{
			var radius:Float = inRadius + ( Math.random() * ( outRadius - inRadius ) );
			var angle:Float = Math.random() * Math.PI * 2;
			
			var p:Point = new Point( Math.cos( angle ) * radius, Math.sin( angle ) * radius );
			asteroids.push( p );
			
			angles.push( angle );
			speeds.push( ( .5 * Math.random() ) / 180 * Math.PI ); 
			radii.push( radius ); 
			
		}
		
		bd = new BitmapData( Std.int( outRadius * 2 ), Std.int( outRadius * 2 ), true, 0 );
		bmp = new Bitmap( bd, PixelSnapping.AUTO, true );
		bmp.x = bmp.y = - bd.width * .5;
		addChild( bmp );
		
		
	}
	
	public function update( t:Float ):Void
	{
		for ( i in 0...asteroids.length )
		{
			
			angles[ i ] -= speeds[ i ];
			
			asteroids[ i ].x = outRadius + Math.cos( angles[ i ] ) * radii[ i ];
			asteroids[ i ].y = outRadius + Math.sin( angles[ i ] ) * radii[ i ];
			
		}
	}
	
	public function render( ):Void
	{
		//graphics.clear();
		//graphics.beginFill( 0xCCCCCC );
		bd.fillRect( bd.rect, 0 );
		bd.lock();
		for ( p in asteroids )
		{
			//graphics.drawCircle( p.x, p.y, 1 );
			bd.setPixel32( Std.int( p.x ), Std.int( p.y ), Std.int( 0xFF * ( Math.random() ) )<<24 | 0x5983BD  );
		}
		bd.unlock();
		
		//graphics.endFill();
	}
	
}