package net.nicoptere.planets;
import biga.shapes2d.Arc;
import biga.utils.GeomUtils;
import nme.display.Shape;
import net.nicoptere.planets.Planet;
import nme.display.Sprite;
import nme.display.Graphics;
import nme.geom.Point;
import nme.Vector;

/**
 * @author Nicolas Barradeau
 */

class Calendar extends Sprite
{
	private var arrow:Sprite;
	
	private var inRadius:Float;
	private var midRadius:Float;
	private var outRadius:Float;
	private var arrowWidth:Float;
	private var arrowHeight:Float;
	
	private var arc:Arc;
	
	private var _referencePlanet:Planet;
	private var angle:Float = 0;
	private var period:Float = 360.26;
	
	private var segments:Array<Sprite>;
	
	public function new( size:Float ) 
	{
		super();
		
		outRadius = ( size -5 ) * .5;
		midRadius = outRadius - 10;
		inRadius = outRadius - 20;
		
		var overlay:Shape = new Shape();
		
		overlay.graphics.lineStyle( 0, 0x666666 );
		overlay.graphics.drawCircle( 0, 0, inRadius );
		overlay.graphics.drawCircle( 0, 0, midRadius );
		overlay.graphics.drawCircle( 0, 0, outRadius );
		
		segments = new Array<Sprite>();
		arc = new Arc( 0, 0, outRadius );
		
		var angle:Float = 0;
		var angleStep:Float = Math.PI / 6;
		while ( angle < Math.PI * 2 )
		{
			
			var s:Sprite = new Sprite();
			
			s.graphics.beginFill( 0x5983BD );
			
			var o:Point, p:Point;
			var pts:Vector<Point>;
			
			arc.startAngle = angle;
			arc.arcLength = angleStep;
			
			arc.radius = outRadius;
			o = arc.getPointAt( 0 );
			pts = arc.subdivide( 20 );
			s.graphics.moveTo( pts[ 0 ].x, pts[ 0 ].y );
			for ( p in pts )
			{
				s.graphics.lineTo( p.x, p.y );
			}
			
			arc.radius = midRadius + 4;
			pts = arc.subdivide( 20 );
			pts.reverse();
			for ( p in pts )
			{
				s.graphics.lineTo( p.x, p.y );
			}
			s.graphics.lineTo( o.x, o.y );
			s.graphics.endFill();
			addChild( s );
			s.alpha = 0;
			segments.push( s );
			
			overlay.graphics.moveTo( Math.cos( angle ) * inRadius, Math.sin( angle ) * inRadius );
			overlay.graphics.lineTo( Math.cos( angle ) * outRadius, Math.sin( angle ) * outRadius );
			angle += angleStep;
			
			
		}
		
		overlay.graphics.lineStyle( 0, 0x444444 );
		angle = 0;
		angleStep = Math.PI / 180;
		while ( angle < Math.PI * 2 )
		{
			overlay.graphics.moveTo( Math.cos( angle ) * midRadius, Math.sin( angle ) * midRadius );
			overlay.graphics.lineTo( Math.cos( angle ) * inRadius, Math.sin( angle ) * inRadius );
			angle += angleStep;
		}
		
		arrowHeight = 10;
		arrowWidth = 6;
		
		addChild( overlay );
		
		arrow = new Sprite();
		addChild( arrow );
		
	}
	
	public function update( t:Float ):Void
	{
		
		//angle -= Math.PI / 180;
		angle += ( ( Math.PI * 2 ) / period );
			
		arrow.x = Math.cos( angle ) * ( inRadius - arrowHeight - 2 );
		arrow.y = Math.sin( angle ) * ( inRadius - arrowHeight - 2 );
		
		arrow.rotation = ( angle + Math.PI * .5 ) * 180 / Math.PI;
		
		for ( s in segments )
		{
			if ( s.alpha > 0 ) s.alpha -= .05;
		}
		
		var f:Int = Std.int( GeomUtils.map( angle % ( Math.PI * 2 ), 0, Math.PI * 2, 0, segments.length ) );
		
		segments[ f ].alpha += 0.1;
		
	}
	
	public function render( g:Graphics ):Void
	{
		
	}
	
	private function get_referencePlanet():Planet 
	{
		return _referencePlanet;
	}
	
	private function set_referencePlanet(value:Planet):Planet 
	{
		
		this.angle = value.angle + Math.PI;
		this.period = value.period;
		
		drawArrow( value );
		
		return _referencePlanet = value;
		
	}
	
	private function drawArrow(value:Planet) 
	{
		
		arrow.graphics.clear();
		arrow.graphics.beginFill( value.color );
		arrow.graphics.moveTo( 0, -arrowHeight );
		arrow.graphics.lineTo( arrowWidth, 0 );
		arrow.graphics.lineTo( -arrowWidth, 0 );
		
		arrow.graphics.beginFill( 0, .5 );
		arrow.graphics.moveTo( 0, -arrowHeight );
		arrow.graphics.lineTo( -arrowWidth, 0 );
		arrow.graphics.lineTo( 0, 0 );
		
	}
	
	public var referencePlanet(get_referencePlanet, set_referencePlanet):Planet;
}