package net.nicoptere.planets;
import biga.shapes2d.Arc;
import biga.utils.GeomUtils;
import net.nicoptere.planets.descriptor.PlanetDescription;
import net.nicoptere.planets.Planet;
import nme.display.Graphics;
import nme.display.Sprite;
import nme.filters.GlowFilter;
import nme.geom.Point;

/**
 * @author Nicolas Barradeau
 */

class Planet extends Point
{
	
	private var arc:Arc;
	
	private var _color:Int;
	private var _angle:Float;
	private var _distance:Float;
	private var _radius:Float;
	private var _period:Float;
	private var _satelites:Array<Planet>;
	private var _isSun:Bool;
	private var _name:String;
	private var _parent:Planet;
	private var _sun:Planet;

	private var _canvas:Sprite;
	static private var static_angle:Float = 0;
	
	public function new( description:PlanetDescription ) 
	{
		super();
		
		this.name = description.name;
		
		if ( description.period < .2 ) description.period = .2;
		this.period = description.period * 360.26;
		
		this.radius = Math.max( 1, Math.min( 10, description.radius * .5 ) );
		this.distance = description.distance;// + ( Math.random() - .5 ) * 2;
		
		this.color = description.color;
		
		angle = static_angle;
		static_angle -= Math.PI / 3;
		
		x = y = 0;
		
		arc = new Arc( 0,0,radius, 0, Math.PI - 0.0001 );
		
		satelites = new Array<Planet>();
		
		canvas = new Sprite();
		drawPlanet();
		
	}
	
	private function drawPlanet() 
	{
		
		canvas.graphics.clear();
		canvas.graphics.beginFill( color );
		canvas.graphics.drawCircle( 0, 0, radius );
		canvas.graphics.endFill();
		
		if ( !isSun )
		{
			
			canvas.graphics.beginFill( 0, 0.5 );
			arc.startAngle = Math.PI * .5;
			arc.draw( canvas.graphics );
			canvas.graphics.endFill();
			
		}
		
		for ( s in satelites )
		{
			if ( !isSun )canvas.graphics.lineStyle( 0, 0x333333 );
			else canvas.graphics.lineStyle( 0, 0x202020 );
			canvas.graphics.drawCircle( 0, 0, s.distance );
		}
		
	}
	
	
	
	public function update( t:Float ):Void
	{
		
		if ( parent != null )
		{
			
			angle += ( ( Math.PI * 2 ) / period );
			
			x = canvas.x = parent.canvas.x + Math.cos( angle ) * distance;
			y = canvas.y = parent.canvas.y + Math.sin( angle ) * distance;
			
			canvas.rotation = GeomUtils.angle( this, sun ) * 180 / Math.PI;
			
		}
		else
		{
			canvas.x = canvas.y = 0;
		}
		
		for ( s in satelites ) 
		{
			s.update( t );
		}
		
	}
	
	public function render( g:Graphics ):Void
	{
		
		for ( s in satelites )
		{
			
			s.render( g );
			
		}
		
	}
	
	public function addSatelite( desc:PlanetDescription ):Planet
	{
		
		var p:Planet = new Planet( desc );
		p.parent = this;
		p.sun = sun;
		satelites.push( p );
		drawPlanet();
		return p;
		
	}
	
	/*
	 * GETTERS / SETTERS
	 */
	
	private function get_satelites():Array<Planet> 
	{
		return _satelites;
	}
	
	private function set_satelites(value:Array<Planet>):Array<Planet> 
	{
		return _satelites = value;
	}
	
	public var satelites(get_satelites, set_satelites):Array<Planet>;
	
	private function get_isSun():Bool 
	{
		return _isSun;
	}
	
	private function set_isSun(value:Bool):Bool 
	{
		
		if ( value ) this.sun = this;
		_isSun = value;
		
		drawPlanet();
		return _isSun;
		
	}
	
	public var isSun(get_isSun, set_isSun):Bool;
	
	private function get_name():String 
	{
		return _name;
	}
	
	private function set_name(value:String):String 
	{
		return _name = value;
	}
	
	public var name(get_name, set_name):String;
	
	private function get_period():Float 
	{
		return _period;
	}
	
	private function set_period(value:Float):Float 
	{
		return _period = value;
	}
	
	public var period(get_period, set_period):Float;
	
	private function get_parent():Planet 
	{
		return _parent;
	}
	
	private function set_parent(value:Planet):Planet 
	{
		return _parent = value;
	}
	
	public var parent(get_parent, set_parent):Planet;
	
	private function get_angle():Float 
	{
		return _angle;
	}
	
	private function set_angle(value:Float):Float 
	{
		return _angle = value;
	}
	
	public var angle(get_angle, set_angle):Float;
	
	private function get_distance():Float 
	{
		return _distance;
	}
	
	private function set_distance(value:Float):Float 
	{
		return _distance = value;
	}
	
	public var distance(get_distance, set_distance):Float;
	
	private function get_radius():Float 
	{
		return _radius;
	}
	
	private function set_radius(value:Float):Float 
	{
		return _radius = value;
	}
	
	public var radius(get_radius, set_radius):Float;
	
	private function get_sun():Planet 
	{
		return _sun;
	}
	
	private function set_sun(value:Planet):Planet 
	{
		return _sun = value;
	}
	
	public var sun(get_sun, set_sun):Planet;
	
	private function get_color():Int 
	{
		return _color;
	}
	
	private function set_color(value:Int):Int 
	{
		return _color = value;
	}
	
	public var color(get_color, set_color):Int;
	
	private function get_canvas():Sprite 
	{
		return _canvas;
	}
	
	private function set_canvas(value:Sprite):Sprite 
	{
		return _canvas = value;
	}
	
	public var canvas(get_canvas, set_canvas):Sprite;
	/*
	override public function toString():String
	{
		return ' x ' + x + ' y ' + y + ' p ' + period + ' a ' + angle + ' d ' + distance + ' r ' + radius;
		//return ' r ' + radius;
	}
	//*/
}