package net.nicoptere.dna.items;
import biga.utils.GeomUtils;
import net.nicoptere.dna.DNA;
import net.nicoptere.dna.IDNAHolder;
import nme.display.Graphics;
import nme.geom.Point;
import nme.geom.Rectangle;
import nme.text.TextField;
import nme.text.TextFieldAutoSize;
import nme.Vector;

/**
 * @author Nicolas Barradeau 
 */

class ShapeItem extends DnaHolder
{
	private var rect:Rectangle;
	public var tf:TextField;
	public function new( count:Int, rect:Rectangle ) 
	{
		
		this.rect = rect.clone();
		tf = new TextField();
		tf.textColor = 0xFFFFFF;
		tf.x = rect.x;
		tf.y = rect.y + rect.height;
		tf.width = rect.width;
		tf.autoSize = TextFieldAutoSize.CENTER;
		
		super( count );
		
	}
	
	override public function randomGene():Dynamic 
	{
		
		return new Point( 	Math.round( Math.random() * rect.width	)	,
							Math.round( Math.random() * rect.height )	);
		
		
	}
	
	override public function computeFitness(target:DNA):Float 
	{
		
		//si les 2 ADNs n'ont pas la même taille, on prend le plus court
		var total:Int = Std.int( Math.min( target.length, dna.length ) );
		
		_fitness = 0;
		
		for( i in 0...total )
		{
			var p:Point = cast( dna.getGeneAt( i ), Point );
			
			var o:Point = cast( target.getGeneAt( i ), Point );
			
			_fitness += GeomUtils.distance( p, o );
		}
		
		return _fitness;
		
	}
	
	override public function mutate():Void 
	{
		for ( i in 0...dna.length )
		{
			var p:Point = cast( dna.genes[ i ], Point );
			
			if ( Math.random() < mutationRate )
			{
				dna.setGeneAt( i, randomGene() );
			}
		}
	}
	
	override public function clone():ShapeItem 
	{
		
		var tmp:ShapeItem = new ShapeItem( dna.length, rect );
		tmp.dna.setGenes( dna.genes );
		tmp.mutationRate = this.mutationRate;
		tmp.fitness = this.fitness;
		tmp.tf = this.tf;
		return tmp;
		
	}
	
	override public function express(context:Dynamic):Void 
	{
		
		var g:Graphics = cast( context, Graphics );
		g.lineStyle( 0, 0xFFFFFF ); 
		
		g.moveTo( rect.x + dna.genes[ 0 ].x, rect.y + dna.genes[ 0 ].y );
		
		for ( p in dna.genes )
		{
			g.lineTo( rect.x + p.x, rect.y + p.y );
		}
		
		g.lineTo( rect.x + dna.genes[ 0 ].x, rect.y + dna.genes[ 0 ].y );
		tf.text = '' + Math.round( _fitness * 100 ) / 100;
		
	}
	
}