package net.nicoptere.dna.tests;
import biga.distribution.Circular;
import biga.utils.GeomUtils;
import haxe.Timer;
import net.nicoptere.dna.DNA;
import net.nicoptere.dna.items.ColorItem;
import nme.display.Sprite;
import nme.geom.Point;
import nme.Vector;

/**
 * @author Nicolas Barradeau 
 */

class ColorDna extends Sprite
{

	private var pool:Vector<ColorItem>;
	
	private var target:ColorItem;
	
	private var timer:Timer;
	
	public function new( w:Float, h:Float ) 
	{
		
		super();
		
		var center:Point = new Point( w * .5, h * .5 );
		
		target = new ColorItem( center, 50 );//rond centre 
		target.dna.setGenes( [ 0x00, 0x99, 0xCC ] );//bleu
		
		// X points autour du centre
		var count:Int = 20;
		var positions:Vector<Point> = new Circular( center, 150, 150 ).distribute( count );
		
		pool = new Vector<ColorItem>();
		
		for ( i in 0...count )
		{
			
			var item:ColorItem = new ColorItem( positions[ i ], 10 );
			
			item.mutationRate = 0.02;
			
			pool.push( item );
			
		}
		
		//loop
		timer = new Timer( 1 );
		timer.run = update;
		update();
		
	}
	
	private function update() 
	{
		graphics.clear();
		
		for ( item in pool )
		{
			
			item.computeFitness( target.dna );
			
			//le plus grand delta entre 2 fitness c'est : Math.sqrt( 0xFF * 0xFF + 0xFF * 0xFF + 0xFF * 0xFF )
			// soit ~441
			//on mappe le fitness courant sur 441 pour avoir un ratio de 0 a 1
			
			var t:Float = GeomUtils.map( item.fitness, 0, 441, 1, 0 );
			
			graphics.lineStyle( 0, 0xFFFFFF, t );
			
			//là on tient un champion!
			if ( item.fitness <= 5 )
			{
				graphics.lineStyle( 5, 0xFFFFFF, 1 );
			}
			
			graphics.moveTo( item.position.x, item.position.y );
			graphics.lineTo( target.position.x, target.position.y );
			item.express( graphics );
		}
		
		target.express( graphics );
		
		
		//selection naturelle
		pool.sort( fitnessSort );
		
		//accouplement
		for ( i in 0...pool.length-1 )
		{
			pool[ i ].cross( pool[ i + 1 ].dna );
			pool[ i ].mutate();
		}
		
	}
	
	private function fitnessSort( a:ColorItem, b:ColorItem ) :Int
	{
		
		return a.fitness > b.fitness ? -1 : 1;
		
	}
	
	
	
}