package net.nicoptere.dna.tests;
import biga.distribution.Circular;
import biga.distribution.Star;
import biga.utils.GeomUtils;
import haxe.Timer;
import net.nicoptere.dna.DNA;
import net.nicoptere.dna.IDNAHolder;
import net.nicoptere.dna.items.ShapeItem;
import nme.display.Sprite;
import nme.geom.Point;
import nme.geom.Rectangle;
import nme.Vector;

/**
 * @author Nicolas Barradeau 
 */

class ShapeDna extends Sprite
{

	private var pool:Vector<ShapeItem>;
	
	private var target:ShapeItem;
	
	private var timer:Timer;
	
	public function new( w:Float, h:Float ) 
	{
		
		super();
		
		
		var rect:Rectangle = new Rectangle( w * .5 - 50, 40, 100, 100 );
		target = new ShapeItem( 0, rect );
		
		var positions:Vector<Point> = new Star( new Point( 50, 50 ), 20, 40 ).distribute( 5 );
		target.dna.setGenes([]);
		for( p in positions ) target.dna.addGene( p );
		
		var count:Int = positions.length;
		pool = new Vector<ShapeItem>();
		for ( i in 0...8 )
		{
			for ( j in 0...2 )
			{
				
				rect.x = i * rect.width;
				rect.y = 180 + j * ( rect.height + 20);
				
				var item:ShapeItem = new ShapeItem( count, rect );
				
				item.mutationRate = 0.01;
				
				pool.push( item );
				
				addChild( item.tf );
				
			}
		}
		
		//loop
		timer = new Timer( 1 );
		timer.run = update;
		
	}
	
	private function update() 
	{
		graphics.clear();
		
		graphics.beginFill( 0x0099CC );
		target.express( graphics );
		graphics.endFill();
		
		for ( item in pool )
		{
			
			item.computeFitness( target.dna );
			
			graphics.beginFill( 0xFFFFFF, GeomUtils.normalize( Math.min( item.fitness, 100 ), 100, 20 ) );
			if ( item.fitness < 20 ) 	graphics.beginFill( 0xFF0000, GeomUtils.normalize( item.fitness, 20, 10 ) );
			if ( item.fitness < 10 ) 	graphics.beginFill( 0x0099CC, GeomUtils.normalize( item.fitness, 10, 5 ) );
			
			item.express( graphics );
			
		}
		
		pool.sort( fitnessSort );
		
		for ( i in 0...pool.length - 1 )
		{
			///*
			//mouais
			pool[ i ].cross( pool[ i + 1 ].dna );
			pool[ i ].mutate();
			//*/
			
			/*
			//mieux
			var tmp:ShapeItem = pool[ i ].clone();
			
			tmp.cross( pool[ i + 1 ].dna );
			
			tmp.mutationRate = Math.random() > .5 ? .1 : .05;
			
			tmp.mutate();
			
			var lastFitness:Float = pool[ i ].fitness;
			
			if ( tmp.computeFitness( target.dna ) < lastFitness )
			{
				
				tmp.mutationRate = pool[ i ].mutationRate;
				pool[ i ] = tmp;
				
			}
			/*
			else
			{
				graphics.beginFill( 0x00FF00, .25 );
				tmp.express( graphics );
			}
			
			//*/
		}
		
	}
	
	private function fitnessSort( a:ShapeItem, b:ShapeItem ) :Int
	{
		
		return a.fitness > b.fitness ? -1 : 1;
		
	}
	
	
	
}